export default odaField ({
    _input: 'bind-classes'
})

import '../../../web/core/class/lib/field-tree/field-tree.html';

FIELD({
    is: 'string',
    imports: [
        '@oda/button',
        '@oda/checkbox'
    ],
    extends: 'field-default',
    template: /*html*/`
        <style>
            :host, :host>div>div {
                @apply --flex;
                @apply --vertical;
            }
            :host>div {
                @apply --flex;
                @apply --horizontal;
            }
            input {
                border: none;
                background-color: transparent;
                overflow: hidden;
                text-overflow: ellipsis;
                width: 100%;
                padding: 4px;
                margin-left: 2px;
                margin-right: 2px;
            }
        </style>
        <input v-if="!$object" type="text" :value="value" readonly disabled>
        <div v-if="$object">
            <div>
                <oda-bind-class-link :context-item.prop="toClass" @context-item-changed="_dstClsChanged">
                </oda-bind-class-link>
                <oda-bind-fields-tree ref="bindTree" :context-item.prop="toClass" :data-root.prop="fieldRoot"
                    :$object.prop="$object">
                </oda-bind-fields-tree>
            </div>
            <div style="max-width: 32px">
                <oda-button class="flex" style="max-width: 24px" icon="hardware:keyboard-arrow-left" title="Auto fill"
                    @tap="_fill"></oda-button>
                <oda-button size="24" icon="icons:clear" title="Clear" @tap="_clear"></oda-button>
            </div>
            <div>
                <oda-bind-class-link :context-item.prop="fromClass" @context-item-changed="_srcClsChanged">
                </oda-bind-class-link>
                <oda-source-field-tree :context-item.prop="fromClass" @data-set-changed="_fieldsChanged" @_on-drag-start>
                </oda-source-field-tree>
            </div>
        </div>
        <div v-if="$object">
            <oda-checkbox label="Относительный адрес" :value.prop="relative" @value-changed="_relativeChanged"></oda-checkbox>
        </div>
    `,
})

ODA({
    is: 'oda-source-field-tree',
    extends: 'oda-field-tree',
    _onDragStart(e) {
        const el = e.path.find(p => p.row);
        if (el && (this.allowDrag || el.row.drag)) {
            e.dataTransfer.setDragImage(new Image(), 0, 0);

            let data;
            try {
                data = JSON.stringify(el.row.$item);
            } catch (err) {
                data = JSON.stringify(el.row.$item, prop => {
                    return prop instanceof Object ? undefined : prop;
                });
            };

            e.dataTransfer.setData('text/plain', data);
        }
    }
})

ODA({
    is: 'oda-bind-class-link',
    template: /*html*/`
        <style>
            :host {
                @apply --no-flex;
                @apply --horizontal;

                align-items: center;
            }
        </style>
        <oda-icon size="24" :icon.prop="contextItem?.icon"></oda-icon>
        <span v-text="contextItem?.displayLabel" @tap="_selectClass"></span>
    `,
    props: {
        contextItem: Object,
    },
    async _selectClass() {
        const component = await ODA.createComponent('/web/core/structure/lib/select-class/select-class.html',
            { contextItem: this.contextItem ? this.contextItem.$base : CORE.$host.findItem('P:ROOT') });
        const result = await ODA.showDialog(component, { title: 'Select destination Class', icon: 'odant:class' });
        if (result && result.item) {
            this.contextItem = result.item;
            return true;
        }

        return false;
    }
})

ODA({
    is: 'oda-bind-fields-tree',
    extends: 'oda-item-tree',
    template: /*html*/`
        <style>
            :host {
                @apply --flex;
                @apply --raised;
            }
            input {
                border: none;
                background-color: transparent;
                overflow: hidden;
                text-overflow: ellipsis;
                width: 100%;
                padding: 4px;
                margin-left: 2px;
                margin-right: 2px;
            }
        </style>
        <input class="flex" :value="item?.root?.src" @change="item.root.src = $event.target.value"
            @drop="_drop" @dragOver="_dragOver">
    `,
    props: {
        allowFocus: true,
        allowDrop: 'extended',
        autoWidth: true,
        showHeader: true,
        colLines: true,
        rowLines: true,
        $object: {
            type: Object,
            watch(n) {
                if (n) {
                    this.listen(n, 'changed', (e) => this.fire('update'));
                    this.listen(n, 'update', (e) => this.fire('update'));
                }
            }
        },
        dataRoot: {
            type: Object,
        },
        columns: {
            type: Array,
            default: [
                { name: 'path', label: 'Путь', treeMode: true },
                { name: 'src', label: 'Из поля', template: 'oda-bind-expression-cell' }
            ]
        },
        contextItem: {
            type: Object,
        },
        itemSourceNames: 'fields',
        defaultTemplate: 'oda-field-tree-cell',
        defaultGroupTemplate: 'oda-field-tree-group'
    },
    observers: [
        async function _setDataSet(contextItem) {
            if (!contextItem)
                return;

            this.debounce('_setDataSet', async () => {
                this.dataSet = await this._getFields(contextItem).catch(e => []);
                this.expandAll();
            }, 10);
        }
    ],
    _onDragOver(e) {
        e.stopPropagation();
        e.preventDefault();
    },
    _clear(root = this.dataRoot) {
        if (root && root.$LINK && (root.$LINK.length > 0)) {
            root.$LINK.forEach((e) => {
                e.src = '';
                this._clear(e);
            });
        }
        
        this.fire('update');
    },
    async _fill(fields, root = this.dataRoot) {
        await fields.forEach(async (f) => {
            if (root.$LINK && (root.$LINK.length > 0)) {
                const rootIdx = root.$LINK.findIndex((e) => this._fuzzyCompare(this._clearPath(e.path), f.name));
                if (~rootIdx) {
                    const fldRoot = root.$LINK[rootIdx];
                    const fldLst = await f.fields;
                    if (fldLst && (fldLst.length > 0) && !f.isLink) {
                        fldRoot.src = f.name;
                        await this._fill(fldLst, fldRoot);
                    } else {
                        fldRoot.src = `@${f.name}`;
                    }
                }
            }
        });

        this.fire('update');
    },
    _clearPath(path) {
        const idx = path.lastIndexOf(path.includes('@') ? '@' : '/');
        return (~idx) ? path.substr(idx + 1) : path;
    },
    _fuzzyCompare(val1, val2) {
        return val1.toLowerCase() === val2.toLowerCase();
    },
    // From fat cleint))
    _getCSharpPath(path) {
        let output = path.replace('[FIELDS]/', '')
            .replace('[STATIC]/', 'STATIC/')
            .replace('[DETAILS]/', 'DETAILS/');
        const idx = output.lastIndexOf('/');
        output = (~idx)
            ? `${output.slice(0, idx + 1)}@${output.slice(idx + 1)}`
            : '@' + output;
        return output;
    },
    _getLinkRoot(dataRoot, path) {
        if (!dataRoot.$LINK) {
            dataRoot.$LINK = [];
        }

        let linkRoot;
        const idx = dataRoot.$LINK.findIndex((f) => f.path === path);
        if (~idx) {
            linkRoot = dataRoot.$LINK[idx];
        } else {
            linkRoot = { path: path };
            dataRoot.$LINK.push(linkRoot);
        }

        return linkRoot;
    },
    async _getFields(source) {
        const ds = await Promise.all((await source.fields).map(async (i) => {
            try {
                const fields = (await i.fields || []).filter((i, n) => !n).map(() => {
                    return { template: 'oda-table-loading' };
                });

                const path = this._getCSharpPath(i.path);
                const linkRoot = this._getLinkRoot(this.dataRoot, path);

                return { $item: i, items: fields, group: i.$group.name, root: linkRoot };
            } catch (err) {
                console.error(err);
            }
            return { $item: i, items: [], group: i.$group.name };
        }));

        if (ds?.length) return ds;
        
        return [];
    },
    async _beforeExpand(node, deep = 1) {
        let res = this.itemSourceNames.split(',').map(async src => {
            src = src.trim();
            return node.$item && (await node.$item[src] || []);
        });

        res = (await Promise.all(res)).flat();
        res = res.map(async i => {

            const path = this._getCSharpPath(i.path)
            const linkRoot = this._getLinkRoot(node.root, path)

            const obj = {};
            this.set(obj, '$item', i);
            this.set(obj, 'items', []);
            this.set(obj, 'root', linkRoot);
            // const obj = { $item: i, items: [], root: linkRoot };
            if (deep) {
                const ch = await this._checkChildren(obj);
                if (ch) obj.items = [{}]
            }

            return obj;
        });

        res = await Promise.all(res);
        return res;
    },
})

ODA({
    is: 'oda-bind-expression-cell',
    extends: 'oda-table-cell-base',
    props: {
        item: Object,
    },
    attached() {
        if (this.table) {
            this.listen(this.table, 'update', (e) => this.debounce('updateCell', (o) => this.render(), 100));
        }
    },
    _drop(e) {
        e.preventDefault();
        e.stopPropagation();

        if (e?.dataTransfer?.items) {
            const data = e.dataTransfer.items;
            for (let i = 0; i < data.length; i++) {
                if ((data[i].kind === 'string') && (data[i].type.startsWith('text'))) {
                    data[i].getAsString((s) => {
                        const fld = JSON.parse(s);
                        try {
                            this.item.root.src = fld.isTable ? fld.name : `@${fld.name}`;
                            this.render();
                        } catch(e) { }
                    });
                    break;
                }
            }
        }
    },
    _dragOver(e) {
        e.preventDefault();
        e.dataTransfer.dropEffect = 'move';
    },
})