(class extends odaService {

    static get properties() {
        return {
            allowAcces: 'R',
            allowUse: true,
            isActive: true,
            label() {
                return 'Filling';
            },
            subMenu: async function () {
                if (!this.$owner || !this.$owner.Root
                    || !this.$owner.Root.$SERVICES || this.$owner.Root.$SERVICES.length <= 0
                    || !this.$owner.Root.$SERVICES[0].$BIND || this.$owner.Root.$SERVICES[0].$BIND.length <= 0
                    || !this.$owner.Root.$SERVICES[0].$BIND[0].$DOC || this.$owner.Root.$SERVICES[0].$BIND[0].$DOC.length <= 0)
                    return [{ 'label': 'Settings', 'icon': 'icons:settings', 'execute': (e) => this.execute({ settings: true }) }];

                const promArr = this.$owner.Root.$SERVICES[0].$BIND[0].$DOC.map(async (doc) => {
                    try {
                        const obj = await this.$owner.findItem(doc.$link[0].link);
                        return { 'label': obj.name, 'execute': (e) => obj.show() };
                    } catch (e) {
                        return undefined;
                    }
                });
                const objArr = await Promise.all(promArr);
                objArr.push({ 'label': 'Settings', 'icon': 'icons:settings', 'execute': (e) => this.execute({ settings: true }) })
                return objArr.filter(e => e);
            }
        }
    }

    async execute(params) {
        if (!params || !params.settings) {
            const SOURCE_CLASS_ID = 'M:1CD85C475AC7F6F/C:143574104309355';
            const idsStr = await this.getAllSourceClassesIds(this.$owner.$class);
            const getTemplatesOidsQuery = `PACK/OBJECT[contains("${idsStr}", Bind[1]/oda:right(@SrcClass, 15))]/string(@oid)`;

            const srcCls = await this.$owner.$base.findItem(SOURCE_CLASS_ID);
            const index = await srcCls.getIndex("Pack");
            
            const oids = await index.XQuery(getTemplatesOidsQuery);
            if (oids && oids.length === 1) {
                const item = await srcCls.getObject(oids[0]);
                return item.execute('run-fill', { srcObj: this.$owner.fullId });
            }
        }

        const url = this.resolveUrl('template-selector/fill-template-selector.html');
        const selector = await ODA.createComponent(url);
        selector.contextItem = this.$owner;

        const dialog = {
            title: 'Select template',
            icon: 'image:center-focus-strong',
            dblclick: true,
            ok: {
                show: true,
                disable: false
            },
            buttons: [
                { label: 'New', icon: 'icons:add', special: 'add' },
                { label: 'Remove', icon: 'icons:remove', special: 'del' },
                { label: 'Settings', icon: 'icons:settings', special: 'set' },
                { label: 'OK', special: 'ok' }
            ]
        };
        const dialogResult = await ODA.showDialog(selector, dialog);
        if (!dialogResult.button) {
            if (!dialogResult.focusedItem || !dialogResult.focusedItem.item)
                return;

            const item = dialogResult.focusedItem.item;
            return item.execute('run-fill', { srcObj: this.$owner.fullId });
        } else {
            switch (dialogResult.button.special) {
                case 'add':
                    const fillCls = await this.$owner.$base.findItem(SOURCE_CLASS_ID);
                    const newTmpObj = await fillCls.createObject();

                    if (newTmpObj.Root) {
                        const component = await ODA.createComponent('/web/lib/dialogs/select-class/select-class.html',
                            { contextItem: this.$owner.$base });
                        const result = await ODA.showDialog(component, { title: 'Select destination Class', icon: 'odant:class' });
                        if (result && result.item) {
                            newTmpObj.Root.Bind = `${this.$owner.$class.label} -> ${result.item.label}`;
                            newTmpObj.Root.$Bind = [];
                            const bind = {
                                SrcClass: fillCls.getRelativeId(this.$owner.$class),
                                DstClass: fillCls.getRelativeId(result.item),
                                Relative: true
                            };
                            newTmpObj.Root.$Bind.push(bind);

                            newTmpObj.show();
                        }
                    }
                    break;
                case 'del':
                    if (dialogResult.control.focusedItem && dialogResult.control.focusedItem.item) {
                        if (await ODA.confirm('Are you sure?')) {
                            dialogResult.control.focusedItem.item.remove();
                        }
                    }
                    break;
                case 'set':
                    if (dialogResult.control.focusedItem && dialogResult.control.focusedItem.item) {
                        dialogResult.control.focusedItem.item.show();
                    }
                    break;
                case 'ok':
                    if (!dialogResult.control.focusedItem || !dialogResult.control.focusedItem.item)
                        return;

                    const item = dialogResult.control.focusedItem.item;
                    return item.execute('run-fill', { srcObj: this.$owner.fullId });
                default:
                    alert('Я таких значений не знаю');
            }
        }
    }
    getAllSourceClassesIds(i = this.contextItem) {
        let idsStr = i.id;
        let c = i.$owner;
        while (c && c.prefix.toLowerCase() === "c") {
            idsStr += "|" + c.id;
            c = c.$owner;
        }

        return idsStr;
    }
}).register();