export default odaService({
    props: {
        allowAcces: 'R',
        allowUse: true,
        isActive: true,
        label: 'Filling',
        subMenu: async function () {
            const docs = this.$owner?.root?.$SERVICES?.[0]?.$BIND?.[0]?.$DOC;
            if (!docs || (docs.length <= 0))
                return [{ 'label': 'Settings', 'icon': 'icons:settings', 'execute': (e) => this.execute({ settings: true }) }];

            const promArr = docs.map(async (doc) => {
                try {
                    const obj = await this.$owner.findItem(doc.$link[0].link);
                    return { 'label': obj.name, 'execute': (e) => obj.show() };
                } catch (e) {
                    return undefined;
                }
            });
            const objArr = await Promise.all(promArr);
            objArr.push({ 'label': 'Settings', 'icon': 'icons:settings', 'execute': (e) => this.execute({ settings: true }) })
            return objArr.filter(e => e);
        }
    },
    async execute(params) {
        if (!params?.settings) {
            const SOURCE_CLASS_ID = 'M:1CD85C475AC7F6F/C:143574104309355';
            const idsStr = await this.getAllSourceClassesIds(this.$owner.$class);
            const getTemplatesOidsQuery = `PACK/OBJECT[contains("${idsStr}", Bind[1]/oda:right(@SrcClass, 15))]/string(@oid)`;

            const srcCls = await this.$owner.$base.findItem(SOURCE_CLASS_ID);
            const index = await srcCls.getIndex('Pack');
            
            const oids = await index.XQuery(getTemplatesOidsQuery);
            if (oids && (oids.length === 1)) {
                const item = await srcCls.getObject(oids[0]);
                return item.execute('run-fill', { srcObj: this.$owner.fullId });
            }
        }

        // TODO: show dialog
        const dialog = {
            title: 'Select template',
            icon: 'image:center-focus-strong',
            dblclick: true,
            ok: {
                show: true,
                disable: false
            },
            buttons: [
                { label: 'New', icon: 'icons:add', special: 'add' },
                { label: 'Remove', icon: 'icons:remove', special: 'del' },
                { label: 'Settings', icon: 'icons:settings', special: 'set' },
                { label: 'OK', special: 'ok' }
            ]
        };
        const dialogResult = await ODA.showDialog('./template-selector/fill-template-selector.js',
            { contextItem: this.$owner},
            dialog);
        if (!dialogResult.button) {
            if (!dialogResult?.focusedItem?.item)
                return;

            const item = dialogResult.focusedItem.item;
            return item.execute('run-fill', { srcObj: this.$owner.fullId });
        } else {
            const item = dialogResult.control?.focusedItem?.item;
            switch (dialogResult.button.special) {
                case 'add':
                    const fillCls = await this.$owner.$base.findItem(SOURCE_CLASS_ID);
                    const newTmpObj = await fillCls.createObject();

                    if (newTmpObj.root) {
                        // TODO: show dialog
                        const result = await ODA.showDialog('../../../../../web/lib/dialogs/select-class/select-class.html',
                            { contextItem: this.$owner.$base },
                            { title: 'Select destination Class', icon: 'odant:class' });
                        if (result?.item) {
                            newTmpObj.root.Bind = `${this.$owner.$class.label} -> ${result.item.label}`;
                            newTmpObj.root.$Bind = [];
                            const bind = {
                                SrcClass: fillCls.getRelativeId(this.$owner.$class),
                                DstClass: fillCls.getRelativeId(result.item),
                                Relative: true
                            };
                            newTmpObj.root.$Bind.push(bind);

                            // TODO: show object
                            newTmpObj.show();
                        }
                    }
                    break;
                case 'del':
                    if (item && await ODA.confirm('Are you sure?')) item.remove();
                    break;
                case 'set':
                    // TODO: show object
                    item?.show();
                    break;
                case 'ok':
                    item?.execute('run-fill', { srcObj: this.$owner.fullId });
                    break;
                default:
                    alert('Я таких значений не знаю');
            }
        }
    },
    getAllSourceClassesIds(i = this.contextItem) {
        let idsStr = i.id;
        let c = i.$owner;
        while (c && c.prefix.toLowerCase() === 'c') {
            idsStr += "|" + c.id;
            c = c.$owner;
        }

        return idsStr;
    }
})