export default odaField ({
    is:'odaDateTimeField',
    props: {
        _input: 'date-time',
        format: {
            type: String,
            get() {
                return this.getAttr('Format');
            },
            set(v) {
                this.setAttr('Format', v);
            },
            enum: [
                'date',
                'full',
                'fulltime',
                'short',
                'month',
                'time',
                'D',
                'm',
                's',
                'S',
                'ShortDate',
                'FullDateTimeMs',
                'GeneralDateTime',
                'LongDate',
                'ShortTime',
                'LongTime',
                'FullDateTime',
                'MonthDay',
                'YearMonth',
                'MonthYear',
                'SortableDateTime',
                'UniversalDateTime',
                'isoDate',
                'isoTime',
                'isoDateTime',
                'isoUtcDateTime',
                'dddd, dd mmmm yyyy г.',
                'dd.mm.yyyy hh:MM:ss.lll',
                'dd.mm.yyyy hh:MM:ss',
                'dd.mm.yyyy hh:MM',
                'yyyy-mm-dd hh:MM:ss',
                'dd.mm.yyyy',
                'dd / mm / yyyy',
                'yyyy-mm-dd',
                'mm.yyyy',
                'hh:MM:ss',
                'hh:MM'
            ]
        },
        mask: {
            category: 'DateTime',
            type: String,
            get() {
                return this.getAttr('Mask');
            },
            set(v) {
                this.setAttr('Mask', v);
            },
            enum: [
                'dd.mm.yyyy hh:MM:ss.lll',
                'dd.mm.yyyy hh:MM:ss',
                'dd.mm.yyyy hh:MM',
                'yyyy-mm-dd hh:MM:ss',
                'dd.mm.yyyy',
                'dd / mm / yyyy',
                'yyyy-mm-dd',
                'mm.yyyy',
                'hh:MM:ss',
                'hh:MM'
            ]
        },
        locale: {
            type: String,
            get() {
                return this.getAttr('Locale');
            },
            set(v) {
                this.setAttr('Locale', v);
            },
            enum: ['ru-RU', 'en-US', 'ua-UA', 'ar-EG', 'zh-Hans-CN-u-nu-hanidec', 'th-TH-u-nu-thai'],
        },
        testFormat: {
            label: 'Тест',
            type: Date,
            get() {
                let date = new Date();
                return date.getFormattedValue(this.getArgs());
            }
        },
        _labelTop: true
    },

    getArgs() {
        return {
            format: this.Format,
            mask: this.Mask,
            locale: this.Locale,
            placeholder: this.placeholder,
            title: this.description
        };
    },

    getValue($object, root) {
        const value = this.$super.getValue.call(this, $object, root);
        if (value && value.then){
            return value.then(value=>{
                return value && new Date(Date.parse(value)) || undefined;
            })
        }
        return value && new Date(Date.parse(value)) || undefined;
    },

    getFormattedValue($object, root, format) {
        format = format || this.getArgs();
        const value = this.getValue($object, root);
        if (value && value.then){
            return value.then(value =>{
                return value ? value.getFormattedValue(format) : value;
            })
        }
        return value ? value.getFormattedValue(format) : value;
    },

    async setValue($object, value, root) {
        if ($object) {
            this.$super.setValue.call(this, $object, (value && value.toISOString()) || null, root);
        }
    }
})
FIELD({
    is: 'date-time',
    template:`
        <input type="date" ::value>
    `,
    props: {
        value: Date,
    }
})