export default odaField ({
    is:'odaNumericField',
    props:{
        _input: 'number',
        _hasList() {
            return false;
        },
        format: {
            category: 'Number properties',
            type: String,
            get() {
                return this.getAttr('Format');
            },
            set(v) {
                this.setAttr('Format', v);
            },
            enum: ['integer', 'decimal', 'percent', 'currency', 'currency-eur', 'currency-usd', 'bank', 'exponential', 'text']
        },
        showCalculator: {
            category: 'Number properties',
            type: Boolean,
            get() {
                return this.getBoolAttr('showCalculator', this.getBoolAttr('ShowCalc'));
            },
            set(v) {
                this.setBoolAttr('showCalculator', v);
            },
        },
        places: {
            category: 'Number properties',
            type: Number,
            get: function () {
                return this.getNumAttr('Places');
            },
            set: function (v) {
                this.setNumAttr('Places', v);
            }
        },
        locale: {
            category: 'Number properties',
            type: String,
            get: function () {
                return this.getAttr('Locale');
            },
            set: function (v) {
                this.setAttr('Locale', v);
            },
            enum: ['ru-RU', 'en-US', 'ua-UA', 'ar-EG', 'zh-Hans-CN-u-nu-hanidec', 'th-TH-u-nu-thai']
        },
        showZero: {
            type: Boolean,
            get: function () {
                return this.getBoolAttr('showZero');
            },
            set: function (v) {
                this.setBoolAttr('showZero', v);
            }
        },
        min: {
            type: Number,
            get: function () {
                return this.getNumAttr('Min');
            },
            set: function (v) {
                this.setNumAttr('Min', v);
            }
        },
        max: {
            category: 'Number properties',
            label: 'Max - Наибольшее допустимое значение',
            type: Number,
            get: function () {
                return this.getNumAttr('Max');
            },
            set: function (v) {
                this.setNumAttr('Max', v);
            }
        },
        useMinMax: {
            category: 'Number properties',
            label: 'useMinMax - Использовать min/max допустимые значения',
            type: Boolean,
            get: function () {
                return this.getBoolAttr('useMinMax');
            },
            set: function (v) {
                this.setBoolAttr('useMinMax', v);
            }
        },
        getArgs() {
            return {
                format: this.Format,
                places: this.Places,
                locale: this.Locale,
                showZero: this.showZero,
                min: this.Min,
                max: this.Max,
                useMinMax: this.useMinMax
            };
        },
    },

    // testFormat: {
    //     label: 'Тест',
    //     type: Number,
    //     get: function () {
    //         let numb = 12345.12345;
    //         return numb.getFormattedValue(this.getArgs());
    //     }
    // },




    async getValidate($object, root) {
        let invalidateText = await this.$super.getValidate.call(this, $object, root);
        const { useMinMax, Min: min, Max: max } = this;
        const val = await this.getValue($object, root) || 0;
        if (!invalidateText && useMinMax) {
            try {
                if (isNaN(min)) throw `Invalid min limit in the '${ this.label || this.name } ' field"`;
                if (isNaN(max)) throw `Invalid max limit in the '${ this.label || this.name } ' field"`;
                if (val < min) throw `The value of the '${this.label || this.name}' field cannot be less than '${min}'`;
                if (val > max) throw `The value of the '${this.label || this.name}' field cannot be greater than '${max}'`;
            } catch (err) {
                invalidateText = err;
            }
        }
        return invalidateText;
    },

    getValue($object, root) {
        const value = this.$super.getValue.call(this, $object, root);
        if (value && value.then) {
            return value.then(value => {
                return isNaN(value) ? 0 : Number(value) || undefined;
            })
        }
        return isNaN(value) ? 0 : Number(value) || undefined;
    },

    // getFormattedValue($object, root, format) {
    //     return this.getValue($object, root);
    //     // format = format || this.getArgs();
    //     // const value = this.getValue($object, root);
    //     // if (value && value.then) {
    //     //     return value.then(value => {
    //     //         return value ? value.getFormattedValue(format) : value;
    //     //     })
    //     // }
    //     // return value ? value.getFormattedValue(format) : value;
    // },

    async setValue($object, value, root) {
        if ($object) {
            this.$super.setValue.call(this, $object, !value || isNaN(value) ? null : value, root);
        }
    },

    _sumCalc(items) {
        return items.map(i => i[this.name]).reduce((a, b) => {
            a += (parseFloat(b) || 0);
            return a;
        }, 0);
    },
    _maxCalc(items) {
        return Math.max(...items.map(d => parseFloat(d[this.name]) || 0));
    },
    _minCalc(items) {
        return Math.min(...items.map(d => parseFloat(d[this.name]) || 0));
    },
    _avgCalc(items) {
        return this._sumCalc(items) / this._countCalc(items);
    }
})
FIELD({
    is: 'number',
    template: `
        <input type="number" ::value>
    `,
    props: {
        value: Number,
        showCalculator: {
            type: Boolean,
            get() {
                return this.$field?.showCalculator;
            }
        }
    }
})