/*
 * brain.js v1.0
 * (c) 2021 R.A. Perepelkin
 * Under the MIT License.
 */

import '/web/oda/components/layouts/app-layout/app-layout.js';
import '/web/oda/tools/property-grid/property-grid.js';
import './space/space.js'
import NeuroNet from './neuronet.js';

ODA({
    is: 'oda-brain', extends: 'oda-app-layout',
    template: `
        <div slot="title" class="horizontal flex between" style="align-items:center;">
            <div class="no-flex horizontal">
                <oda-button allow-toggle :toggled="!net?.stop" icon="av:play-arrow" @tap="net.stop=!net.stop;net.step()"></oda-button>
                <span style="align-self: center">|</span>
                <oda-button :disabled="!net?.stop" @tap="net.step()" icon="av:skip-next"></oda-button>
            </div>
            <div class="flex horizontal center">
                <oda-button class="border modulation" icon="space-sci-fi:space-ship" @tap="net.modulation = 100" ~style="{background:'linear-gradient(to top, #f00 '+net?.modulation+'%, transparent 0%);'}"></oda-button>
            </div>
            <div class="no-flex horizontal">
                <oda-button allow-toggle ::toggled="stats" icon="icons:info-outline"></oda-button>
                <oda-button allow-toggle ::toggled="charts" icon="office-set:chart"></oda-button>
                <oda-button allow-toggle ::toggled="rotate" icon="icons:3d-rotation"></oda-button>
                <oda-button allow-toggle :toggled="net?.locked" icon="icons:lock" @tap="net.locked=!net.locked"></oda-button>
            </div>
        </div>
        <div slot="left-drawer" style="padding: 4px">
            <div style="font-size:small;text-align:center">SENSORS</div>
            <div ~for="k in Object.keys(keys)">
                <div ~if="keys[k].sensors && keys[k].value" style="font-size:small;position:absolute;margin: 2px 0 0 2px;z-index:1">{{keys[k].value}}</div>
                <oda-button ~if="keys[k].sensors" ~style="{backgroundColor:keys[k].pressed?'silver':''}" style="border: 1px solid black; border-radius: 5px; margin: 4px 0px; font-size: xx-large; font-weight: bolder;" @tap="signal(keys[k])">{{keys[k].label}}</oda-button>
            </div>
            <oda-button ~if="Object.keys(keys).length" style="border: 1px solid black; border-radius: 5px; margin: 4px 0px; font-size: xx-large; font-weight: bolder;" 
                @tap="Object.keys(keys).forEach(i=>{if(keys[i].fnUp)keys[i].fnUp()})">All</oda-button>
        </div>
        <oda-neural-space show-grid slot="main" :net ::focused :stats :charts :rotate></oda-neural-space>
        <oda-property-grid slot="right-drawer" :component="focused"></oda-property-grid>
    `,
    props: {
        net: Object,
        data: {
            set(n, o) {
                if (n) {
                    this.net = new NeuroNet(n);
                    (this.net.layers[0].neurons || []).forEach(n => {
                        const arr = this.keys[n.props.code]?.neurons || [];
                        this.keys[n.props.code] = {
                            sensors: true, label: n.props.label, value: 0, pressed: false, neurons: [...arr, n],
                            fn: (obj = this.keys[n.props.code]) => obj.value++,
                            fnUp: (obj = this.keys[n.props.code]) => this.signal(obj),
                        };
                    });
                }
            }
        },
        sets: {
            set(n, o) {
                if (n)
                    Object.keys(n).forEach(k => n[k] = this.sets[k]);
                // this.net = new NeuroNet(n);
            }
        },
        keys: {},
        iconSize: 32,
        focused: Object,
        stats: false,
        charts: false,
        rotate: false
    },
    attached() {
        window.addEventListener('keydown', e => {
            //console.log(e.code);
            if (this.keys[e.code]) {
                this.keys[e.code] = this.keys[e.code] || {};
                this.keys[e.code].pressed = true;
                this.press();
            }
        }, true);
        window.addEventListener('keyup', e => {
            if (this.keys[e.code]) {
                this.keys[e.code].pressed = false;
                if (this.keys[e.code].fnUp) this.keys[e.code].fnUp();
            }
        }, true);
        this.render();
    },
    press() {
        if (this.isPressed) return;
        Object.keys(this.keys).forEach(k => {
            if (this.keys[k].pressed)
                if (this.keys[k].fn) {
                    this.keys[k].fn();
                    this.isPressed = true;
                }
        });
        if (!this.isPressed) return;
        requestAnimationFrame(() => {
            this.isPressed = false;
            this.press();
        });
    },
    signal(obj) {
        if (!obj) return;
        (obj.neurons || []).forEach(n => n.signal(obj.value || 1));
        obj.value = 0;
    }
})
