import '../space/space.js';
import '/web/oda/components/layouts/app-layout/app-layout.js';
import '../scheme-designer/scheme-designer.js';
import '/web/oda/components/grids/tree/tree.js';
import '/web/oda/tools/property-grid/property-grid.js';
import '/web/oda/tools/property-grid/test/new/property-grid.js';
import NeuralNetwork from './network.js';
import * as LAYERS from "./layers.js";

ODA({
    is: 'oda-neural-framework',
    extends: 'oda-app-layout',
    template: `
        <oda-neural-layers-tree  bar-autofocus slot="left-drawer" ></oda-neural-layers-tree>
        <oda-property-grid slot="right-drawer" :io="focusedNode?.item || net" show-title></oda-property-grid>
        <div ~for="i in _grids2" :is=i.is :slot="i.slot" :title="i.title" :icon="i.icon" :io="i.io"></div>
        <oda-scheme-designer ~show="mainView==='2d'" @dragover="dragover" @drop="drop" slot="main" ref="mainView2d"></oda-scheme-designer>
        <oda-neural-space ~show="mainView==='3d'" @dragover="dragover" @drop="drop" slot="main" ref="mainView3d"></oda-neural-space>
        <oda-button slot="left-drawer" down icon="icons:file-download" @tap="download()">DOWNLOAD</oda-button>
        `,
    props: {
        focusedNode: Object,
        stat: String,
        iteration: 0,
        loss: 0.01,
        net: {
            type: Object,
            set(n, o) {
                this.focused = n;
            },
            shared: true
        },
        stop: true,
        data: {
            set(n, o) {
                if (o)
                    delete this.net;
                if (n)
                    this.net = new NeuralNetwork(n);
            }
        },
        mainView: {
            type: String,
            list: ['2d', '3d'],
            default: '2d',
            set() { this.render() }
        },
        _grids2() {
            return [
                { is: 'oda-property-grid2', slot: 'right-drawer', title: 'NeuralFramework', icon: 'icons:extension', io: this, mainView: '2d3d' },
                { is: 'oda-property-grid2', slot: 'right-drawer', title: 'FocusedItem', icon: 'image:filter-center-focus', io: this.focusedNode?.item || this.net, mainView: '2d3d' },
                { is: 'oda-property-grid2', slot: 'right-drawer', title: 'SchemeDesigner', icon: 'icons:content-copy', io: this.$refs?.mainView2d, mainView: '2d' },
                { is: 'oda-property-grid2', slot: 'right-drawer', title: 'NeuralSpace', icon: 'icons:3d-rotation', io: this.$refs?.mainView3d, mainView: '3d' },
            ].filter(i => i.mainView.includes(this.mainView));
        }
    },
    ready() {
        if (!this.net)
            this.net = new NeuralNetwork({ data: {} });
    },
    async load(e) {
        const json = await ODA.loadJSON(e.target.value);
    },
    dragover(e) {
        e.preventDefault();
    },
    drop(e) {
        const data = e.dataTransfer.getData('string');
        this.net.addLayer({ class: data, count: 25 });
    },
    download() {
        const a = document.createElement("a");
        a.href = URL.createObjectURL(
            new Blob([JSON.stringify(this.net.getData(), null, 2)], {
                type: "application/json"
            })
        );
        a.setAttribute("download", "data.json");
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        // browser.downloads.download({})
    },
    train() {
        this.stop = false;
        function run() {
            this.async(() => {
                while (true) {
                    this.iteration++;
                    const MutatedNetwork = this.bestNN.clone(); // Clone the current
                    // best network
                    MutatedNetwork.mutate(); // Mutate the clone
                    let MutatedNetworkCost = 0;
                    let CurrentNetworkResults = new Array(4); // The results that the mutated
                    // network calculated

                    // Calculate the cost of the mutated network
                    for (let i = 0; i < Inputs.length; i++) {
                        const Result = MutatedNetwork.FeedForward(Inputs[i]);
                        MutatedNetworkCost += Math.abs(Result[0] - ExpectedOutputs[i]);
                        CurrentNetworkResults[i] = Result[0];
                    }

                    // Does the mutated network perform better than the last one
                    if (MutatedNetworkCost < BestCost) {
                        delete this.bestNN;
                        BestNetwork = MutatedNetwork;
                        BestCost = MutatedNetworkCost;
                        BestNetworkResults = CurrentNetworkResults;
                    }
                    stop = BestCost < 0.01;

                    // Print only each 20000 iteration in order to speed up the training process
                    if (this.iteration % 2000 === 0 || stop === true) {
                        this.stat = '';
                        // Console.Clear(); // Clear the current console text

                        for (let i = 0; i < BestNetworkResults.length; i++) {
                            this.stat += Inputs[i][0] + "," + Inputs[i][1] + " | " + BestNetworkResults[i] + '\r';
                        }
                        this.stat += "Loss: " + new Intl.NumberFormat('en-IN', { maximumFractionDigits: 2, minimumFractionDigits: 2 }).format((BestCost * 100)) + "%\r"; // Print the best cost
                        this.stat += "Iteration: " + Iteration + "\r"; // Print the current Iteration
                        if (!this.stop)
                            run();
                        return;
                    }
                }
            }, 1)
        }
        run();
    }
})
ODA({
    is: 'oda-neural-layers',
    extends: 'this, oda-tree',
    template: `
        <style>
            :host{
              @apply --vertical;
              overflow: auto;
            }
            oda-button{
                width: 80px;
            }
        </style>
        <div class="horizontal" style="border-bottom: 1px solid silver;">
            <oda-button icon="icons:add" icon-pos="top" @tap="_tapAdd($event)">Add</oda-button>
            <oda-button :disabled="!focusedNode" icon="editor:mode-edit" icon-pos="top">Edit</oda-button>
            <oda-button :disabled="!focusedNode" icon="icons:remove" icon-pos="top" @tap="removeLayer(focusedNode)">Remove</oda-button>
        </div>
<!--        <oda-tree default-template="oda-neural-tree-item" class="flex" allow-focus allow-selection ref="tree" :data-set ::focused-node></oda-tree>-->
    `,
    props: {
        defaultTemplate: 'oda-neural-tree-item',
        allowFocus: true,
        allowSelection: true,
        net: {
            type: Object,
            set(n, o) {
                this.dataSet = (n?.layers || []).map(layer => {
                    return {
                        item: layer, name: layer.constructor.name, items: function () {
                            return this.item.neurons.map(n => {
                                return {
                                    item: n, name: n.constructor.name, items: function () {
                                        return this.item.inputs.map(s => ({ item: s, name: s.constructor.name }))
                                    }
                                };
                            })
                        }
                    }
                })
            }
        },
        lazy: true
    },
    async _tapAdd(e) {
        let res = await ODA.showDropdown('oda-neural-layer-list', { layers: Object.values(LAYERS) }, { parent: e.target.parentElement });
        if (!res) return;
        res = this.net.addLayer({ class: res.focused.name, neurons: 25 });
        res = {
            item: res, name: res.constructor.name, items: function () {
                return this.item.neurons.map(n => {
                    return {
                        item: n, name: n.constructor.name, items: function () {
                            return this.item.inputs.map(s => ({ item: s, name: s.constructor.name }))
                        }
                    };
                })
            }
        };
        this.dataSet.push(res);
        this.focusedNode = res;
        this._refresh(); // todo Это неправильно !!!
    },
    removeLayer(layer) {
        this.net.removeLayer(layer);
        this._refresh();
    }
})
ODA({
    is: 'oda-neural-layer',
    template: `
        <div>{{lid}} layer</div>
    `,
    props: {
        lid: 'layer'
    }
})
ODA({
    is: 'oda-neural-layer-list',
    template: `
        <style>
           div:hover{
                @apply --active;
           }
           div{
                align-items: center;
           }
           oda-icon{
                margin: 10px;
           }
        </style>
        <div ~for="layers" :focused="focused === item" @tap="focused = item" class="horizontal">
            <oda-icon :icon="item.icon"></oda-icon>
            {{item.name}}
        </div>
    `,
    props: {
        focused: Object,
        iconSize: {
            value: 32,
            shared: true
        },
        layers: Array
    }
})
ODA({
    is: 'oda-neural-tree-item',
    template: `
        <oda-icon :icon-size style="padding: 4px;" :icon="item?.item.icon"></oda-icon>
        <div>{{item?.item.constructor.name}}</div>
    `,
    props: {
        item: Object,
        iconSize: 32
    }
})
ODA({
    is: 'oda-neural-layer-item',
    template: `
        <oda-icon :icon-size style="padding: 4px;" :icon="item?.item.icon"></oda-icon>
        <div>{{item?.item.constructor.name}}</div>
    `,
    props: {
        item: Object,
        iconSize: 32
    }
})
ODA({
    is: 'oda-neural-layers-tree',
    template: `
        <style>
            :host{
                @apply --vertical;
            }
            .group{
                padding: 8px;
                font-size: large;
                font-weight: bold;
                @apply --header;
                @apply --horizontal;
                align-items: center;
            }
            .layer{
                padding-left: 32px;
                font-size: large;
                @apply --horizontal;
                align-items: center;
            }
            .layer:hover{
                background-color: whitesmoke;
            }
            label{
                margin: 8px;
                /*white-space: nowrap;*/
                text-overflow: ellipsis;
                overflow: hidden;
            }
        </style>
        <div ~for="group of groups" class="vertical">
            <div class="group">
                <oda-icon :icon="group.icon"></oda-icon>
                <label disabled>{{group.label}}</label>
            </div>
            <div ~for="getLayers(group)" class="vertical" :item draggable="true" @dragstart="dragstart">
                <div class="layer">
                    <oda-icon icon-size="48" :icon="item.icon"></oda-icon>
                    <label>{{item.label}}</label>
                </div>
            </div>
        </div>
    `,
    props: {
        icon: 'icons:receipt',
        groups() {
            return this.getLayers(LAYERS.NeuralLayer);
        }
    },
    dragstart(e) {
        e.dataTransfer.setData("string", e.target.item.name);
    },
    getLayers(parent) {
        return Object.values(LAYERS)
            .filter(l => l.__parent__ === parent)
            .sort((a, b) => a.__id__ > b.__id__ ? 1 : -1)
    }
})