import '/web/kernel.js';
const Synapse = CLASS({is: 'Synapse',
    created(data){
        this.neuron = data.neuron;
        this.source = data.source;
        this.w = data.w || Math.random()-.5;
    },
    props:{
        icon: 'odant:field',
        w: Number,
        neuron: Object,
        source: Object,
        x(){return this.neuron.x - this.source.x},
        y(){return this.neuron.y - this.source.y},
        z(){return this.neuron.z - this.source.z}
    }
})
const Neuron = CLASS({is: 'Neuron',
    created(data){
        this.layer = data.layer;
        this.x = data.x;
        this.y = data.y;
        this.id = data.id;
    },
    props:{
        id: Number,
        icon: 'odant:object',
        layer: Object,
        x: Number,
        y: Number,
        z(){return this.layer.z},
        inputs: {
            default: [],
            set(v){
                const prior = this.layer.prev;
                if (!prior) return;
                (v || []).map(i=>{
                    return this.connectTo(prior.neurons.find(n=>n.id === i.id), i)
                })
            }
        },
        charge(){
            return (this.inputs).reduce((res, i)=>{
                return (res += i.w);
            },0)
        }
    },
    getData(){
        if (!this.inputs.length) return;
        const data = {id: this.id};
        data.weights = this.inputs.map(i=>i.w);
        return data;
    },
    connectTo(neuron, props = {}){
        if (!neuron) return;
        props.neuron = this;
        props.source = neuron;
        const syn = new Synapse(props);
        this.inputs.push(syn);
        return syn
    }
})

function reLU(x) {
    if (x >= 0)
        return x;
    else
        return 0;
}
function reLU_der(x)
{
    if (x >= 0)
        return 1;
    else
        return 0;
}
function leakeyReLU(x) {
    if (x >= 0)
        return x;
    else
        return x / 20;
}
function leakeyReLU_der(x)
{
    if (x >= 0)
        return 1;
    else
        return 1.0 / 20;
}
function sigmoid(t) {
    return 1/(1+Math.pow(Math.E, -t));
}
function sigmoid_der(y) {
    return y*(1-y);
}
function softmax(arr) {
    return arr.map(function(value,index) {
        return Math.exp(value) / arr.map( (y /*value*/)=>{ return Math.exp(y) } ).reduce((a,b)=>{ return a+b })
    })
}
export const NeuralLayer = CLASS({is: 'NeuralLayer',
    created(data){
        this.index = data.index;
        this.net = data.net;
        if (data.neurons)
            this.neurons = data.neurons
        else if (data.size)
            this.size = data.size;
        else if (data.count)
            this.count = data.count;
        this.net.fire('changed', this);
    },
    props:{
        label: 'Слои нейронов',
        icon: 'icons:menu',
        net: Object,
        z(){
            return this.prev?(this.prev.z + this.prev.height):0;
        },
        prev(){
            return this.net.layers[this.index-1];
        },
        index: Number,
        count:{
            type: Number,
            set(v) {
                if (this.neurons?.length)
                    this.neurons.forEach(n => n._removePoint(n));
                
                const res = [];
                for (let i = 1; i <= v; i++){
                    if (v%i === 0)
                        res.push(i);
                }
                while (res.length > 2){
                    res.pop(); res.shift();
                }
                this.size = {width: res[0], length: res[1], height: res[2]};
            },
            get(){
                return this.neurons.length;
            }
        },
        size:{
            type: Object,
            set(v){
                const res = [];
                this.width = v.width;
                this.length = v.length || this.width;
                this.height = v.height || 1;
                const w = (this.width-1)/2;
                const l = (this.length-1)/2;
                for (let x=0; x<this.width; x++)
                    for (let y=0; y<this.length; y++)
                        for (let z=0; z<(this.height || 1); z++)
                            res.push({x: x-w, y: y-l, z})
                this.neurons = res;
            }
        },
        neurons:{
            type: Array,
            notify: true,
            set(v){
                return v.map((n, i)=>{
                    n.id = i;
                    n.layer = this;
                    return new Neuron(n);
                })
            }
        }
    },
    getData(){
        const data = {name: this.name, class: this.constructor.name};
        const neurons = this.neurons.map(i=>i.getData());
        if (neurons.filter(i=>i).length)
            data.neurons = neurons;
        else
            data.neurons = this.neurons.length;
        return data;
    }
})

export const NeuralInputLayer = NeuralLayer({is: 'NeuralInputLayer',
    props:{
        icon: 'icons:input',
        label: 'Входные слои'
    }
})
export const NeuralInputImageLayer = NeuralInputLayer({is: 'NeuralInputImageLayer',
    props:{
        icon: 'image:image',
        label: 'Изображение'
    }
})
export const NeuralInputSoundLayer = NeuralInputLayer({is: 'NeuralInputSoundLayer',
    props:{
        icon: 'image:music-note'
    }
})
export const NeuralInputTextLayer = NeuralInputLayer({is: 'NeuralInputTextLayer',
    props:{
        icon: 'icons:text-format'
    }
})
export const NeuralDenseGroup = NeuralLayer({is: 'NeuralDenseGroup',
    props: {

    }
})
export const NeuralDenseLayer = NeuralDenseGroup({is: 'NeuralDenseLayer',
    props: {

    }
})