/*
 * neuronet.js v1.0
 * (c) 2021 R.A. Perepelkin
 * Under the MIT License.
 */
// import '/web/oda/ext/gpu-browser.min.js';
const angles = {
    0: {x:-1,y:0},
    1: {x:-1,y:-1},
    2: {x:0,y:-1},
    3: {x:1,y:-1},
    4: {x:1,y:0},
    5: {x:1,y:1},
    6: {x:0,y:1},
    7: {x:-1,y:1},
}
function ReLU(x)
{
    if (x >= 0)
        return x;
    else
        return x / 20;
}
function getCosFI(a, b){
   let res =  (a.x * b.x + a.y * b.y + a.z * b.z);
   if (res)
        res = res / (Math.sqrt(a.x * a.x + a.y * a.y + a.z * a.z) * Math.sqrt(b.x * b.x + b.y * b.y + b.z * b.z));
   return res;
}
function sigmoid(t) {
    return 1/(1+Math.pow(Math.E, -t));
}
function getRandomAxis(){
    return Math.round(Math.random() * 3 - 1.5)
}
let id = 0;
function getId(){
    return id++;
}
class Neuron{
    threshold = 0   // Порог срабатывания
    sum = 0       // Уровень возбуждения
    inputs = [] // Входные синапсы
    synapses = [] // Выходные синапсы
    vector = undefined // Вектор предпочитаемого распространения
    charge = 0 // Уровень поляризации
    impulses = [] // Импульсы активации
    step = 0
    constructor(props, layer) {
        this.layer = layer; // Указатель нейросети
        this.brain = layer.net;
        this.props = props; // Свойства нейрона
        this.x = this.props.x || 0;
        this.y = this.props.y || 0;
        this.z = this.props.z || 0;
        (this.props.s || []).forEach(s => this.forceSynapse(s))
        this.id = getId();
    }
    get plast(){
        return this.layer.plast;
    }
    // Прием сигналов от входящих синапсов и  активация потенциала действия
    signal(v){
        // if (this.charge>0) return false;
        this.sum *= .9;
        this.threshold ++;
        this.sum += v;
        if (this.sum < this.threshold)
            return false;
        this.charge = this.sum;
        this.sum = this.threshold;
        return true;
    }
    tick(){
        this.synapses.forEach(s=>s.push());
        this.sum *= .5;
        if (this.sum < 0.5)
            this.sum = 0;
        this.threshold *= .4;
        if (this.threshold < 0.5)
            this.threshold = 0;
        if (this.charge>0){
            if (this.charge < 100){
                this.charge = 100;
                this.sum = 0;
                this.activation();
            }
            else{
                this.charge = -(this.charge * .1);
            }
        }
        else if (this.charge){
            this.charge = 0;
        }
        //this.charge && console.log(this.charge)
        return (this.charge || this.sum);
    }
    activation(){
        let v;
        if (Number.isNaN(this.brain.x)){
            const a = Math.round(Math.random() * 7)
            v = Object.assign({}, angles[a]);
        }
        else{
            v = {
                x: this.brain.x - this.x,
                y: this.brain.y - this.y,
                z: this.brain.z - this.z,
            }
        }
        while (!v.x && !v.y){
            const a = Math.round(Math.random() * 7)
            v = Object.assign({}, angles[a]);
        }
        let max = Math.sqrt((v.x * v.x)+(v.y * v.y));
        if (max > this.z){
            const k = (this.plast || 1) / max;
            v.x = Math.round(v.x * k);
            v.y = Math.round(v.y * k);
        }
        if (!this.layer.index)
            v.z = 1;
        else if (this.layer.isLast)
            v.z = 0;
        else
            v.z = (max < 1.5)?1:0;
        // if (this.vector && this.vector.x === v.x && this.vector.y === v.y){
        //     this.forceSynapse(this.vector);
        // }
        // else
        this.vector = v;
        this.step += this.plast;
        if (!this.synapses.length){
            this.step = 0;
            this.forceSynapse(this.vector);
        }
        else
            if (this.step > 1){
                this.forceSynapse(this.vector);
                this.step = 0;

        }
        for (let s of this.synapses){
            const weight = getCosFI(this.vector, s);
            s.signal(weight);
        }

    }
    forceSynapse(p){
        let syn = this.synapses.find(s=>(s.x === p.x && s.y === p.y && s.z === p.z))
        if (syn || this.layer.net.locked) return syn;
        syn = new Synapse(this, p);
        const inSyn = syn.target.synapses.find(s=>{
            return (s.x === -p.x && s.y === -p.y && s.z === -p.z);
        })
        if (inSyn){
            syn.strong = 0;
            syn.remove();
            return;
        }
        this.synapses.push(syn);
        const s = this.props.s || (this.props.s = []);
        s.push(p)
        return syn;
    }
}
class Synapse{
    impulses = []
    repeat = this.push.bind(this)
    charge = 0
    constructor(p, props) {
        this.parent = p; // Родительский нейрон
        this.props = props;
        this.x = this.props.x;
        this.y = this.props.y;
        this.z = this.props.z;
        this.length = Math.max(Math.abs(this.x), Math.abs(this.y),Math.abs(this.z))
        this.target = this.connectNeuron();
        this.strong = this.props.s || 1;
    }
    get plast(){
        return this.parent.plast;
    }
    connectNeuron(){
        const data = {
            x: this.parent.x + (this.x || 0),
            y: this.parent.y + (this.y || 0),
            z: this.parent.z + (this.z || 0)
        }
        const neuron = this.parent.layer.forceNeuron(data);
        if (!neuron.inputs.includes(this))
            neuron.inputs.push(this);
        return neuron;
    }
    signal(w){
        this.target.impulses.push(this.length);
        this.impulses.push({s:this.length-1, w});
        this.push();
    }
    push() {
        this.charge && (this.charge *= .9);
        if (this.charge<.5)
            this.charge = 0;
        const impulse = this.impulses.shift();
        if (!impulse) return;
        if (impulse.s) {
            impulse.s--;
            this.impulses.push(impulse);
            return;
        }
        let v = this.strong + impulse.w;
        this.charge ++;
        if (this.charge>this.strong){
            v *= -1;
            this.charge = 0;
        }

        if (this.target.signal(v)){
            if (this.strong<10)
                this.strong += (impulse.w * this.plast);
        }
        else if (v>0){
            this.strong *= (.95  * this.plast);
            if (this.strong<.5){
               this.remove();
            }
        }
    }
    remove(){
        if (this.target.synapses.length>0) return;
        this.parent.synapses.splice(this.parent.synapses.indexOf(this), 1);
        this.target.inputs.splice(this.parent.synapses.indexOf(this), 1);
        this.parent.layer.net.fire('removeSynapse', this);
        delete this;

    }
}
class NeuroLayer{
    neurons = []
    constructor(net, index) {
        this.net = net;
        this.index = index;
        this.isLast = index === this.net.max-1;
    }
    get plast(){
        let k = this.index + 1;
        const s = 1/k;
        if (!this.modulation) return s;
        return s * (this.net.modulation / 100 * k);
    }
    forceNeuron(data){
        const z = data.z || 0;
        if (z !== this.index)
            return this.net.forceNeuron(data);
        let n = this.neurons.find(i => i.x === data.x && i.y === data.y)
        if (!n){
            n = new Neuron(data, this);
            this.neurons.push(n);
        }
        return n;
    }
    step(){
        this.net.step();
    }
    get x(){
        return this.net.x;
    }
    get y(){
        return this.net.y;
    }
    get z(){
        return this.net.z;
    }
    get charge(){
        return this.neurons.length;
    }
}
export default class NeuroNet {
    tact = 0
    precision = 2
    _delay = 0
    layers = {}
    max = 10
    stop = false
    impulse = 50
    focusedNeurons = []
    subs = {}
    modulation = 0;
    collect = {};
    constructor(data) {
        data.map(i=>this.forceNeuron(i))
        this.step();
    }
    signal(){
        this.layers[0].neurons.forEach(n=>n.signal(1))
    }
    get sensors(){
        return (this.layers[0]?.neurons || []);
    }
    set delay(v){
        this.stop  = 0;
        clearInterval(this.active);
        this.active = 0;
        this._delay = v;
        this.step();
    }
    get delay(){
        return this._delay;
    }
    get inputs(){
        return (this.layers[0]?.neurons || []);
    }
    step(){
        if (this.active) return;
        this.active = setInterval(()=>{
            this.modulation && (this.modulation *=.99);
            if (this.modulation<.5)
                this.modulation = 0;
            this.tact++;
            let i = 0;
            let X = 0;
            let Y = 0;
            let Z = 0;
            let T = 0;
            for (let layer of Object.values(this.layers)){
                for (let n of layer.neurons){
                    i++;
                    const val = n.tick(this);
                    const c = this.collect[n.id];
                    if (c && val) {
                        c[this.tact] = val;
                    }
                    if (this.focusedNeurons.includes(n)){
                        //сбор массивов значений для выбранных нейронов
                    }
                    if (n.charge) {
                        T += n.charge;
                        X += n.x * n.charge;
                        Y += n.y * n.charge;
                        Z += n.z * n.charge;
                    }
                }
            }
            this.x = Math.round(X/T);
            this.y = Math.round(Y/T);
            this.z = Math.round(Z/T);
            if (!this.stop) return;
            clearInterval(this.active);
            this.active = 0;
        }, this.delay)
    }
    forceNeuron(data){
        const index = data.z || 0;
        const layer = this.layers[index] || (this.layers[index] = new NeuroLayer(this, index));
        return layer.forceNeuron(data);
    }
    fire(e, s) {
        const ev = new CustomEvent(e, { detail: { value: s }, composed: true })
        this.subs[e].forEach(i => {
            i(ev);
        })
    }
    listen(e, h) {
        let a = this.subs[e];
        if (!a) a = this.subs[e] = [];
        a.push(h);
    }
}