/*
 * neuronet.js v1.0
 * (c) 2021 R.A. Perepelkin
 * Under the MIT License.
 */
// import '/web/oda/ext/gpu-browser.min.js';
// Нахождение ближайшей точки по вектору направления

function NOD (p) {
    const vars = Object.values(p);
    for (var x = Math.abs(vars[0]), i = 1; i < vars.length; i++) {
        let y = Math.abs(vars[i]);
        while (x && y)
            x > y ? x %= y : y %= x;
        x += y;
    }
    const v = Math.abs(x);
    if (v){
        Object.keys(p).forEach(i=>{
            p[i] = p[i]/v;
        })
    }
    return p;
}
const angles = {
    0: {x:-1,y:0},
    1: {x:-1,y:-1},
    2: {x:0,y:-1},
    3: {x:1,y:-1},
    4: {x:1,y:0},
    5: {x:1,y:1},
    6: {x:0,y:1},
    7: {x:-1,y:1},
}
function getCosFI(a, b){
   let res =  (a.x * b.x + a.y * b.y + a.z * b.z);
   if (res)
        res = res / (Math.sqrt(a.x * a.x + a.y * a.y + a.z * a.z) * Math.sqrt(b.x * b.x + b.y * b.y + b.z * b.z));
   return res;
}
function sigmoid(t) {
    return 1/(1+Math.pow(Math.E, -t));
}
function getRandomAxis(){
    return Math.round(Math.random() * 3 - 1.5)
}
let id = 0;
function getId(){
    return id++;
}
class Neuron{
    threshold = 0   // Порог срабатывания
    sum = 0       // Уровень возбуждения
    inputs = [] // Входные синапсы
    synapses = [] // Выходные синапсы
    vector = undefined // Вектор предпочитаемого распространения
    charge = 0 // Уровень поляризации
    impulses = [] // Импульсы активации
    step = 0
    constructor(props, layer) {
        this.layer = layer; // Указатель нейросети
        this.brain = layer.net;
        this.props = props; // Свойства нейрона
        this.x = this.props.x || 0;
        this.y = this.props.y || 0;
        this.z = this.props.z || 0;
        (this.props.s || []).forEach(s => this.forceSynapse(s))
        this.impulse = this.layer.net.impulse;
        this.id = getId();
        this.activation = this.activation1.bind(this);
        this.plast = this.layer.plast;
    }
    // Прием сигналов от входящих синапсов
    signal(v){
        this.sum *= .9;
        if (this.charge)
            return false;
        this.threshold += .5;
        this.sum += v;
        if (this.sum<this.threshold)
            return false;
        this.charge = 50;
        this.sum = 0;
        setTimeout(this.activation);
        return true;
    }
    tick(){
        this.charge = -this.charge/10;
        if (Math.abs(this.charge)<.5)
            this.charge = 0;
        this.sum *= .7;
        if (Math.abs(this.sum)<.5)
            this.sum = 0;
        this.threshold *= .5;
        if (this.threshold<.5)
            this.threshold = 0;
        for (let s of this.synapses)
            s.charge && (s.charge-=1);
        return this.charge;
    }
    activation1(){
        this.charge = 50;
        let v;
        if (Number.isNaN(this.brain.x)){
            const a = Math.round(Math.random() * 7)
            v = Object.assign({}, angles[a]);
        }
        else{
            v = {
                x: this.brain.x - this.x,
                y: this.brain.y - this.y,
                z: this.brain.z - this.z,
            }
        }
        // while (!v.x && !v.y){
        //     const a = Math.round(Math.random() * 7)
        //     v = Object.assign({}, angles[a]);
        // }
        let max = Math.sqrt((v.x * v.x)+(v.y * v.y));
        if (max > this.plast){
            const k =/* (this.plast || 1)*/2 / max;
            v.x = Math.round(v.x * k);
            v.y = Math.round(v.y * k);
        }
        if (!this.layer.index)
            v.z = 1;
        else if (this.layer.isLast)
            v.z = 0;
        else
            v.z = (max < 1.5)?1:0;

        for (let s of this.synapses){
            const weight = getCosFI(v, s);
            s.signal(weight);
        }
        // if (this.vector && this.vector.x === v.x && this.vector.y === v.y){
        //     this.forceSynapse(this.vector);
        // }
        // else
        this.vector = v;
        this.step++;
        if (this.step>this.plast){
            this.forceSynapse(this.vector);
            this.step = 0
        }


    }
    forceSynapse(p){
        let syn = this.synapses.find(s=>(s.x === p.x && s.y === p.y && s.z === p.z))
        if (syn || this.layer.net.locked) return syn;
        syn = new Synapse(this, p);
        this.synapses.push(syn);
        const s = this.props.s || (this.props.s = []);
        s.push(p)
        return syn;
    }
    toString(){
        return `Neuron: ${this.x},${this.y},${this.z}\rthreshold: ${this.threshold}\rimpulses: ${this.impulses.length}\rcharge: ${this.charge}\rsynapses: ${this.synapses.length}\rinputs: ${this.inputs.length}`;
    }
}
class Synapse{
    impulses = []
    repeat = this.push.bind(this)
    charge = 0
    constructor(p, props) {
        this.parent = p; // Родительский нейрон
        this.props = props;
        this.x = this.props.x;
        this.y = this.props.y;
        this.z = this.props.z;
        this.length = Math.max(Math.abs(this.x), Math.abs(this.y),Math.abs(this.z))
        this.target = this.connectNeuron();
        this.strong = this.props.s || 1;
    }
    get plast(){
        return this.z + 1;
    }
    toString(){
        return `Synapse: ${this.x},${this.y},${this.z}\rstrong: ${this.strong}\rcharge: ${this.charge}\rlength: ${this.length}`;
    }
    connectNeuron(){
        const data = {
            x: this.parent.x + (this.x || 0),
            y: this.parent.y + (this.y || 0),
            z: this.parent.z + (this.z || 0)
        }
        const neuron = this.parent.layer.forceNeuron(data);
        if (!neuron.inputs.includes(this))
            neuron.inputs.push(this);
        return neuron;
    }
    signal(w){
        this.impulses.push({s:this.length, w});
        setTimeout(this.repeat);
    }
    push() {
        const impulse = this.impulses.shift();
        if (!impulse) return;
        if (impulse.s) {
            impulse.s--;
            this.impulses.push(impulse);
            setTimeout(this.repeat);
            return;
        }
        this.charge++;
        let v = this.strong + impulse.w;
        if (this.charge>10){
            v *= -1;
            this.charge = 0;
        }

        if (this.target.signal(v)){
            if (this.strong<10)
                this.strong += impulse.w/this.plast;
        }
        else{
            this.strong *= .9 / this.plast;
            if (this.strong<.5){
                this.remove();
                return;
            }
        }
        setTimeout(this.repeat);
    }
    remove(){
        this.parent.synapses.splice(this.parent.synapses.indexOf(this), 1);
        this.target.inputs.splice(this.parent.synapses.indexOf(this), 1);
        delete this;
    }
}
class NeuroLayer{
    neurons = []
    constructor(net, index) {
        this.net = net;
        this.index = index;
        this.isLast = index === this.net.max-1;
        this.plast = Math.round(Math.sqrt(this.index))+1;
    }
    forceNeuron(data){
        const z = data.z || 0;
        if (z !== this.index)
            return this.net.forceNeuron(data);
        let n = this.neurons.find(i => i.x === data.x && i.y === data.y)
        if (!n){
            n = new Neuron(data, this);
            this.neurons.push(n);
        }
        return n;
    }
    step(){
        this.net.step();
    }
    get x(){
        return this.net.x;
    }
    get y(){
        return this.net.y;
    }
    get z(){
        return this.net.z;
    }
    get charge(){
        return this.neurons.length;
    }
}
export default class NeuroNet {
    tact = 0
    precision = 2
    delay = 0
    layers = {}
    max = 10
    stop = false
    impulse = 50
    constructor(data) {
        data.map(i=>this.forceNeuron(i))
        this.step();
    }
    signal(){
        this.layers[0].neurons.forEach(n=>n.signal(1))
    }
    get sensors(){
        return (this.layers[0]?.neurons || []);
    }
    step(){
        if (this.active) return;
        this.active = setInterval(()=>{
            this.tact++;
            let i = 0;
            let X = 0;
            let Y = 0;
            let Z = 0;
            let T = 0;
            for (let layer of Object.values(this.layers)){
                for (let n of layer.neurons){
                    i++;
                    const val = n.tick(this);
                    if (n.charge) {
                        T += n.charge;
                        X += n.x * n.charge;
                        Y += n.y * n.charge;
                        Z += n.z * n.charge;
                    }
                }
            }
            this.x = Math.round(X/T);
            this.y = Math.round(Y/T);
            this.z = Math.round(Z/T);
            if (!this.stop) return;
            clearInterval(this.active);
            this.active = 0;
        })
    }
    forceNeuron(data){
        const index = data.z || 0;
        const layer = this.layers[index] || (this.layers[index] = new NeuroLayer(this, index));
        return layer.forceNeuron(data);
    }
    toString(){
        return `layers: x: ${this.x}, y: ${this.y}, z: ${this.z}\rneurons: ${this.neurons.length}`;
    }
}