const TIMEOUT = 604800000; // millis in week
const LOGIN_STORAGE_KEY = 'logins';
const SECURITY_STORAGE_KEY = 'security';

export function getFromLS(key) {
  const val = localStorage.getItem(key);
  return val && JSON.parse(val) || null;
}

export function setToLS(key, val) {
  localStorage.setItem(key, JSON.stringify(val));
}

export async function API(params, init) {
  const url = new URL(`${location.origin}/api`);
  for (const m in params) {
    url.searchParams.set(m, params[m]);
  }
  return fetch(url, init);
}

export function accumulateLogin(login) {
  const logins = getFromLS(LOGIN_STORAGE_KEY) || {};
  for (const l in logins) {
    if (l === login) {
      logins[l] = { date: Date.now(), current: true };
    } else {
      if (login[l].date + TIMEOUT < Date.now()) {
        delete login[l];
      } else {
        login[l].current = false;
      }
    }
  }
  logins[login] = { date: Date.now(), current: true };
  setToLS(LOGIN_STORAGE_KEY, logins);
}

export function getLogins() {
  return getFromLS(LOGIN_STORAGE_KEY) || {};
}

export function setLogins(logins) {
  setToLS(LOGIN_STORAGE_KEY, logins);
}

export function getSecurity() {
  return getFromLS(SECURITY_STORAGE_KEY) || {};
}

export function setSecurity(security) {
  setToLS(SECURITY_STORAGE_KEY, security);
}

export function purgeLogin(login) {
  const logins = getFromLS(LOGIN_STORAGE_KEY) || {};
  delete logins[login];
  setToLS(LOGIN_STORAGE_KEY, logins);
}

export async function goToRegistration() {

}

export function getForgotUrl(login, redirect_url) {
  return appendSearchParams('./change-pass.html', [{ name: 'login', value: login }, { name: 'redirect_url', value: redirect_url }]);
}

export function appendSearchParams(url = '', params) {
  let s = url.includes('?') ? '&' : '?';
  for (const p of params) {
    if (p.value) {
      url += `${s}${encodeURIComponent(p.name)}=${encodeURIComponent(p.value)}`;
      s = '&';
    } else {
      url.replace(new RegExp(`[&?]${p.name}=[^?&]*`, 'ig'), '');
      s = url.includes('?') ? '&' : '?';
    }
  }
  return url;
}

export async function findAccount(login) {
  const response = await API({ method: 'find_account', login });
  const json = await response.json();
  return json.result;
}

export function goToAuthorize(code, login, redirect_url) {
  const url = new URL(`${(new URL(redirect_url)).origin}/web/auth/authorize.html`);
  url.searchParams.set('code', code);
  url.searchParams.set('login', login);
  url.searchParams.set('redirect_url', redirect_url);
  setTimeout(() => {
    location.assign(url);
  }, 500);
}

export async function saveAuthorize() {
  const url = new URL(location);
  const login = url.searchParams.get('login');
  const code = url.searchParams.get('code');
  const redirect_url = url.searchParams.get('redirect_url') || '/';
  const security = getSecurity();
  security.login = login;
  security.code = code;
  setSecurity(security);
  try {
    const res = await authorize(code);
    if (res) {
      location.assign(redirect_url);
      accumulateLogin(login);
    }
  } catch (err) {
    location.assign(redirect_url);
    console.error(err);
  }
}
export async function authorize(body){
  const response = await API({ method: 'authorize' }, { method: 'POST', body });
  const json = await response.json();
  return json.result;
}
export async function checkSsid(){
  const response = await API({ method: 'check_ssid' });
  const json = await response.json();
  return json.result;
}
