import * as Auth from '../auth.js';
export async function authenticate(login, password) {
  const body = await sha256(password);
  const response = await Auth.API({ method: 'authentication', login, version: 2 }, { method: 'POST', body });
  if (response.ok) {
    return response.text();
  } else {
    throw new Error((await response.text()) || `Error on server side: ${response.statusText}`);
  }
}
export async function registration(login, password) {
  const body = await sha256(password);
  const register_res = await Auth.API({ method: 'registration', login, referrer: location, type: 'email' }, { method: 'POST', body });
  if (register_res.ok) {
    return true;
  } else {
    throw new Error(register_res.statusText || (await register_res.text()));
  }
}
export async function changePass(password) {
  return Auth.API({ method: 'change_password' }, { method: 'POST', body: await sha256(password) });
}
export async function sendPin(login, type = 'email') {
  return Auth.API({ method: 'one_time_password', login, type });
}
async function sha256(message) {
  const msgBuffer = new TextEncoder().encode(message);
  const hashBuffer = await crypto.subtle.digest('SHA-256', msgBuffer);
  const hashArray = Array.from(new Uint8Array(hashBuffer));
  const hashHex = hashArray.map(b => ('00' + b.toString(16)).slice(-2)).join('');
  return hashHex;
}