export default odaField({
    is: 'odaSystemField',
    get input() {
        return this.cache('input', async () => {
            return await FIELD.regComponent(this._input);
        })
    },
    props:{
        icon() {
            if (this.isTable)
                return 'odant:grid';
            if (this.$typeClass && !this.$typeClass.isAbstract)
                return 'icons:link';
            if (this.hasFields)
                return 'odant:fields';
            return 'odant:field';
        },
        _input() {
            if (this.isTable) return '';
            if (this.$typeClass && !this.$typeClass.isAbstract)
                return 'field-link';
            return 'field-default';
        },
        _complex() {
            if (this.isTable)
                return 'field-table';
            if (this.hasChildren)
                return 'field-structure';
        },
    }
})

FIELD({
    is: 'default',
    template: /*html*/`
    <style>
        :host{
            @apply --horizontal;
            @apply --flex;
            align-items: center;
        }
        input, span, a {
            @apply --flex;
            @apply --label;
            @apply --horizontal;
            padding: 4px;
            border: none;
            background-color: transparent;
            text-overflow: ellipsis;
            font-size: small;
        }
        :host([link]) > span {
            text-decoration: underline;
            color: #0000ff;
            cursor: pointer;
        }
        ::-webkit-input-placeholder {
            opacity: 0.5;
        }
        ::-moz-placeholder {
            opacity: 0.5;
        }
        :-moz-placeholder {
            opacity: 0.5;
        }
        :-ms-input-placeholder {
            opacity: 0.5;
        }
    </style>
    <div ~is="readOnly?'span':'input'" ~text="value?.name || value" :value="value?.name || value" ref="input" :placeholder="_placeholder" :title="_title" @tap="_tap" @input.stop="_input"></div>
    `,
    props: {
        link: {
            type: String,
            reflectToAttribute: true
        },
        _placeholder: {
            type: String,
        },
        _title: {
            type: String,
        },
    },
    observers: [
        function _byFocused(focused) {
            if (focused) this.$refs.input.focus();
            else this.$refs.input.blur();
        },
        function _fieldChanged($field) {
            if ($field) {
                this._placeholder = $field.placeholder;
                this._title = $field.Note;
            }
        },
        function _setLink(dataRoot, $field, readOnly) {
            if (readOnly) {
                const link = dataRoot['$' + $field.name];
                if (link) {
                    if (Array.isArray(link) && link.length > 0)
                        this.link = link[0].link;
                    else if (typeof (link) === 'string')
                        this.link = link;
                }
                else {
                    this.link = undefined;
                }
            }
        },
        function _valueChanged(value) {
            if (this.focused) this.setValue(value);
            if (!this.focused) this.$refs.input.value = value;
        }
    ],
    _input(e) {
        this.fire('input', e.currentTarget.value);
        this.value = e.currentTarget.value;
    },
    async _tap(e) {
        if (this.link) {
            try {
                const obj = this.$object || await this.$field.$class.getObject(this.dataRoot.oid);
                const link = await this.$field.getValue(obj, this.dataRoot);
                link.show();
            } catch (e) {
                ODA.push(e)
            }
        }
    }
})
FIELD({
    is: 'link',
    extends: 'field-default'
})
FIELD({
    is: 'table',
    template:`
        ЭТО ТАБЛИЦА!!!
    `
})
FIELD({
    is: 'structure',
    imports: '/web/core/object/client/views/editor/editor.js',
    extends: 'odant-layout-structure'
})