import '/web/core/class/fields/lib/base/base.js';

ODA({
    is: 'odant-field-default',
    extends: 'this, odant-field-input',
    template: /*html*/`
    <style>
        :host{
            @apply --horizontal;
            @apply --flex;
            align-items: center;
        }
        input, span, a {
            @apply --flex;
            @apply --label;
            @apply --horizontal;
            padding: 4px;
            border: none;
            background-color: transparent;
            text-overflow: ellipsis;
            font-size: small;
        }
        :host([link]) > span {
            text-decoration: underline;
            color: blue;
            cursor: pointer;
        }
        ::-webkit-input-placeholder {
            opacity: 0.5;
        }
        ::-moz-placeholder {
            opacity: 0.5;
        }
        :-moz-placeholder {
            opacity: 0.5;
        }
        :-ms-input-placeholder {
            opacity: 0.5;
        }
    </style>
    <input ~show="!readOnly" ref="input" :placeholder="_placeholder" :title="_title" @tap="_tap" @input.stop="_input">
    <span ~show="readOnly" ~text="value && value.name || value"></span>
    `,
    props: {
        link: {
            type: String,
            reflectToAttribute: true
        },
        _placeholder: {
            type: String,
        },
        _title: {
            type: String,
        },
    },
    observers: [
        function _byFocused(focused) {
            if (focused) this.$refs.input.focus();
            else this.$refs.input.blur();
        },
        function _fieldChanged($field) {
            if ($field) {
                this._placeholder = $field.placeholder;
                this._title = $field.Note;
            }
        },
        function _setLink(dataRoot, $field, readOnly) {
            if (readOnly) {
                const link = dataRoot['$' + $field.name];
                if (link) {
                    if (Array.isArray(link) && link.length > 0)
                        this.link = link[0].link;
                    else if (typeof (link) === 'string')
                        this.link = link;
                }
                else {
                    this.link = undefined;
                }
            }
        },
        function _valueChanged(value) {
            if (this.focused) this.setValue(value);
            if (!this.focused) this.$refs.input.value = value;
        }
    ],
    _input(e) {
        this.fire('input', e.currentTarget.value);
        this.value = e.currentTarget.value;
    },
    async _tap(e) {
        if (this.link) {
            try {
                const obj = this.$object || await this.$field.$class.getObject(this.dataRoot.oid);
                const link = await this.$field.getValue(obj, this.dataRoot);
                link.show();
            } catch (e) {
                ODA.push(e)
            }
        }
    }
});