ODANT({
    is: 'odant-grid',
    imports: [
        '/web/core/index/lib/index/index.js',
        '@lib/item-menu',
        '/web/core/class/lib/grid/table-grid.js',
    ],
    template: /* html*/ `
        <style>
            :host {
                @apply --flex;
                @apply --vertical;
                overflow: hidden;
            }
        </style>
        
        <odant-index style="display: none;" ref="index" :screen="screen" :mask="packMask" ::size="size" :context-item="contextItem" ::rows="rows" ::fields="fields"></odant-index>
        <odant-table-grid allow-sort show-borders even-odd id-name="oid" :size="size"
                    :allow-drag="allowDrag"
                    :allow-drop="allowDrop"
                    ref="table" lazy class="flex" :columns="columns" @selection-changed="_selection"
                    @screen-changed="_screen" @row-dblclick="_dblclick"
                    @row-contextmenu="_onRowContextMenu"
                    :allow-selection="allowSelection"
                    allow-focus show-footer show-header col-lines row-lines
                    :preFooter="footer"
                    :custom-rows="_rows">
        </odant-table-grid>
    `,
    dataSet: [],
    props: {
        rows: {
            type: Array,
            set(n) {
                if (n) {
                    this._rows = n.filter(i => {
                        if (i.$footer) {
                            this.footer = i;
                            return false;
                        }
                        return true;
                    });
                }
            }
        },
        _rows: Array,
        footer: Object,
        size: 0,
        screen: Object,
        allowDrag: true,
        allowDrop: 'extended',
        allowSelection: true,
        contextItem: null,
        focusedItem: {
            type: Object,
            notify: true
        },
        treeMode: false,
        packMask: String,
        filterVal: {
            type: String,
            watch(v) {
                this.debounce('filter', () => {
                    this.$refs.index.filter = v
                }, 300)
            }
        },
        fields: {
            type: Array,
            async watch(fields) {
                if (fields && (!this.columns || !this.columns.length)) {
                    fields = fields.map(async field => {
                        try {
                            const obj = {
                                name: field.name,
                                label: field.label,
                                template: await field.input,
                                $field: field,
                                type: field.type
                            };
                            return obj;
                        } catch (e) {
                            return { name: field.name, label: field.label, $field: field }
                        }
                    });
                    const cols = await Promise.all(fields);
                    cols.unshift({ width: 24, fix: 'left', template: 'odant-marker-cell' });
                    let name = cols.find(i => i.name === 'name');
                    if (name) {
                        name.fix = 'left';
                        name.treeMode = true;
                        name.template = 'odant-field-name-control';
                    }
                    
                    // cols.forEach(i => {
                    //     if (i.name === 'name')
                    //         i.template = 'odant-field-name-control';
                    // });

                    this.columns = cols;
                } else {
                    this.columns = [];
                }
            }
        },
        columns: [],
        data: {
            type: Object,
            async watch(n) {
                if (n) {
                    ODA.loader.on();
                    try {
                        let items = await n.Items;
                        items = items.reduce((res, i, idx, arr) => {
                            if (i.p) {
                                const parent = arr.find(p => p.oid === i.p);
                                if (parent && parent !== i) {
                                    parent.items = parent.items || [];
                                    parent.items.push(i);
                                    return res;
                                }
                            }
                            res.push(i);
                            return res;
                        }, []);

                        this.$options.dataSet = items;
                        this.render();
                    } finally {
                        ODA.loader.off()
                    }
                }
            }
        }
    },
    ready() {
        const isMobile = /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);
        if (isMobile) {
            this.allowDrag = false;
            this.allowDrop = 'none';
        }
    },
    async _dblclick(e) {
        const item = e.detail.value;
        const obj = await this.getObject(item);
        ODANT.show(obj);
    },
    async getBase(item) {
        if (item.bid)
            return this.contextItem.$base.findItem(`B:${item.bid}`);
        return this.contextItem.$base;
    },
    async getClass(item) {
        if (item.cid) {
            const base = await this.getBase(item);
            return (base || this.contextItem.$base).findItem(`C:${item.cid}`);
        }
        return this.contextItem;

    },
    async getObject(item) {
        let res;
        if (item.bid && item.cid) {
            res = await this.contextItem.$base.findItem(`B:${item.bid}/C:${item.cid}/O:${item.oid}`);
        }
        if (!res) {
            const cls = this.contextItem || await this.getClass(item);
            res = await cls.getObject(item.oid);
        }
        return res;
    },
    async _onRowContextMenu(e) {
        e.preventDefault();
        e.stopPropagation();
        const context = await this.getObject(e.detail.value);

        ODA.showDropdown('odant-item-menu', {contextItem: context, groups: '*', collapseLimit: 3});//, {parent: e.detail});
    },
    async _selection(e, d) {
        const selection = e.detail.value;
        if (!selection.length)
            this.focusedItem = null;
        else {
            this.focusedItem = await this.getObject(selection[0]);
            if (selection.length > 1)
                this.focusedItem = new odaItemList({
                    $owner: this.contextItem,
                    itemType: odaObject,
                    items: selection
                });
        }
    },
    _screen(e) {
        // this.screen = null;
        this.screen = { ...e.detail.value };
    }
})

ODANT({
    is: 'marker-cell',
    extends: 'odant-base-field',
    template: `
        <style>
            :host {
                @apply --vertical;
                justify-content: center;
                cursor: pointer;
            }

            oda-icon {
                border-radius: 50%;
            }
        </style>
        <oda-icon :style="{backgroundColor: value}" :size="iconSize*.5" :icon="item && item.icon"></oda-icon>
    `,
    props: {
        value: {
            get() {
                return this.dataRoot?.clr || '';
            }
        }
    },
    listeners: {
        tap: (e) => {

        }
    }
})

ODANT({
    is: 'class-item-field-control',
    imports: [
        '/web/core/item/lib/item-node/item-node.js',
    ],
    extends: 'odant-base-field, odant-item-node',
    props: {
        hidePrefix: true,
        adminMode: false,
        allowExpand: true,
        $base: {
            type: Object,
            async watch(base) {
                if (base) {
                    this.contextItem = (this.dataRoot.cid && await (this.$base || this.column.$class.$base).findItem('C:' + this.dataRoot.cid)) || this.column.$class;
                }
            }
        }
    },
    observers: [
        function _init(column, dataRoot) {
            if (dataRoot.bid) {
                column.$class.$base.findItem('B:' + dataRoot.bid).then(base => {
                    this.$base = base;
                })
            } else
                this.$base = column.$class.$base; //(dataRoot.bid && await column.$class.$base.findItem('B:'+dataRoot.bid)) || column.$class.$base;

        }
    ]
})

ODANT({
    is: 'field-name-control',
    extends: 'odant-base-field',
    template: `
        <span class="bold" ~style="{opacity:(item && item.name)?1:.3}" style="font-size: small"> {{item && (item.name || item.oid)}}</span>
    `,
})

