if (!globalThis.CORE) {
    JSON.selectSingleNode = function (json, path, force) {
        if (!path || !json)
            return json;
        path = path.split('/');
        let step = path.shift();
        if (step) {
            switch (step) {
                case 'CDATA()':
                    return json['$CDATA$'];
                case 'TEXT()':
                    return json['$TEXT$'];
                default: {
                    if (step[0] === '@') {
                        return json[step.substring(1)];
                    }
                    else {
                        json = json['$' + step];
                        if (json && Array.isArray(json) && json.length > 0) {
                            return JSON.selectSingleNode(json[0], path.join('/'));
                        }
                    }
                }
                    break;
            }
        }
        return undefined;
    };
    function getBool(v, def = false) {
        if (!v)
            return def;
        switch (typeof v) {
            case 'object':
                return true;
            case 'string':
                return v.toLowerCase() === 'true';
            case 'boolean':
                return v;
            case 'number':
                return v !== 0;
        }
        return false;
    }
    const attrMap = {
        Name: 'name',
        Label: 'label',
        Type: 'type'
    }
    const configMap = {
        i: 'id',
        l: 'label',
        n: 'name',
        t: 'type',
        a: 'accessLevel',
        v: 'version',
    }
    const oldModulesExclude = [
        'demo', 'lib', 'dist'
    ]
    const coreReservedWords = [
        'is', 'template', 'props', 'extends', 'keys', 'observers', 'listeners', 'hostAttributes'
    ]
    const prefixMap = {
        H: 'host',
        P: 'part',
        B: 'base',
        // S: 'system',
        W: 'workplace',
        D: 'domain',
        M: 'module',
        C: 'class',
        O: 'object',
        // I: 'index',
    }
    const prefMap = {
        'C:': ['B:', 'M:', 'W:', 'D:'],
        'B:': ['P:', 'B:'],
        'M:': ['B:', 'P:'],
        'W:': ['B:'],
        'D:': ['B:', 'P:'],
        'P:': ['H:'],
        'H:': null
    };
    const ACTION_DIR_MASKS = ['client', 'server', 'filed', 'client/developer/*', 'client/pages/*', 'client/create/*', 'client/views/*', 'client/methods/*', 'server/methods/*', 'server/tasks/*', 'server/tasks/on-start/*', 'server/tasks/on-timer/*', 'server/triggers/*', 'server/triggers/after-create/*', 'server/triggers/after-delete/*', 'server/triggers/after-update/*', 'server/triggers/before-create/*', 'server/triggers/before-delete/*', 'server/triggers/before-update/*',];
    //const prefArray = ['', 'H:', 'P:', ['B:', 'M:', 'W:', 'D:'], 'C:'];
    const targetStack = [];
    const SERVER_MODE = typeof globalThis.process !== 'undefined';
    const DEFAULTORIGIN = 'https://current.odant.org';
    // const ACTION_DIR_MAP = {
    //     'client': ['views', 'pages', 'methods', 'admin'],
    //     'server': ['methods', 'tasks', 'triggers']
    // };
    const proxyActionHandler = {
        get: (target, key) => {
            const prop = Object.getOwnPropertyDescriptor(target.props, key);
            if (!prop || (typeof prop.value === 'function'))
                return target[key];
            return target.getAttr(key, target[key], prop.value && prop.value.type);
        },
        set: (target, key, value) => {
            const prop = Object.getOwnPropertyDescriptor(target.props, key);
            if (prop && !prop.readOnly && (prop.set || prop.value) && target.$class) {
                // TODO
                if (!target.root) {
                    let action;
                    if (target.$module) {
                        if (!target.$class.root.$MODULES) {
                            target.$class.root.$MODULES = [{ $MODULE: [{ id: target.$module.id, $ACTION: [] }] }];
                            action = this.$class?.root?.$MODULES?.[0].$MODULE[0].$ACTION;
                        } else {
                            let module = this.$class?.root?.$MODULES?.[0].$MODULE.find((m) => m.id === target.$module.id);
                            if (!module) {
                                const l = this.$class?.root?.$MODULES?.[0].$MODULE.push({ id: target.$module.id, $ACTION: [] })
                                module = this.$class?.root?.$MODULES?.[0].$MODULE[l - 1];
                            }
                            action = module.$ACTION;
                        }

                    } else {
                        if (!target.$class.root.$ACTIONS) {
                            target.$class.root.$ACTIONS = [{ $ACTION: [] }];
                        }
                        action = target.$class.root.$ACTIONS[0].$ACTION;
                    }
                    action.push({ id: target.id });
                }
                // delete if equal default value
                target.setAttr(key, (target[key] !== value) ? value : undefined)
                target.$class.fireChanged();
            }
            else {
                target[key] = value;
            }
            return true;
        }
    }
    globalThis.RequestError = globalThis.RequestError || class RequestError extends Error {
        constructor(response, message) {
            super(message);
            this.errorCode = response.headers.get('odant-error-code');
            this.status = response.status;
            this.statusText = response.statusText;
            this.url = response.url;
        }
    };
    globalThis.WebSocketEvents = globalThis.WebSocketEvents || class WebSocketEvents {
        constructor(url) {
            this._url = url;
            this._start();
        }
        _start() {
            try {
                this._webSocket = new WebSocket(this._url);
                this._webSocket.onopen = (event) => this._onOpen(event);
                this._webSocket.onclose = (event) => this._onClose(event);
                this._webSocket.onmessage = (event) => this._onMessage(event);
            } catch (e) { }
        }
        _onOpen(event) {
            this._recoverySubscribe();
            console.log('Open webSocket connection');
        }
        _onClose(event) {
            setTimeout(() => {
                this._start();
            }, 3000);
            console.log('Close webSocket connection');
        }
        _onError(event) {
            this._webSocket.close();
            console.log('Error in webSocket connection');
        }
        _onMessage(event) {
            let info = JSON.parse(event.data);
            if (info.event && (info.data || info.custom))
                this._fire(info.event, (info.data || info.custom));
        }
        _fire(name, data) {
            this._callbacks = this._callbacks || {};
            let args = [].slice.call(arguments, 1);
            let callbacks = this._callbacks['$' + name];

            if (callbacks) {
                callbacks = callbacks.slice(0);
                for (let i = 0, len = callbacks.length; i < len; ++i) {
                    callbacks[i].apply(this, args);
                }
            }

            return this;
        }
        _is_connected() {
            return this._webSocket && this._webSocket.readyState === 1;
        }
        _send(data) {
            if (this._is_connected() && (typeof data === 'string') && (data.length > 0)) {
                this._webSocket.send(encodeURIComponent(data));
            }
        }
        _sendSubscribe(name) {
            if (this._is_connected()) this._send('+' + name);
        }
        _sendUnSubscribe(name) {
            if (this._is_connected()) this._send('-' + name);
        }
        _sendUnSubscribeAll() {
            if (this._is_connected()) this._send('-*');
        }
        _recoverySubscribe() {
            if (this._is_connected() && this._callbacks) {
                let data;
                for (let name in this._callbacks) {
                    if (data) data += '\r\n';
                    else data = '';
                    data += '+' + name.substring(1);
                }

                if (data.length > 0) this._send(data);
            }
        }
        addEventListener(name, callback) {
            this._callbacks = this._callbacks || {};
            this._callbacks['$' + name] = this._callbacks['$' + name] || []
            if (this._callbacks['$' + name].push(callback) === 1)
                this._sendSubscribe(name);
            return this;
        }
        removeEventListener(name, callback) {
            this._callbacks = this._callbacks || {};

            if (0 === arguments.length) return this;

            // specific event
            let callbacks = this._callbacks['$' + name];
            if (!callbacks) return this;

            // remove all handlers
            if (1 === arguments.length) {
                this._sendUnSubscribe(name);
                delete this._callbacks['$' + name];
                return this;
            }

            // remove specific handler
            let cb;
            for (let i = 0; i < callbacks.length; i++) {
                cb = callbacks[i];
                if (cb === callback) {
                    callbacks.splice(i, 1);
                    break;
                }
            }

            if (0 === callbacks.length)
                this._sendUnSubscribe(name);

            return this;
        }
        removeAllListeners() {
            this._callbacks = {};
            this._sendUnSubscribeAll();
            return this;
        }
        listeners(name) {
            this._callbacks = this._callbacks || {};
            return this._callbacks['$' + name] || [];
        }
        hasListeners(name) {
            return !!this.listeners(name).length;
        }
        get URL() {
            return this._url;
        }
        sendCustomEvent(name, eventObject) {
            if (typeof name !== "string") {
                throw Error("sendCustomEvent error: name is not string");
            }
            if (name.length == 0) {
                throw Error("sendCustomEvent error: name is empty");
            }
            let isDataString = (typeof eventObject === "string");
            let isDataObject = (typeof eventObject === "object");
            if (!isDataString && !isDataObject) {
                throw Error("sendCustomEvent error: eventObject must be string or object");
            }
            const msgObj = {
                root: {
                    event: name,
                    custom: JSON.stringify(eventObject)
                }
            };
            this._send(JSON.stringify(msgObj));
        }
    };
    globalThis.CORE = function (prototype) {
        const parents = (typeof prototype.extends === 'string'
            ? prototype.extends.replace(/ /g, '').split(',').map((e) => {
                if (globalThis[e])
                    return globalThis[e];
                throw Error(`Parent class '${e}' for inherit to '${prototype.is}' not found!`)
            })
            : [prototype.extends]).filter(Boolean);
        prototype.listeners = prototype.listeners || {};
        prototype.props = prototype.props || {};
        //#region Наследование свойств и методов
        for (let key in prototype) {
            if (coreReservedWords.includes(key)) continue;
            const item = Object.getOwnPropertyDescriptor(prototype, key);
            if (typeof item.value === 'function')
                continue;
            // props вне props
            if (item.value) {
                prototype.props[key] = prototype.props[key] || item.value;
            }
            else if (item.get || item.set) {
                prototype.props[key] = prototype.props[key] || {};
                prototype.props[key].get = item.get;
                prototype.props[key].set = item.set;
            }
            delete prototype[key];
        }

        for (let key in prototype.props) {
            let prop = prototype.props[key];
            if (typeof prop === 'function')
                prop = (prop.name === key) ? { get: prop } : { type: prop };
            if (Array.isArray(prop))
                prop = { default: prop, type: Array };
            else if (prop === null || typeof prop !== "object")
                prop = { default: prop, type: prop ? prop.__proto__.constructor : Object };
            else if (Object.keys(prop).length === 0 || (!prop.get && !prop.set && prop.default === undefined && !prop.type))
                prop = { default: prop, type: Object };
            if (typeof prop.get === 'string')
                prop.get = prototype[prop.get];
            if (typeof prop.set === 'string')
                prop.set = prototype[prop.set];
            if (!prop.type && prop.default !== undefined)
                prop.type = prop.default.__proto__.constructor;
            if(prop.default && !prop.get){
                if(Array.isArray(prop.default)){
                    prop.get = function(){
                        return Array.from(prop.default);
                    }
                }else if(typeof prop.default === 'object'){
                    prop.get = function(){
                        return Object.assign({}, prop.default);
                    }
                }else{
                    prop.get = function () {
                        return prop.default;
                    }
                }
            }
            prototype.props[key] = prop;
        }

        // ... от родителей
        if (parents) {
            for (let parent of parents) {
                for (let key in parent.$prototype.props) {
                    const parentProp = parent.$prototype.props[key];
                    //if (parentProp) continue;
                    const targetProp = prototype.props[key];
                    if (targetProp) {
                        for (let k in parentProp) {
                            targetProp[k] = targetProp[k] || parentProp[k];
                        }
                    }
                    prototype.props[key] = targetProp || parentProp;
                }
                for (const k in parent.$prototype.listeners) {
                    if (!prototype.listeners[k])
                        prototype.listeners[k] = parent.$prototype.listeners[k];
                }
                for (let key in parent.$prototype) {
                    const targetMethod = Object.getOwnPropertyDescriptor(prototype, key);
                    if (targetMethod && targetMethod.value)
                        continue;
                    const parentMethod = Object.getOwnPropertyDescriptor(parent.$prototype, key);
                    if (parentMethod && (parentMethod.get || typeof parentMethod || typeof parentMethod.value === 'function'))
                        Object.defineProperty(prototype, key, parentMethod);
                }
            }
        }
        if (this) {
            for (const k in this.$prototype.listeners) {
                if (!prototype.listeners[k])
                    prototype.listeners[k] = this.$prototype.listeners[k];
            }
        }
        //#endregion

        const clsName = prototype.is || (this ? ('$' + this.name) : 'cls');
        const body = /*javascript*/`with (globalThis) {
            function ${clsName} (props){
                if (!new.target)
                    return CORE.call(${prototype.is}, props);
                this.__cache__ = {};
                this.__customCache__ = {};
                this.__deps__ = {};
                this.__handlers__ = {};
                if (!this.constructor.created.hasParams) {
                    //debugger;
                    for (let p in props) {
                        let value = props[p];
                        const prop = this.props[p];
                        if (value !== undefined && prop) {
                            if (prop.type === Boolean)
                                value = (value === 'true') || !!value;
                            else if (prop.type === Number)
                                value = +value;
                        }
                        this[p] = value;
                        // this.__cache__[p] = value;
                    }
                }
                this.constructor.created.call(this, props);

                if (prototype.listeners && typeof prototype.listeners === 'object') {
                    for (const k in prototype.listeners) {
                        const h = typeof prototype.listeners[k] === 'string'
                            ? this[prototype.listeners[k]]
                            : prototype.listeners[k];
                        this.listen(k, h);
                    }
                }
            }
            return ${clsName}
        }`;
        const cls = (new Function('p, prototype', body))(this, prototype);
        cls.prototype = Object.create((this || CORE).prototype);
        cls.prototype.constructor = cls;
        cls.$prototype = prototype;

        Object.defineProperties(cls.prototype, {
            $super: {
                enumerable: true,
                get() {
                    return cls.prototype.__proto__;
                }
            },
            listen: {
                enumerable: false,
                value: function (type, handler) {
                    if (!this.__handlers__[type]) this.__handlers__[type] = [];
                    this.__handlers__[type].push(handler);
                }
            },
            unlisten: {
                enumerable: false,
                value: function (type, handler) {
                    if (!this.__handlers__[type]) return;
                    const idx = this.__handlers__[type].indexOf(handler);
                    if (~idx) this.__handlers__[type].splice(idx, 1);
                }
            },
            addEventListener: {
                enumerable: false,
                value: cls.prototype.listen
            },
            removeEventListener: {
                enumerable: false,
                value: cls.prototype.unlisten
            },
            fire: {
                enumerable: false,
                value: async function (type, detail) {
                    if (!this.__handlers__[type]) return;
                    const promises = [];
                    for (const h of this.__handlers__[type]) {
                        if (h.constructor.name === 'AsyncFunction') {
                            promises.push(h.call(this, undefined, detail));
                        } else {
                            h.call(this, undefined, detail);
                        }
                    }
                    await Promise.all(promises);
                }
            },
        })
        Object.defineProperty(cls.prototype, 'props', {
            get() {
                return prototype.props;
            }
        })
        if (prototype.is && !globalThis[prototype.is]) {
            globalThis[prototype.is] = cls;
        }
        for (let key in prototype.props) {
            const prop = prototype.props[key];
            const descriptor = { enumerable: !key.startsWith('_'), configurable: false };
            descriptor.get = function () {
                const stack = targetStack ? [...targetStack].reverse() : [];
                for (let i of stack) {
                    if (i.target === this && i.prop === key)
                        continue;
                    let deps = this.__deps__[key];
                    if (!deps)
                        deps = this.__deps__[key] = [];
                    if (deps.find(d => {
                        return d.target === i.target && d.prop === i.prop;
                    })) continue;
                    deps.push(i);
                }
                if (prop.get && !(key in this.__cache__)) {
                    targetStack.push({ target: this, prop: key })
                    this.__cache__[key] = prop.get.call(this);
                    targetStack.pop();
                    const deps = this.__deps__[key] || [];
                    for (let d of deps) {
                        delete d.target.__cache__[d.prop];
                    }
                }
                return this.__cache__[key];
            }
            descriptor.set = prop.set ?
                function (val) {
                    this.__cache__[key] = val;
                    const res = prop.set.call(this, val);
                    if (res !== undefined)
                        this.__cache__[key] = res;
                    const deps = this.__deps__[key] || []
                    for (let d of deps) {
                        delete d.target.__cache__[d.prop];
                    }
                } : (prop.readOnly ? undefined :
                    function (val) {
                        this.__cache__[key] = val;
                        const deps = this.__deps__[key] || []
                        for (let d of deps) {
                            delete d.target.__cache__[d.prop];
                        }
                    })
            Object.defineProperty(cls.prototype, key, descriptor);
        }
        for (let key in prototype) {
            const d = Object.getOwnPropertyDescriptor(prototype, key);
            if (typeof d.value === 'function' && key !== 'created') {
                    cls.prototype[key] = function (...args) {
                        return d.value.call(this, ...args);
                    }
            }
            else if (d.get || d.set)
                Object.defineProperty(cls.prototype, key, d);
        }
        const prefix = prototype.props?.prefix?.default;
        if (prefix) {
            CORE.__factory__[prefix] = CORE.__factory__[prefix] || cls;
        }
        cls.defaults = cls.defaults || {};
        //let obj = cls.prototype.__proto__.constructor;
        //while (obj) {
            // if (this) {
            //     for (const k in this.defaults) {
            //         if ((k in cls.defaults) || this.$prototype.props[k]?.get) continue;
            //         cls.defaults[k] = this.defaults[k];
            //     }
            // }

            const created = this?.created;
            const myCreated = prototype.created;
            cls.created = function (...args) {
                created?.call(this, ...args);
                myCreated?.call(this, ...args);
            }
            const funcStr = myCreated?.toString().replace(/\s*/g, '') || '';
            const startArgs = funcStr.indexOf('(');
            const endArgs = funcStr.indexOf(')');
            cls.created.hasParams = created?.hasParams || ((endArgs - startArgs) > 1);

            // cls.created = cls.created
            //                   ? (props) => {
            //                       this.$prototype?.created?.call(this, props);
            //                       cls.created.call(this, props);
            //                     } 
            //                   : this.$prototype?.created;
            //if (!created) created = obj?.$prototype?.created;
            //obj = obj.prototype.__proto__?.constructor;
        //}
        // console.dir(cls);
        return cls;
    }
    CORE.getContextFromUrl = function (url) {
        if (url) {
            url = decodeURI(url);
            url = url.split('?')[0];
            url = url.split('#')[0];
            let i = url.indexOf('/api/');
            if (i < 0)
                return null;
            url = url.substring(i + 4);
            i = url.lastIndexOf('/~');
            if (i < 0) {
                url = url.split('/').filter(s => {
                    return s.indexOf(':') === 1;
                }).join('/');
            }
            else {
                url = url.substring(0, i);
            }

            return url;
        }
        return '';
    }
    CORE.mixin = function (from, to, deep = 0) {
        if (from && to) {
            for (let i in from) {
                if (!to[i])
                    to[i] = from[i];
                if (deep)
                    CORE.mixin(from[i], to[i], deep - 1);
            }

            return to;
        }
    }
    CORE.__factory__ = {};
    CORE.getFactory = function (prefix) {
        return CORE.__factory__[prefix];
    }
    if (!SERVER_MODE && !document.location.host.startsWith('localhost'))
        CORE.URL = new URL(document.location.origin);
    else
        CORE.URL = new URL(DEFAULTORIGIN);
    CORE({
        is: 'odaAccess',
        props: {
            level: Number,
            label() {
                switch (this.level) {
                    case 6: return 'admin';
                    case 5: return 'delete';
                    case 4: return 'create';
                    case 3: return 'write';
                    case 2: return 'read';
                    case 1: return 'preview';
                    case 0: return 'none';
                    default: return 'unknown';
                }
            },
            color() {
                switch (this.level) {
                    case 6: return 'var(--admin-color)';
                    case 5: return 'var(--rwcd-color)';
                    case 4: return 'var(--rwc-color)';
                    case 3: return 'var(--rw-color)';
                    case 2: return 'var(--r-color)';
                    case 1: return 'var(--p-color)';
                }
                return 'var(--no-access-color)';
            },
            letter() {
                return this.label[0].toUpperCase();
            }
        }
    })
    CORE({
        is: 'odaIndexDataSet',
        props: {
            $index: Object,
            mask: String,
            filter: String,
        },
        get Items() {
            return (async () => {
                let data = await this.$index.request((this.filter) ? 'search_index' : 'xquery_index',
                    { loadmask: this.mask, format: 'json', by_user: this.ByUser, async: true },
                    this.filter || '*/*');

                data = data ? data[Object.keys(data)[0]] : [];

                return data;
            })();
        },
        get metadata() {
            return this.$index.$$fields;
        },
        async getItems(from = 1, length = 100) {
            try {
                const data = await this.$index.request('xquery_index',
                    { loadmask: this.mask, format: 'json', by_user: this.ByUser, async: true },
                    `subsequence(*/*, ${from}, ${length})`);
                return data[Object.keys(data)[0]];
            } catch (e) {
                console.error(e);
                return [];
            }
        },
        async getRowItem(row) {
            if (row) {
                let rows = Array.isArray(row) ? row : [row];
                if (rows.length === 0)
                    return Promise.resolve(null);

                const item = await this.$index.$class.getObject(rows[0].oid);
                if (rows.length === 1)
                    return item;
                return null;
                // TODO odaItemList
                //return new odaItemList({ $owner: this.$index.$class, $item: item, $$items: rows.map(i => { return i.oid }) });
            }
            return Promise.reject();
        }
    })
    CORE({
        is: 'odaInherit',
        props: {
            name: String,
            level: Number,
            color() {
                switch (this.name) {
                    case 'owner':
                        return 'var(--inherit-owner)';
                    case 'parent':
                        return 'var(--inherit-parent)';
                    case 'type':
                        return 'var(--inherit-type)';
                    case 'core':
                        return 'var(--inherit-core)';
                    case 'service':
                        return 'var(--inherit-service)';
                    default:
                        return 'var(--inherit-self)';
                }
            }
        }
    })
    CORE({
        is: 'odaItem',
        props: {
            $part() {
                return this.$owner.$path;
            },
            id: String,
            typeName: String,
            $class() {
                // No $class in host
                return this.$owner?.$class;
            },
            $module() {
                return this.$owner?.$module;
            },
            $base() {
                // No $base in host
                return this.$owner?.$base;
            },
            $host() {
                return this.$owner.$host;
            },
            $owner: null,
            name() {
                return this.id;
            },
            label() {
                return this.name;
            },
            icon: 'odant:item',
            defaultViewName: 'editor',
            defaultPage: 'navigator',
            defaultForm: 'form',
            accessLevel: undefined,
            access: {
                category: 'system',
                get() {
                    if (!this.__cache__.accessLevel && this.$owner && this.$part?.id === 'ROOT')
                        return this.$owner.access;
                    return new odaAccess({ level: this.__cache__.accessLevel });
                }
            },
            inherit() {
                const root = this.root || this.config || {};
                let level = root.inherit;
                if (level === undefined && this.$owner)
                    return this.$owner.inherit;
                return new odaInherit({ name: level });
            },
            _color: 'var(--item-color)',
            color() {
                if (this.access?.level >= 6 && this.inherit.name) return this.inherit?.color || this._color;
                return this._color;
            },
            fill() {
                return this.color;
            },
            actionsPath() {
                return `web/${this.typeName}/client/*/*`;
            },
            isRef() {
                return this.$owner && (this.$owner.isRef || this.$owner instanceof odaWorkPlace);
            },
            zIndex: 0,
        },
        get $$items() {
            return this.cache('$$items', async () => {
                let items = [];
                for (let i in this) {
                    if (i.startsWith('$$') && i !== '$$items')
                        items.push(...(await this[i]));
                }
                return items;
            })
        },
        get $$actions() {
            return this.getActions();
        },
        debounce(key = this.id, callback, delay) {
            this._denounces = this._denounces || {};
            clearTimeout(this._denounces[key]);
            this._denounces[key] = setTimeout(callback, delay);
        },
        save(body) {
            if (this.access.level > 2) {
                this.fire('before-save', this);
                return this._save(body).then(res => {
                    this.fire('saved', this);
                    this.fire('after-save', this);
                    return res
                })

                // TODO events
                // if (!this.fire('before-save', this)) {
                //     return this._save(body).then(res => {
                //         this.fire('saved', this);
                //         this.fire('after-save', this);
                //         return res
                //     })
                // }
                // else {
                //     let p = this.events['before-save'].map((i) => {
                //         if (i.then)
                //             return i;
                //         return Promise.resolve(i());
                //     });
                //     return Promise.all(p).then(() => {
                //         return this._save(body).then(res => {
                //             this.fire('saved', this);
                //             this.fire('after-save', this);
                //             return res
                //         })
                //     })
                // }
            }
        },
        async _save(body) {
            body = body || (await this.body);
            if (body) {
                return this.request(undefined, undefined, body).then(res => {
                    this.body = null;
                })
            }
            throw new Error('no data for save');
        },
        import(path) {
            return import(this.url + '/~/' + path);
        },
        getActionFiles(node) {
            return (node.$DIR || []).reduce((res, cat) => {
                const files = (cat.$DIR || []).reduce((res, d) => {
                    const f = d.$FILE?.find(f => f.id === `${d.id}.js`);
                    if (f) {
                        f.parent = d;
                        f.owner = cat;
                        res.push(f);
                    }
                    return res;
                }, [])
                res.push(...files);
                return res;
            }, [])
            // return node?.$DIR?.reduce((res, d) => {
            //     // if (map) {
            //     //     if (map === d.id || map[d.id] || map.includes?.(d.id)) {
            //     //         res.push(...this.getActionFiles(d, map[d.id] || null, d));
            //     //     }
            //     // } else {
            //         const f = d.$FILE?.find(f => f.id === `${d.id}.js`);
            //         if (f) {
            //             f.parent = d;
            //             f.owner = cat;
            //             res.push(f);
            //         }
            //     // }
            //     return res;
            // }, []);
        },
        async getActions($item = this) {
            return this.cache('getActions' + $item?.typeName, async () => {
                if (!$item.config) await $item.getConfig?.();
                const body = await $item?.$class?.body;
                let path = $item.actionsPath;
                const endIdx = path.indexOf('*');
                if (~endIdx) path = path.slice(0, endIdx - 1);
                const url = `${this.$folder.url}/${path}?get_dirlist&deep=3`;
                const dirStruct = await (await fetch(url)).json();
                //TODO: доработать для клиентской и серверной стороны
                const actFiles = this.getActionFiles(dirStruct/*, ACTION_DIR_MAP.client*/);
                let actions = actFiles.map(async f => {
                    const module = { /* $folder: f, */ id: f.parent.id, path: f.path, $owner: $item, $source: this };
                    let action;
                    try {
                        const result = await import(f.path);
                        action = new result.default(module);
                    }
                    catch (e) {
                        console.error(f.path, e);
                        action = new odaActionError(module);
                    }
                    action.category = action.category || f?.owner?.id;
                    return action;
                })
                actions = (await Promise.all(actions));
                actions = actions.filter(Boolean);

                // вызывает все тормоза при открытии контекстного меню и т.п.
                console.time(`getActionsExp ${this.path}`);
                if (!(this.$owner instanceof odaModule)) {
                    const exports = (await this.$base?.getActionsExp(this)) || [];
                    // console.dir(exports);
                    actions.push(...exports);
                }
                console.timeEnd(`getActionsExp ${this.path}`);

                //todo фильтрация конечного списка перед выводом
                return actions.map(a => new Proxy(a, proxyActionHandler)).filter(a => a.active);
            });
        },
        async getActionsExp($item) {
            return this.cache(`getActionsExp-${$item.typeName}`, async () => {
                const actions = await this.getActions($item);
                return actions.filter(i => i.export);
            });
        },
        async findAction(name) {
            const actions = await this.getActions();
            return actions.find(i => i.name === name);
        },
        cache(key, callback) {
            let result = this.__customCache__[key];
            if (result === undefined) {
                if (typeof callback === 'function')
                    result = callback();
                else
                    result = callback;
                this.__customCache__[key] = result;
            }
            return result;
        },
        toJSON() {
            let obj = {};
            for (let i in this) {
                if (this[i] && typeof this[i] !== 'object')
                    obj[i] = this[i];
            }
            return obj;
        },
        getFormUrl(form, view) {
            const $form = `${form || this.defaultForm}/index2.html`;
            const $view = view ? `?view=${view}` : '';
            const url = `${this.url}/~/client/views/${$form}${$view}`;

            //return ODANT.URL(url);
            return url;
        },
        open(form, view) {
            const url = this.getFormUrl(form, view);
            window.open(url, '_blank');
        },
        navigate(form, view) {
            const url = this.getFormUrl(form, view);
            window.open(url);
        },
        getAttr(name, def, type) {
            let res = undefined;
            if (this.root) {
                name = name.split('|');

                while (!res && name.length) {
                    const n = name.shift().trim();
                    res = this.root[n];
                }
                if (res === undefined || res === '' || res === null) return def;
                switch (type) {
                    case Boolean: {
                        res = String(res).toLowerCase() === 'true';
                    } break;
                    case Number: {
                        res = Number(res);
                    } break;
                    case Date: {
                        res = Date.parse(res);
                    } break;
                }
                return res;
            }
            return def;
        },
        /** @param {string} name @param {any} v */
        setAttr(name, v) {
            if (this._itemCode && this.root && this.root[name] !== v) {
                if (v === undefined || v === null)
                    delete this.root[name];
                else
                    this.root[name] = v;
                this.fireChanged();
            }
        },
        /** @param {string} name @param {Boolean} def @returns {Boolean} */
        getBoolAttr(name, def = false) {
            return getBool(this.getAttr(name), def);
        },
        /** @param {string} name @param {Boolean} v */
        setBoolAttr(name, v) {
            this.setAttr(name, v === undefined ? v : v ? 'True' : 'False');
        },
        /** @param {string} name @param {number} def @returns {Boolean} */
        getNumAttr(name, def = 0) {
            return Number(this.getAttr(name)) || def || 0;
        },
        /** @param {string} name @param {number} v */
        setNumAttr(name, v) {
            this.setAttr(name, v === undefined ? v : String(v).replace(',', '.'));
        },
        /** @param {string} name @param {Date} def @returns {Date} */
        getDateAttr(name, def) {
            return Date.parse(this.getAttr(name)) || def;
        },
        /** @param {string} name @param {Date} v */
        setDate(name, v) {
            this.setAttr(name, v === undefined ? v : v.toISOString());
        },
        getRelativeId(item) {
            if (!item || !item.path) {
                return null;
                // throw 'incorrect item';
            }

            let res = item.path;
            let cnt_id = this.path;
            while (cnt_id.indexOf("/") > -1) {
                res = res.replace(cnt_id + "/", '');
                cnt_id = cnt_id.substring(0, cnt_id.lastIndexOf("/"));
            }
            return res;
        }
    })
    odaItem({
        is: 'odaVirtualItem'
    })
    odaItem({
        is: 'odaBodyOwner',
        props: {
            root: Object,
            body() {
                return this.cache('body', async () => {
                    const body = await this.request();
                    this.root = body?.[this.rootTag]?.[0];
                    await this.fire('body');
                    return body;
                })
            }
        }
    })
    odaItem({
        is: 'odaIndexesOwner',
        get $$indexes() {
            return this.cache('$$indexes', async () => {
                await this.body;
                let folders = await this.getFolders('index/*')
                if (!folders) return [];
                const indexes = folders.map(async (f) => {
                    // TODO временно через файл
                    const file = await f.getFile(f.name + '.xq');
                    if (!file) return null;
                    const result = await file.request('xquery', { format: 'json' }, '*');
                    const conf = { id: f.name, name: f.name, $owner: this };
                    if (result?.$XQUERY) {
                        conf.root = result.$XQUERY[0];
                        for (let p in conf.root) {
                            const s = attrMap[p];
                            if (s === undefined) continue;
                            conf[s] = conf.root[p];
                        }
                    }
                    return new odaIndex(conf);
                });
                return (await Promise.all(indexes)).filter(Boolean);

            });
        },
        getIndex(name, XQ, type) {
            return this.cache('Index:' + name, async () => {
                if (XQ) {
                    const param = { name: name };
                    if (type) param.type = type;
                    await this.API.POST('create_index', param, XQ);
                }
                const index = (await this.$$indexes).find(i => i.name === name)
                return index || new odaIndex({ id: name, name, $owner: this });
            })
        }
    })
    odaItem({
        is: 'odaServerItem',
        props: {
            version: 0,
            prefix: String,
            body: Object,
            id: undefined,
            name() {
                return this.root?.name || this.id;
            },
            label() {
                return this.root?.label || this.name;
            },
            icon() {
                if (this.config.icon)
                    return this.config.icon;
                return this.__cache__.icon;
            },
            path() {
                let pref = '';
                let s = '';
                let own = this;
                while (own) {
                    let step = (own.id ? own.id : null);
                    if (step && own.prefix)
                        step = own.prefix + ':' + step;
                    if (step) {
                        step = step;
                        if (!own.prefix || own.prefix !== pref) {
                            s = `${step}${s && `/${s}` || ''}`
                        }
                    }
                    pref = own.prefix;
                    own = own.$owner;
                }
                return s;
            },
            _fullPath() {
                return (this.$owner ? (this.$owner._fullPath + '/') : '') + (this.prefix ? this.prefix + ':' : '') + this.id;
            },
            url() {
                return this.$host.origin + '/api' + (this.id ? ('/' + this.path) : '');
            },
            $folder() {
                return new odaFolder({ $owner: this, id: '~' });
            },
            defaultViewName() {
                return 'editor';
            },
            $$files: {
                type: Array,
                get() {
                    return this.getFiles();
                }
            },
            $$folders: {
                type: Array,
                get() {
                    return this.getFolders();
                }
            },
            $$actionFolders: {
                type: Array,
                    get() {
                    return this.getActionFolders();
                }
            }
        },
        get hasChildren() {
            return this.config && this.config.ch === 'T';
        },
        resolveUrl(path) {
            return this.url + '/' + path;
        },
        XQuery(xq, { format = 'json' } = {}) {
            return this.request('xquery', { format: format/* , async: true */ }, xq)
        },
        async request(method = '', params = {}, postObject) {
            try {
                let url = encodeURI(this.url + (method ? '?' + method : '') + Object.keys(params).reduce((res, p) => {
                    res += ((res || method) ? '&' : '?') + p + '=' + params[p];
                    return res;
                }, ''))
                if (typeof postObject !== 'undefined' && typeof postObject !== 'string') {
                    postObject = JSON.stringify(postObject);
                }
                params = { method: postObject ? 'POST' : 'GET', body: postObject, credentials: 'include' };
                const response = await fetch(url, params)
                switch (response.status) {
                    case 200: {
                        let content_type_header = response.headers.get('Content-Type');
                        let res = content_type_header ? content_type_header.split(';')[0] : content_type_header;
                        switch (res) {
                            case 'application/x.odant.async':
                            case 'application/x.odant.async+json':
                            case 'text/x-json':
                            case 'application/json':
                                return response.json();
                            case 'text/cmd':
                            case 'text/css':
                            case 'text/csv':
                            case 'text/javascript':
                            case 'application/javascript':
                            case 'text/php':
                            case 'text/html':
                            case 'text/plain':
                            case 'text/xml':
                                return response.text();
                            case 'image/png':
                            case 'image/gif':
                            case 'image/jpg':
                            case 'image/jpeg':
                            case 'application/octet-stream':
                                return CORE.webMode ? response.blob() : response.buffer();

                            default: {
                                throw new RequestError(response, 'Unresolved fetch mime-type: ' + res);
                            }
                        }
                    }
                    case 500: {
                        const text = await response.text();
                        throw new RequestError(response, text);
                    }
                    default: {
                        // throw new RequestError(response, response.statusText);
                    }
                }
            }
            catch (e) {
                // console.error(e)
            }
        },
        async load(params, fetchParams) {
            try {
                const res = await this.request(undefined, params, fetchParams);
                if (typeof res === 'object')
                    this.body = res;
                else
                    this.data = res;
                return res;
            }
            catch (e) {
                this.error = e;
                console.error(e);
            }
        },
        async findItem(path, params = { path: path, chain: 'full', prefix: true }) {
            const cacheKey = 'find-item:' + Object.keys(params).reduce((res, key) => {
                return res + `${key}=${JSON.stringify(params[key])}`;
            }, '');
            return this.cache(cacheKey, async () => {
                try {
                    if (!params.path.includes('~')) {
                        //todo: доработать
                        if (params.path.startsWith('/')) params.path = params.path.substring(1);
                        let s = params.path.split('/');
                        let last = s.pop();

                        const prefixKeys = Object.keys(prefMap);
                        let pref = prefixKeys.find(p => last.startsWith(p));
                        if (!pref) {
                            pref = 'C:';
                            last = 'C:' + last;
                        }
                        params.path = last;

                        while (prefMap[pref]) {
                            last = s.pop();
                            if (!last) {
                                pref = prefMap[pref][0];
                                params.path = `${pref}*/${params.path}`;
                            } else {
                                const map = prefMap[pref];
                                pref = map.find(p => last.startsWith(p));
                                if (!pref) {
                                    pref = map[0];
                                    params.path = `${pref}${last}/${params.path}`;
                                } else {
                                    params.path = `${last}/${params.path}`;
                                }

                            }
                        }
                        // s = params.path.split('/');
                        // let node = this.config;
                        // let path = this.path;
                        // while (s.length > 0 && node) {
                        //     const id = s.shift();
                        //     path += '/' + id;
                        //     const prefixId = id.split(':');
                        //     node = this.findConfigNode(node, prefixId[0], prefixId[1]);
                        // }
                        //
                        // if (node) {
                        //     return this.getItem(path);
                        // }
                    }

                    path = await this.request('find_item', params);
                    if (!path) return null;
                    return this.$host.getItem(path?.result);
                }
                catch (e) {
                    console.error(e)
                }
            });
        },
        findConfigNode(node, prefix, id) {
            let result = node[`$${prefix}`]?.find(n => n.i === id);
            if (result) return result;

            let root = node;
            for (const key in root) {
                if (!key.startsWith('$')) continue;

                for (const child of root[key]) {
                    result = this.findConfigNode(child, prefix, id)
                    if (result) return result;
                }
            }

            return result;
        },
        async getItem(path) {
            return this.cache('get-item:' + path, async () => {
                if (this.config === undefined) {
                    await this.getConfig?.();
                }

                let step = this.prefix + ':' + this.id;
                let idx = path?.indexOf(step);
                if (idx < 0) {
                    // TODO удалить, после появления серверного кэша
                    let item;
                    if (this.$owner) {
                        item = await this.$owner.getItem(path);
                    } else {
                        step = path.split('/').filter(Boolean)[0];
                        const prefix = step.split(':')[0];
                        const id = step.split(':')[1];
                        if (prefix === this.$host.prefix) {
                            const h = this.$host.$$hosts.find(h => h.id === id) || new odaHost({ origin: CORE.URL.origin, id: id });
                            this.$host.$$hosts.push(h);
                            item = await h.getItem(path);
                        }
                    }

                    if (item) return item;
                    throw new Error('Undefined search context: ' + path);
                }
                path = path?.substring(++idx + step.length);
                if (!path)
                    return this;
                if (!path.includes(':')) {
                    // File
                    return path.includes('.')
                        ? this.$folder.getFile(path.replace('~/', ''))
                        : this.$folder.getFolder(path.replace('~/', ''));
                }
                step = path.split('/')[0];
                const prefix = step.split(':')[0];
                const id = step.split(':')[1];
                let ext, config;
                if (prefix === 'O') {
                    return this.getObject(id);
                } else {
                    if (this.config === undefined && Object.keys(this.config).some(key => key.startsWith('$'))) await this.getConfig?.();
                    const items = this.config['$' + prefix];
                    config = items?.find(i => i.i === id);
                    ext = config?.extend || `/web/core/${prefixMap[prefix]}/extend.js`;
                }
                try {
                    const imp = await import(ext);
                    const item = new imp.default({ id, prefix, $owner: this });
                    item.config = config || (await item.getConfig?.());
                    return item.getItem(path);
                } catch (err) {
                    console.error(err);
                    const imp = await import(`/web/core/${prefixMap[prefix]}/extend.js`);
                    const item = new imp.default({ id, prefix, $owner: this });
                    item.config = config || (await item.getConfig?.());
                    return item.getItem(path);
                }
            });
        },
        async getDirList() {
            return this.cache('get_dirlist', async () => await this.request('get_dirlist', { deep: 2 }));
        },
        async getActionFolders(){
            return (await this.$$folders).filter(f => ACTION_DIR_MASKS.some(an => {
                return f.path.endsWith(an) || (an.endsWith('/*') && (f.path.endsWith(an.replace('/*', '')) || f.path.endsWith(an.replace('*', f.name))));
            }));
        },
        async getFolders(path = '*') {
            return this.cache('get-folder-' + path, async () => {
                if (path === '*') {
                    try {
                        let files = await this.getDirList();
                        this.__items__ = this.__items__ || {};
                        return (files?.$DIR || []).map(i => {
                            let folder = this.__items__[i.name];
                            if (!folder) {
                                i.$owner = this;
                                folder = this.__items__[i.name] = new odaFolder({ id: i.id, name: i.name, $owner: this, accessLevel: i.access });
                                folder.config = i;
                            }
                            return folder;
                        })
                    }
                    catch (e) {
                        console.error(e)
                    }
                    return [];
                }
                else {
                    const steps = path.split('/');
                    path = steps.shift();
                    // todo переделать на маски
                    let folders = await this.getFolders();
                    if (path !== '*')
                        folders = folders.filter(i => i.name === path);
                    if (steps.length) {
                        path = steps.join('/');
                        folders = folders.map(async f => {
                            return (await f.getFolders(path)).flat();
                        });
                    }
                    return (await Promise.all(folders)).flat();
                }
            })
        },
        async getFiles(path = '*') {
            return this.cache('get-files-' + path, async () => {
                if (path === '*') {
                    try {
                        let files = await this.getDirList();
                        return (files.$FILE || []).map(i => {
                            i.$owner = this;
                            const file = new odaFile({ id: i.id, name: i.name, $owner: this, accessLevel: i.access });
                            file.config = i;
                            return file;
                        })
                    }
                    catch (e) {
                        console.error(e)
                    }
                    return [];
                } else {
                    const steps = path.split('/');
                    const name = steps.pop();
                    path = steps.join('/');
                    const folders = await this.getFolders(path)
                    let files = folders.map(async f => (await f.getFiles('*')).flat());
                    files = (await Promise.all(files)).flat();
                    if (name === '*') return files;
                    return files.filter(i => i.name === name)
                }
            })
        },
        async getFolder(path) {
            return await this._getFromPath(path, false);
        },
        async getFile(path) {
            return await this._getFromPath(path, true);
        },
        async _getFromPath(path, isFile) {
            const steps = path.split('/');
            const name = steps.pop();
            path = steps.join('/');
            const items = await this[isFile ? 'getFiles' : 'getFolders'](path ? path + '/*' : '*');
            return items.find(i => i.name === name);

        },
        saveFile(fileBody, path = '', inherit = false, mime) {
            let name = path.split('/').pop();
            switch (fileBody.constructor.name.toLowerCase()) {
                case 'string':
                    //if (ODANT.webMode) {
                    fileBody = new File([fileBody], name, { type: mime || "text/plain" });
                    //}
                    break;
                case 'array':
                case 'object':
                    //if (ODANT.webMode) {
                    fileBody = new File([JSON.stringify(fileBody)], name, { type: mime || "application/json" });
                    //}
                    break;
                case 'odafile': // todo <-- Еще смотреть как определять тип файла (MIME)
                    name = fileBody.id;
                    return fileBody.load().then(fileBody => {
                        return this.saveFile(fileBody, path || name);
                    });
                case 'file':
                case 'blob':
                case 'readstream':
                    break;
                default:
                    throw new Error('Unknown file type')
            }
            name = name || fileBody.name;
            const formData = new FormData();
            formData.append("file", fileBody, name);
            const url = inherit ? this.url : this.url.replaceAll('/~/', '/');
            return fetch(url + '/' + path, { method: 'POST', body: formData, credentials: 'include' }).then((res) => {
                this.reset?.();
                return this.getFile(path).then(file => {
                    file.$owner.fireChanged?.(file);
                    file.$owner.fire?.('create', file);
                    return file;
                });
            })
        }
    })
    odaServerItem({
        is: 'odaStructureItem',
        props: {
            config: {
                set(v) {
                    if (v) {
                        for (let p in v) {
                            const s = configMap[p];
                            if (s === undefined) continue;
                            this[s] = v[p];
                        }
                    }
                }
            },
            category() {
                return this.config.c || '';
            },
            structureListNames: [
                '$$classes',
                '$$bases',
                '$$parts',
                '$$workplaces',
                '$$modules',
            ],
            structureExcludes: ['P:CACHE']
        },
        async getConfig(params = { prefix: true, deep: 2, group: false, byType: false }) {
            return this.cache(`get-config:${JSON.stringify(params)}`, async () => {
                // try{
                const res = await this.request('get_config', params);
                this.config = res['$' + this.prefix][0];
                return this.config;
                // }
                // catch (e){
                //     console.error(e);
                // }
            })
        },
        async _getItems(prefix) {
            const config = (this.config && Object.keys(this.config).some(key => key.startsWith('$'))) ? this.config : await this.getConfig()
            return Promise.all((config['$' + prefix] || []).map(i => {
                return this.getItem(this.path + '/' + prefix + ':' + i.i);
            }));
        },
        async getNearestAccessibleItem() {
            if (this.access.level > 3) return this;
            await this.getConfig({ prefix: true, deep: 10, group: false, byType: false });
            const configRoot = this.config;
            let nodes = await this.getAccessibleNodes({ item: configRoot, level: 0 });
            if (!nodes || nodes.length <= 0) return null;

            let itemNode;
            if (nodes.length === 1) {
                itemNode = nodes[0];
            } else {
                const minLevel = nodes.reduce((res, n) => (res > n.level) ? n.level : res, Infinity) - 1;

                nodes = nodes.reduce((res, n) => {
                    const max = n.level - minLevel;
                    let owner = n.owner;
                    for (let i = 1; i < max; i++) {
                        owner = owner.owner;
                    }

                    res.add(owner);
                    return res;
                }, new Set());

                while (nodes.size > 1) {
                    nodes = Array.from(nodes).reduce((res, n) => {
                        if (n.item.owner) res.add(n.item.owner);

                        return res;
                    }, new Set());
                }

                itemNode = nodes?.values?.()?.next?.()?.value;
            }

            let path;
            while (itemNode) {
                path = `${itemNode.item.t?.[0] || 'H'}:${itemNode.item.i}/${path ? path : ''}`;
                itemNode = itemNode.owner;
            }

            return await this.$host.getItem(path || this.path);
        },
        async getAccessibleNodes(owner, items = []) {
            for (const key in owner.item) {
                if (!key.startsWith('$')) continue;

                const level = owner.level + 1;
                for (const item of owner.item[key]) {
                    const o = { item, owner, level };
                    if (item.a > 3 && !item.hide) {
                        items.push(o);
                    } else {
                        await this.getAccessibleNodes(o, items);
                    }
                }
            }

            return items;
        },
        get _$$structure() {
            return this._getStructure();
        },
        async _getStructure() {
            const items = (await Promise.all(this.structureListNames.map(async n => ((await this[n]) || [])))).flat();
            const groups = { other: [] };
            const map = { 'M': '$$Modules', 'W': '$$Workplaces', 'C': 'other', 'P': 'other', 'B': 'other' };
            const allowAdmin = this.access.level >= 6;
            items: for (const i of items) {
                if (!allowAdmin && (i.hidden || i.config.hidden)) continue;
                for (let ex of this.structureExcludes) {
                    const parts = ex.split(':');
                    ex = `${parts[0].replace('*', i.prefix)}:${parts[1].replace('*', i.name)}`;
                    if (ex === `${i.prefix}:${i.name}`) continue items;
                }
                const category = i.category ? i.category : map[i.prefix];
                if (!(category in groups)) groups[category] = [];
                groups[category].push(i);
            }
            const result = [];
            for (const g in groups) {
                if (g !== 'other') {
                    const group = new odaGroup();
                    group.$owner = this;
                    switch (g) {
                        case '$$Modules':
                            group.id = 'Modules';
                            group.type = 'module';
                            break;
                        case '$$Workplaces':
                            group.id = 'Workplaces';
                            group.type = 'workplace';
                            break;
                        default:
                            group.id = g;
                            group.type = 'category';
                    }
                    group._$$structure = groups[g];
                    result.push(group);
                } else {
                    result.push(...groups[g]);
                }
            }
            return result;
        }
    })
    odaStructureItem({
        is: 'odaHost',
        props: {
            typeName: 'host',
            origin: String,
            $host() {
                return this;
            },
            $authHost: Object,
            $user: Object,
            prefix: 'H',
            icon: 'odant:host',
            defaultViewName: 'desktop-ext',
            uid: String,
            actionsPath: 'web/core/structure/client/*/*',
            // TODO удалить, после появления серверного кэша
            $$hosts: [],
            supportUrl() {
                const rootPart = (this.config?.$P || this.config?.$D)?.find(n => n.i === 'ROOT');
                if (rootPart && Object.keys(rootPart).find(k => k === '$M')) return this.url;
                return `${this.origin}/api/H:${this.config.s}`;
            },
        },
        async created() {
            //await this.getConfig();
            if (!SERVER_MODE) {
                try {
                    this.uid = (await this.request('check_ssid')).result; // todo убрать result
                }
                catch (e) {
                    const security = JSON.parse(localStorage.getItem('security'));
                    if (!security || security.login === 'GUEST') return;
                    if (security.login && security.ticket) {
                        const ts = JSON.stringify(security.ticket);
                        try {
                            const response = await this.request('authorize', {}, ts)
                            security.ssid = response.result;
                            // return this.loginIn();
                        }
                        catch (e) {
                            // ODANT.push(e);
                            localStorage.removeItem('security');
                        }
                        // this.request('check_ssid').then(res => {
                        //     this.uid = res?.result;
                        //     this.$user = new odaUser({
                        //         $owner: this,
                        //         id: this.uid || 'GUEST',
                        //         root: userRoot || {}
                        //     })
                        // })
                        // this.uid = (await this.request('check_ssid')).result; //todo убрать result
                    }
                }
                // const response = await ODANT.API.request(ODANT.origin + '/api', { method: 'find_account', login: login, type: type });
                const find_account_res = await this.request('find_account');

                // todo:
                // try {
                //     const check_ssid_res = await this.request('check_ssid');
                //     const uid = check_ssid_res?.result;
                //     const userRoot_res = await fetch(`${this.$authHost.url}/U:${uid}`);
                //     const userRoot = await userRoot_res.json();
                //     this.$user = new odaUser({
                //         $owner: this,
                //         id: this.uid,
                //         root: userRoot
                //     });
                // } catch (err) {
                //     console.error(err);
                //     this.$user = new odaUser({
                //         $owner: this,
                //         id: 'GUEST',
                //     });
                // }
            }
            await this.getConfig();
        },
        get _$$support() {
            return this.cache('_$$support', async () => {
                const support = [];
                const item = new odaMarketHost({ id: 'ROOT', $owner: this });

                let hostId = this.config.s;
                while (hostId && hostId !== '000000000000000') {
                    const h = await this.getItem(`H:${hostId}`);
                    //const h = new odaHost({ id: hostId, origin: item.$host.origin });
                    await h.getConfig();

                    const root = new odaMarketHost({ id: 'ROOT', $owner: h });
                    support.unshift(root);

                    hostId = h.config.s;
                }
                if (item) {
                    support.push(item);
                }
                return support;
            });
        },
        get $$parts() {
            return this._getItems('P');
        },
        get EventSource() {
            return this.cache('EventSource', () => {
                const parser = new URL(this.url);

                let url;
                url = (parser.protocol === 'https:')
                    ? 'wss://'
                    : 'ws://';

                url += parser.hostname;

                if (parser?.port?.length > 0) {
                    url += ':' + parser.port;
                }

                url += '/ws';
                return new WebSocketEvents(url);
            })
        },
        getFullId(item) {
            let s = '';
            let own = item;
            let pref = '';
            while (own) {
                let step = own.id || null;
                if (step && own.prefix) {
                    let convertedPrefix = own.prefix;
                    switch (convertedPrefix) {
                        case 'B':
                        case 'M':
                        case 'P':
                        case 'S':
                        case 'W':
                            convertedPrefix = 'D';
                            break;
                    }
                    step = convertedPrefix + ':' + step;
                }
                if (step) {
                    step = '/' + step;
                    if (!own.prefix || own.prefix !== pref) {
                        s = step + s;
                    }
                }
                pref = own.prefix;
                own = own.$owner;
            }
            return s;
        },
        addListen(item, callback) {
            this.EventSource.addEventListener(this.getFullId(item), callback, false);
        },
        remListen(item, callback) {
            this.EventSource.removeEventListener(this.getFullId(item), callback);
        },
        async updateAuthorization() {
            const security = JSON.parse(localStorage.getItem('security') || '{}');
            const login = security.login;
            const body = security.code || JSON.stringify(security.ticket);
            if (body) {
                try {
                    const response = await fetch('/api?method=authorize', { method: 'POST', body });
                    if (response.ok) {
                        const id = (await response.json()).result;
                        // const authHostUrl_res  = await fetch(`/api?method=find_account&login=${login}`);
                        // const authHostUrl_json = await authHostUrl_res.json();
                        // const authHostUrl      = authHostUrl_json.result;
                        // const root             = await fetch(`${authHostUrl}/U:${id}`);
                        this.$user = new odaUser({ $owner: this, name: login, id, /* root */ });

                        setInterval(() => {
                            fetch('/api?method=authorize', { method: 'POST', body });
                        }, 15 * 60 * 1000);
                        return;
                    } else {
                        throw new Error(`Error on authorize: ${await response.text()}`);
                    }
                } catch (err) {
                    console.warn(err);
                }
            }
            this.$user = new odaUser({ $owner: this, name: 'GUEST' });
        },
        async logout() {
            await fetch('/api?method=logout');
            localStorage.removeItem('security');
            location.reload();
        }
    })
    odaStructureItem({
        is: 'odaClass',
        extends: 'odaBodyOwner, odaIndexesOwner',
        props: {
            $FIELDS() {
                return new odaExtFields({ $owner: this });
            },
            $STATIC() {
                return new odaStatic({ $owner: this });
            },
            $DETAILS() {
                return this.$base.$STATIC;
            },
            isAbstract: {
                type: Boolean,
                get() {
                    return getBool(this.config?.abstract);
                },
                set(v) {
                    this.setBoolAttr('Abstract', v);
                }
            },
            rootTag: '$CLASS',
            $class() {
                return this;
            },
            icon() {
                if (this.config?.icon) {
                    if (this.config.icon.includes(':'))
                        return this.config.icon;
                    return this.config.icon;
                }
                if (/*(await this.isAbstract) || */this.hasChildren) { // TODO isAbstract лезет в рут который получается асинхронно, иконка в таком случае async (продумать)
                    return 'odant:abstract-class';
                }
                return 'odant:class';
            },
            prefix: 'C',
            typeName: 'class',
            defaultViewName: 'grid-view',
            // defaultViewName: {
            //     //category: 'Views',
            //     get() {
            //         if (this.isAbstract) {
            //             if (this.access.allowAdmin)
            //                 return 'configurator';
            //             return 'qr-code';
            //         }
            //         return this.getAttr('default-class-view', 'qr-code');//'grid');
            //     },
            //     set(v) {
            //         this.setAttr('default-class-view', v);
            //     }
            // },
            defaultObjectView: {
                //category: 'Views',
                get() {
                    return this.getAttr('default-object-view', 'editor');
                },
                set(v) {
                    this.setAttr('default-object-view', v);
                }
            },
            zIndex: 10,
            $$fields() {
                return [this.$FIELDS, this.$STATIC, this.$DETAILS];
            }
        },
        created() {
            this.eventHandler_ = this.eventHandler.bind(this);
            this.$host.addListen(this, this.eventHandler_);
        },
        get ACTIONS() {
            return this.getRootElement('ACTIONS');
        },

        get _$$config() {
            return (async () => {
                const actions = await this.$$actions;
                return [
                    ...(await this._$$fields),
                    {
                        _$$config: actions,
                        label: 'ACTIONS',
                        name: 'ACTIONS',
                        typeName: 'category',
                        icon: 'actions',
                        ItemCount: actions.length,
                        hasChildren: actions.length > 0
                    }
                ];
            })();
        },
        get $$classes() {
            return this._getItems('C');
        },
        get $$packets() {
            return (async () => {
                const json = await this.request('get_pack_list');
                return json.$FOLDER[0].$FILE ? json.$FOLDER[0].$FILE.reduce((res, v) => {
                    const names = v.Name.split('#');
                    const name = names.shift();
                    let p = res.find(i => i.name === name);
                    if (!p) {
                        p = { name: name, count: 0, items: [] };
                        res.push(p);
                    }
                    p.count += +v.Count;
                    if (names.length)
                        p.items.push({ count: v.Count, names: names });
                    return res;
                }, []).map(i => {
                    return new odaPack({ $owner: this, name: i.name, root: i });
                }) : [];
            })();
        },
        get count() {
            return this.cache('count', async () => {
                return await this.getObjectCount();
            });
        },
        // TODO super
        // save(body) {
        //     body = body || this.body || '{"$CLASS":[' + JSON.stringify(this.ROOT) + ']}';
        //     return super.save(body);
        // },
        getRootElement(name) {
            return this.root?.[`$${name}`]?.[0];
        },
        async getObjectCount(pack) {
            const json = await this.request('get_class_objects_count', { format: 'json', mask: pack || '*' });
            return json && parseInt(json.result) || 0;
        },
        async createObject(src) {
            if (src && typeof src === 'object')
                src = JSON.stringify(src);
            const json = await this.request('create_object', { recalc: true, meta: true }, src);
            // TODO load extended object script
            const root = json?.$OBJECT?.[0];
            const obj = new odaObject({ id: root?.oid, body: json, root: root, $owner: this });
            obj.currentState = 'new object';
            this.fire('create', obj);
            return obj;

        },
        getObject(id, params = { meta: true, recalc: false }) {
            return this.cache(id, async () => {
                let cls = this;
                let url = `${this.url}/O:${id}`;
                if (params?.length) {
                    Object.keys(params).reduce((url, p, i) => {
                        url += i === 0 ? '?' : '&';
                        url += `${p}=${params[p]}`;
                        return url;
                    }, url);
                }
                const body = await (await fetch(url)).json();
                const root = (body.$OBJECT || body.$STATIC || body.$DETAILS)[0];
                if (this.id !== root.cid) {
                    try {
                        cls = (await this.findItem(root.cid)) || this;
                    }
                    catch (e) {
                        cls = this;
                    }
                }
                // TODO load extended object script
                return new odaObject({ id, body, root, $owner: cls });
            })
        },
        async createClass({ name, label, root = {} }) {
            if (name) root.Name = name;
            if (label) root.Label = label;
            const body = { $CLASS: [root] };
            const res = await this.request('create_class', undefined, body);
            // const cfg = await this.$base.getConfig();
            // this.$base.$set('config', cfg);
            // this.reset();
            const cid = res.result.slice(res.result.lastIndexOf(':') + 1);
            const cls = await this.findItem(`C:${cid}`);
            this.fire('create', cls);
            return cls;

        },
        getInheritRoot() {
            return {
                ClassId: this.id,
                Name: this.name,
                Label: this.label,
                Parent: this.id
            };
        },
        async inheritClass(inheritItem) {
            const item = await this.createClass({ name: inheritItem.name, root: inheritItem.getInheritRoot() });
            const classes = await inheritItem.$$classes;
            for (const child of classes) await item.inheritClass(child);
            item.save();
            return item;
        },
        async _recalcObject(obj, metadata) {
            if (obj instanceof odaObject) {
                obj = obj.body;
            }
            // Temporarily. To delete this and  the "async" function's prefix when it will by realized on the server-side: the object recalculating for workplace's class
            if (this.$owner instanceof odaWorkPlace) {
                const stat = await (await fetch(`/api${this.path}/O:${this.id}`)).json();
                Object.assign(obj, stat);
            }
            ///////////////
            return this.request('recalc', { meta: metadata }, JSON.stringify(obj));
        },
        async eventHandler(event) {
            if (event.type === 'object') {
                if (event.$additional) {
                    this.reset('objectCount');
                    event.$additional.forEach(u => {
                        if (u.$E) {
                            u.$E.forEach(e => {
                                if (e.$P) {
                                    e.$P.forEach(async (p) => {
                                        if (p.$U) {
                                            this.fire('update', this);
                                            // let list = this._objects.filter(i => {
                                            //     return p.$U.find(p => {
                                            //         return p.o === i.id
                                            //     });
                                            // });
                                            // if (list.length) {
                                            //     list = list.map(i => {
                                            //         return i.reload();
                                            //     });
                                            //     await Promise.all(list);
                                            //     this.fire('update', this);
                                            // }
                                            // list = p.$U.filter(p => {
                                            //     return !this._objects.some(o => o.id === p.o);
                                            // });
                                            // if (list.length) {
                                            //     this.fire('create-objects', list.map(p => p.o));
                                            // }
                                        }
                                        if (p.$D) {
                                            this.fire('delete-objects', p.$D);
                                        }
                                    })
                                }
                            })
                        }
                        if (u.$C) {
                            u.$C.forEach(async (c) => {
                                const cls = await this.findItem('C:' + c.i);
                                // let fid = cls.path;
                                // if (!this.config.$C)
                                //     this.config.$C = [];
                                // this.config.$C.push(c);
                                // this.reset('Children Config Classes');
                                // cls.reset();
                                // fid = cls.fulpathlId;
                                this.fire('create', cls, true);
                            })
                        }
                    });
                    this.fireChanged();
                }
            }
            else if (event.type === 'class') {
                switch (event.operation) {
                    case 'delete': {
                        // this.$owner.reset();
                        this.fire('deleted', this);
                    } break;
                    case 'update': {
                        await this.load();
                    } break;
                    case 'create': {
                        // this.reset();
                        this.fire('update');
                    } break;
                    case 'pack create': {
                        this.fire('pack-create', event.$additional);
                    } break;
                    case 'pack delete': {
                        this.fire('pack-delete', event.$additional);
                    } break;
                }
            }
        }
    })
    odaServerItem({
        is: 'odaPack',
        props: {
            icon: 'av:library-books',
            async root() {
                return await this.body;
            },
            mask() {
                const prefix = this.$owner.mask;
                return (prefix ? prefix + '/' : '') + this.name;
            }
        },
        get hasPacks() {
            return this.$$packets.length > 0;
        },
        get $$packets() {
            return this.cache('packets', () => {
                return this.root.items.reduce((res, v) => {
                    const names = v.names;
                    const name = names.shift();
                    let p = res.find(i => i.name === name);
                    if (!p) {
                        p = { name: name, count: 0, items: [] };
                        res.push(p);
                    }
                    p.count += +v.count;
                    if (names.length)
                        p.items.push({ count: v.count, names: names });
                    return res;
                }, []).map(i => {
                    return new odaPack({ $owner: this, name: i.name, root: i })
                });
            });
        }
    })
    odaServerItem({
        is: 'odaGroup',
        props: {
            id() {
                return this.$owner?.id + '_G_' + this.name;
            },
            // access() {
            //     return this.$owner?.access;
            // },
            icon: 'odant:folder',
            prefix: 'G',
            type: 'CATEGORY',
            $owner: String,
            fill() {
                switch (this.type?.toLowerCase()) {
                    case 'module': return 'magenta';
                    case 'workplace': return 'darkblue';
                    case 'group':
                    case 'category': return 'orange';
                    default: return 'black'
                }
            },
            isRef: false,
            zIndex: 0,
            typeName: 'group'
        },
        get hasChildren() {
            return true;
        },
        async getItem(path) {
            return await this.$owner?.getItem(path);
        },
        async findItem(path, params) {
            return await this.$owner?.findItem(path, params);
        }
    })
    odaGroup({
        is: 'odaModuleGroup',
        props: {
            typeName: "modules",
            prefix: 'M',
            icon: 'folders/folder-magenta-open',
            zIndex: 30
        }
    })
    odaClass({
        is: 'odaDomain',
        props: {
            icon() {
                return this.isSystem ? 'icons:lock' : 'odant:domain';
            },
            prefix: 'D',
            typeName: 'domain',
            isSystem() {
                return this.id === '000000000000000';
            },
            _color: 'var(--domain-color)',
            defaultViewName: 'desktop-ext',
            // defaultViewName: {
            //     //category: 'Views',
            //     get() {
            //         return this.getAttr('default-class-view', 'desktop');
            //     },
            //     set(v) {
            //         this.setAttr('default-class-view', v);
            //     }
            // },
            zIndex: 60,
        },
        get count() {
            return '';
        },
        get $$modules() {
            return this._getItems('M');
        },
        get $$workplaces() {
            return this._getItems('W');
        },

        async createDomain({ name, label, prefix, root = {} }) {
            if (name) root.Name = name;
            if (label) root.Label = label;
            const body = { $CLASS: [root] };
            let type;
            prefix = (prefix || '').toUpperCase();
            switch (prefix) {
                case 'B': {
                    type = 'BASE';
                } break;
                case 'M': {
                    type = 'MODULE';
                } break;
                case 'P': {
                    type = 'PART';
                } break;
                case 'W': {
                    type = 'WORKPLACE';
                } break;
            }

            const res = await this.request('create_domain', { type: type, prefix: true }, body);
            // this.reset();
            // const cfg = await this.getConfig();
            // this.$set('config', cfg);
            const domain = await this.findItem(`${prefix}:${res.i}`);
            this.fire('update');
            this.fire('create', domain);
            return domain;

        },
        getInheritRoot() {
            return {
                ClassId: this.id,
                Name: this.name,
                Label: this.label,
                Type: this.type
            };
        },
        async inheritDomain(inheritItem) {
            const item = await this.createDomain({ name: inheritItem.name, prefix: inheritItem.prefix, root: inheritItem.getInheritRoot() });
            const classes = await inheritItem.$$classes;
            for (const child of classes) await item.inheritClass(child);

            this.fire('update');
            return item;
        }
    })
    odaDomain({
        is: 'odaBase',
        props: {
            icon() {
                if (this.type === 'USER') {
                    //TODO: получить объект пользователя, получить от него аватар
                }
                if (this.config?.icon) {
                    if (this.config.icon.includes(':'))
                        return this.config.icon;
                    return this.config.icon;
                }
                if (this.config?.$B)
                    return 'odant:sub-base';
                return 'odant:base';
            },
            prefix: 'B',
            $base() {
                return this;
            },
            typeName: 'base',
            _color: 'var(--base-color)',
            $module() {
                return undefined;
            },
            zIndex: 70,
        },
        created() {
            this.eventHandler_ = this.eventHandler.bind(this);
            this.$host.addListen(this, this.eventHandler_);
        },
        get $$bases() {
            return this._getItems('B');
        },
        async getActionsExp($item) {
            return this.cache(`getActionsExp-${$item.typeName}`, async () => {
                const modules = await this.$$modules;
                let actions = await Promise.all(modules.map((module) => {
                    return module.getActionsExp($item);
                }));
                actions = actions?.flat();
                // console.dir(actions);
                return actions;
            });
        },
        get count() {
            return this.cache('baseCount', async () => {
                if (!this.config?.$B && this.hasChildren) await this.getConfig();
                return this.config.$B?.length || 0;
            })
        },
        eventHandler(event) {
            // TODO CORE.webMode
            // if (CORE.webMode) {
            // const getText = obj => {
            //     return obj.value.split('\n').map(s => {
            //         try {
            //             if (s.includes('(')) {
            //                 const path = s.split('(')[1].split(')')[0];

            //                 let url = new URL(path);
            //                 if (url.protocol === 'file:')
            //                     url = url.pathname.substring(url.pathname.indexOf('/web/'));
            //                 else
            //                     url = url.pathname;

            //                 url = ODANT.origin + url;
            //                 return s.replace(path, url);
            //             }
            //         }
            //         catch (e) {

            //         }
            //         if (s.toLowerCase().includes('error:')) {
            //             s = s.replace(/https?\:\/\/127.0.0.1\:\d{1,5}/gi, ODANT.origin);
            //         }
            //         return s;
            //     }).join('\n');
            // };
            switch (event.operation) {
                case 'delete':
                    // this.$owner.reset();
                    this.fire('deleted', this);
                    break;
                case 'update':
                    //this.load();
                    this.fire('update');
                    break;
                case 'create':
                    // this.reset();
                    this.fire('update');
                    break;
                case 'jscript':
                    event.$additional.forEach(i => {
                        if (i.value) {
                            // const text = getText(i);
                            console.log('%coda-server', 'font: bold; color: white; background-color:blue; padding: 1px 8px;', i.value);
                        }
                    });
                    break;
                case 'jscript-log':
                    event.$additional.forEach(i => {
                        if (i.value) {
                            // const text = getText(i);
                            const color = i.toString().includes('Error') ? 'red' : 'silver';
                            console.error('%coda-server',
                                `font: bold; color: white; background-color: ${color}; padding: 1px 8px;`, i.value);

                        }
                    });
                    break;
                case 'jscript-warn':
                    event.$additional.forEach(i => {
                        if (i.value) {
                            // const text = getText(i);
                            console.warn('%coda-server', 'font: bold; color: white; background-color:orange; padding: 1px 8px;', i.value);
                        }
                    });
                    break;
                case 'jscript-error':
                    event.$additional.forEach(i => {
                        if (i.value) {
                            // const text = getText(i);
                            console.error('%coda-server', 'font: bold; color: white; background-color:red; padding: 1px 8px;', i.value);
                        }
                    });
                    break;
            }
            // }
        }
    })
    odaBase({
        is: 'odaPart',
        props: {
            $part() {
                return this;
            },
            icon() {
                switch (this.id) {
                    case 'ROOT':
                        return 'odant:root';
                    case 'WORK':
                        return 'odant:structure';
                }
                return this.id.toLowerCase();
            },
            prefix: 'P',
            typeName: 'part',
        },
        get count() {
            return this.cache('count', async () => {
                if (!this.config) await this.getConfig();
                return Object.keys(this.config).reduce((prev, curr) => {
                    if (curr.startsWith('$')) prev += this.config[curr].length;
                    return prev;
                }, 0);
            })
        },

    })
    odaDomain({
        is: 'odaModule',
        props: {
            icon: 'odant:module',
            prefix: 'M',
            typeName: 'module',
            _color: 'var(--module-color)',
            defaultViewName: 'module-info',
            $module() {
                return this;
            },
            zIndex: 30,
        },
        async getActionsExp($item) {
            return this.cache(`getActionsExp-${$item.typeName}`, async () => {
                const classes = await this.$$classes;
                let actions = await Promise.all(classes.map((cls) => {
                    // return cls.getActions($item);
                    return cls.getActionsExp($item);
                }))
                actions = actions?.flat();
                console.dir(actions);
                return actions;
            });
        }
    })
    odaDomain({
        is: 'odaWorkPlace',
        props: {
            icon: 'odant:workplace',
            prefix: 'W',
            typeName: 'work place',
            _color: 'var(--workplace-color)',
            zIndex: 50,
        },
        async createLink(cls) {
            const classId = cls.id;
            const res = await this.request('create_link', { classId });
            //this.reset();
        }
    })
    odaServerItem({
        is: 'odaFile',
        props: {
            id() {
                return this.name;
            },
            icon() {
                switch (this.ext) {
                    case 'jpg':
                    case 'png':
                        return this.url + '?mode=preview64';
                }
                return 'files:' + this.ext;
            },
            ext() {
                return this.id.split('.').pop();
            },
            typeName: 'file',
            zIndex: 102,
            actionsPath() {
                return `${this.name}/~/client/*/*`;
            },
            defaultViewName() {
                switch (this.ext) {
                    case 'jpg':
                    case 'png':
                        return 'image-preview';
                    case 'pdf':
                        return '';
                }
                return 'code-editor';
            },
            inherit() {
                const root = this.root || this.config || {};
                let level = root.inherit || 0;
                return new odaInherit({ name: level });
            }
        },
        get time() {
            return this.config?.time;
        },
        get type() {
            return this.typeName;
        },
        get size() {
            return this.config?.size;
        },
        get sizeText() {
            if (this.size && this.size > 0) {
                let v = this.size;
                let pcs;
                if (v < 1000) {
                    pcs = ' b';
                }
                else if (v < 1000000) {
                    v = Math.round(v / 10) / 100;
                    pcs = ' Kb'
                }
                else if (v < 1000000000) {
                    v = Math.round(v / 10000) / 100;
                    pcs = ' Mb'
                }
                else if (v < 1000000000000) {
                    v = Math.round(v / 10000000) / 100;
                    pcs = ' Gb'
                }
                return v.toLocaleString() + pcs;
            } else {
                return '0 b';
            }
        },
        get count() {
            return this.cache('count', () => {
                return this.sizeText;
            })
        },
    })
    odaFile.create = function (props){

    }
    odaFile({
        is: 'odaFolder',
        props: {
            icon: 'odant:folder',
            typeName: 'folder',
            label() {
                return this.name === '~' ? this.$owner.label + ' [~]' : this.name;
            },
            fill() {
                if (!this.inherit.name) return 'var(--folder-color)';
                return this.color;
            },
            zIndex: 101,
            defaultViewName: 'file-explorer',
            $folder() {
                return this;
            }
        },
        get count() {
            return this.cache('count', () => {
                if (this.config) {
                    return this.config.$FILE?.length || 0;
                }
            })
        },
        get hasChildren(){
            if(this.config){
                return this.config.$FILE?.length > 0 || this.config.$DIR?.length > 0;
            }
            return false;
        }
    })
    odaFolder.create = function (props){

    }
    odaServerItem({
        is: 'odaIndex',
        extends: 'odaIndexesOwner',
        props: {
            icon: {
                default: 'odant:index',
                get() {
                    switch (this.type) {
                        case 'Union': return 'odant:index';
                        case 'Cascade': return 'odant:index';
                        default: return 'odant:index';
                    }
                }
            },
            prefix: 'I',
            typeName: 'index',
            type: String,
            root: Object,
            METADATA() {
                return this.root?.$METADATA?.[0] || this.$class.METADATA;
            }
        },
        get $$fields() {
            //todo переделать на собственные метаданные
            return this.$class.$FIELDS.$$fields;
        },
        created() {
            this.$host.addListen(this, () => this.fire('changed'));
        },
        getRows(mask, from, length, post) {
            if (typeof post === 'object')
                post = JSON.stringify(post);
            return this.request('dataset', { loadmask: mask, mask: mask, from: from, length: length }, post);
        },
        getDataSet(mask = '*', filter, clear) {
            if (clear) this.reset();
            return this.cache('DataSet-' + mask + filter, () => {
                return new IndexDataSet(this, mask, filter);
            }, 100000) //todo Предложение по сбросу кэша
        }
    })
    odaServerItem({
        is: 'odaObject',
        extends: 'odaBodyOwner, odaIndexesOwner',
        props: {
            rootTag: '$OBJECT',
            icon: 'odant:object',
            prefix: 'O',
            typeName: 'object',
            $$fields() {
                switch (this.id) {
                    case this.$base.id:
                        return this.$class.$DETAILS.$$fields;
                    case this.$class.id:
                        return this.$class.$STATIC.$$fields;
                    default:
                        return this.$class.$FIELDS.$$fields;
                }
            },
            defaultViewName() {
                return this.$class.defaultObjectView;
            },
            async root() {
                return (this.body.$OBJECT || this.body.$STATIC || this.body.$DETAIL || [{}])[0];
            }

        },
        onbody() {
            //this.ACTIONS = this.getRootElement('ACTIONS');
        },
        async getActions() {
            return this.$class.getActions(this);
        },
        recalc() {
            this.debounce('recalc', async () => {
                if (this._stopRecalc) {
                    this._stopRecalc = false;
                    return;
                }
                if (this.readOnly) return;
                try {
                    this.body = await this.$class._recalcObject(this, true);
                }
                catch (e) {
                    this.error = e;
                    console.error(e);
                }
            }, 300);
        },
        getValue(field, root, def) {
            if (field.name) {
                if (!root) {
                    const path = field.path.split('/');
                    root = this.Root;
                    while (root && path.length > 1) {
                        const step = '$' + path.shift();
                        let array = root[step];
                        if (array && array.length)
                            root = array[0];
                        else
                            root = null;
                    }
                    if (!root)
                        return null;
                }
                if (field.isTable)
                    return root['$' + field.name];
                field = field.name;
            }
            else
                root = root || this.Root;

            let path = field.split('/');
            while (path.length > 0 && root) {
                field = path[0].replace('@', '');
                if (path.length > 1) {
                    root = root['$' + field];
                    root = root && root[0];
                }
                path.splice(0, 1);
            }
            const value = root && root[field];
            //if (!field.isLink) return value;

            let link = JSON.selectSingleNode(root, field + '/@link'); //root && root['$'+field] && root['$'+field].length>0 && (root['$'+field][0].link || root['$'+field][0].file);
            if (link) return this.findItem(link);
            // if (value) return this.restoreLink(field, root, value);
            return value;
        },
        async setValue(field, value = null, root) {
            if (typeof Blob !== 'undefined' && value instanceof Blob) {
                const ext = (value.name.includes('.') && '.' + value.name.split('.').pop()) || '';
                const file = await this.saveFile(ODANT.webMode ? value : (value.stream || value.buffer), ODANT.getGuid() + ext);
                file.label = value.name;
                const res = await this.setValue(field, file, root);
                await this.save();
                return res;
            }
            let name = field;
            if (field.name) {
                const path = field.path.split('/');
                if (!root) {
                    root = this.Root;
                    while (path.length > 1) {
                        const step = '$' + path.shift();
                        let array = root[step];
                        if (!array)
                            root[step] = array = [{}];
                        else if (!array.length)
                            array.push({});
                        root = array[0];
                    }
                }
                name = field.name;
            }
            root = root || this.Root;

            if (!value) {
                delete root[name];
                this.clearRows(field, root);
            }
            else if (typeof value === 'object') {
                this._stopRecalc = true;
                const row = {};
                if (value instanceof odaItem) {
                    this._stopRecalc = true;
                    if (value instanceof odaObject) {
                        if (!field || !field.isLink) Object.assign(row, value.Root)
                        else if (!this._hasOwnersFieldsIsLink(field)) await field.fillLinkRoot(value, row);
                    }
                    row.link = this.getRelativeId(value);
                }
                else if (typeof value === 'object') {
                    CORE.mixin(value, row);
                }
                this._stopRecalc = false;
                this.clearRows(field, root);
                root[name] = String(value && (value.label || value.name || value.value || value));
                Object.assign(this.addRow(field, root), row);
            }
            else {
                root[name] = String(value);
            }
            // if (before !== JSON.stringify(root))
            this.recalc();
            return value
        },
        _hasOwnersFieldsIsLink(field) {
            const owner = field.$owner;
            if (owner instanceof odaField && !(owner instanceof odaExtFields)) {
                return owner.isLink || this._hasOwnersFieldsIsLink(owner);
            }
            return false;
        },
        getMetadata(field, attr, root = this.root, idx) {
            field = field.name || field;
            try {
                root = root['$meta:data'][0];
                root = root['$a:' + field][0];
                root = root['$meta:' + attr];
                if (idx !== undefined)
                    root = root[idx].value;
                return root;
            }
            catch (err) {
                if (idx === undefined)
                    return [];
                return null;
            }
        },
        clearRows(field, root = this.Root) {
            field = field.name || field;
            if (root['$' + field]) {
                delete root['$' + field];
                this.recalc();
            }

        },
        addRow(field, root) {
            let name = field;
            if (field.name) {
                const path = field.path.split('/');
                if (!root) {
                    root = this.Root;
                    while (path.length > 1) {
                        const step = '$' + path.shift();
                        let array = root[step];
                        if (!array)
                            root[step] = array = [{}];
                        else if (!array.length)
                            array.push({});
                        root = array[0];
                    }
                }
                name = field.name;
            }
            root = root || this.Root;
            root = root['$' + name] = root['$' + name] || [];
            let n = {};
            root.push(n);
            if (field.isTable) {
                let max = root.reduce((res, i) => {
                    if (+i.id > res)
                        res = +i.id;
                    return res;
                }, 0);
                root.filter(i => (!i.id)).map(i => {
                    i.id = ++max;
                });
            }
            this.recalc();
            return n;
        },
        delRow(name, row, root = this.Root) {
            if (name && row) {
                let arr = root['$' + name];

                if (!arr || !arr.length) {
                    return;
                }

                let index = row;

                if (typeof row !== 'number') {
                    index = arr.indexOf(row);
                }
                if (index > -1) {
                    let data = arr[index];
                    arr.splice(index, 1);
                    this.recalc();
                    return { index: index, value: data };
                }
            }
        }
    })
    odaBase({
        is: 'odaUser',
        props: {
            prefix: 'U',
            icon() {
                return 'social:person';
                // return this.isNotGuest
                //     ? 'social:person'
                //     : this.$host.$authHost.request('get_user_icon', {id: this.id});
            },
            isNotGuest() {
                return this.name !== 'GUEST';
            },
            fill: 'white',
            async logout() {
                return this.$host.logout();
            },
            async changePass() {
                const authHostUrl_res = await fetch(`/api?method=find_account&login=${this.name}`);
                const authHostUrl_json = await authHostUrl_res.json();
                const authHostUrl = authHostUrl_json.result;
                const authHostOrigin = (new URL(authHostUrl)).origin;
                location.assign(`${authHostOrigin}/web/auth/pap/change-pass.html?login=${this.name}&redirect_url=${location}`);
            }
        }
    })
    odaServerItem({
        is: 'odaAction',
        props: {
            active() {
                return !this.export;
            },
            icon: 'odant:action',
            typeName: 'action',
            $source: Object,
            viewName() {
                return `odant-${this.name.toLowerCase()}`;
            },
            root() {
                const metaRoot = this.$class?.$module
                    ? this.$class?.root?.$MODULES?.[0].$MODULE?.find(e => e.id === this.$class?.$module?.id)
                    : this.$class?.root?.$ACTIONS?.[0];
                return metaRoot?.$ACTION?.find(e => e.id === this.id);
            }
        },
        execute() {
            throw new Error(`Action "${this.label}" not implemented!`)
        },
        getActions() {
            return [];
        }
    })
    odaAction({
        is: 'odaView',
        props: {
            typeName: 'view',
            hideToolbar: false,
            url() {
                return this.path;
            }
        },
        async execute(parent) {
            while (parent && !parent.showView) {
                parent = parent.parent || parent.domHost || parent.parentElement;
            }
            if (parent) {
                parent.showView(this);
            }
            else {
                window.open(this.$owner.url + '/~/client/pages/form/index2.html?view=' + this.id);
            }
        },
        get _control() {
            return ODANT.createComponent(this.viewName);
        },
        async createComponent(component) {
            await import(`${this.url}/${this.name}.js`);
            const control = await ODA.createComponent(component || this.viewName);
            control.contextItem = this.$owner;
            control.sourceItem = this.$source;
            return control;
        }
    })
    odaAction({
        is: 'odaActionError',
        props: {
            icon: 'icons:error'
        },
        execute() {
            window.open(this.url + '/~/client/app-studio/index2.html');
        }
    })
    odaAction({
        is: 'odaPage',
        props: {
            typeName: 'page'
        },
        execute() {
            window.open(this.$owner.url + '/~/client/pages/' + this.name + '/index2.html');
            // console.warn('page not defined!!!');
        }
    })
    odaAction({
        is: 'odaMethod',
        props: {
            typeName: 'method'
        },
        execute() {
            console.warn('method not defined!!!');
        }
    })
    odaMethod({
        is: 'odaButton',
        props: {
            typeName: 'button'
        },
        execute() {
            console.warn('button not defined!!!');
        }
    })
    odaMethod({
        is: 'odaService',
        props: {
            typeName: 'service'
        },
        execute() {
            console.warn('service not defined!!!');
        }
    })
    odaVirtualItem({
        is: 'odaField',
        props: {
            path() {
                if (this.$owner instanceof odaField)
                    return this.$owner.path + '/' + this.name;
                return this.name;
            },
            place: String,
            _input: String,
            _complex: String,
            $folder() {
                return (this.$typeClass || this.$class).$folder
            },
            typeName: 'field',
            icon() {
                if (this.$typeClass?.icon !== 'odant:class' && this.$typeClass?.icon !== 'odant:abstract-class')
                    return this.$typeClass.icon;
                if (this.hasFields)
                    return 'odant:fields';
                return 'odant:field';
            },
            root: Object,
            $typeClass: Object,
            actionsPath() {
                return 'fields/*/*';
            },
            isTable() {
                return this.getBoolAttr('List');
            },
            hasFields() {
                return ((this.$typeClass?.isAbstract && this.$typeClass?.hasFields) || (this.root?.$ATTR?.length > 0));
            }
        },
        get hasChildren() {
            return this.hasFields;
        },
        get input() {
            return this.cache('input', async () => {
                let input = this._input;
                if (input) {
                    if (!input.startsWith('field-')) {
                        input = 'field-' + input;
                    }
                    let suffix = this.$typeClass?.id || '';
                    suffix = suffix.substring(suffix.length - 4);
                    input = await FIELD.regComponent(input, suffix);
                }
                return input;
            })
        },
        get complex() {
            return this.cache('complex', async () => {
                let complex = this._complex;
                if (complex) {
                    if (!complex.startsWith('field-')) {
                        complex = 'field-' + complex;
                    }
                    let suffix = this.$typeClass?.id || '';
                    suffix = suffix.substring(suffix.length - 4);
                    complex = await FIELD.regComponent(complex, suffix);
                }
                return complex;
            })
        },
        get $$fields() {
            return this.cache('$$fields', async () => {
                await this.$class.body;
                const attrs = [];
                if (attrs.length === 0 && this.root?.$ATTR) {
                    attrs.push(...this.root.$ATTR);
                }
                if (this.$typeClass) {
                    await this.$typeClass.body;
                    (this.$typeClass.$FIELDS?.root?.$ATTR || []).forEach(a => {
                        const _a = attrs.find(_a => _a.Name === a.Name);
                        if (_a) {
                            for (const k in a) {
                                _a[k] = _a[k] || a[k];
                            }
                        } else {
                            attrs.push(a);
                        }
                    });
                }

                const promises = attrs.map(async a => {
                    const config = { root: a, $owner: this };
                    for (let p in a) {
                        const s = attrMap[p];
                        if (s === undefined) continue;
                        config[s] = a[p];
                    }
                    try {
                        if (!a.Type)
                            throw new Error('no type for ' + config.name);
                        try {
                            config.$typeClass = (await this.$base.findItem(a.Type));// || (await this.$host.findItem(a.Type));
                            if (!config.$typeClass)
                                throw new Error(`type ${a.Type} not found for ${config.name}`);
                            const url = `${config.$typeClass.url}/~/fields/extend.js`;

                            const imp = await import(url);
                            return new imp.default(config);
                        }
                        catch (e) {
                            console.error(e)
                            throw new Error('error import for ' + config.name);
                        }

                    }
                    catch (e) {
                        console.warn(e)
                        const url = `/web/core/class/fields/extend.js`;
                        const imp = await import(url);
                        return new imp.default(config);
                        // return new odaField(config);
                    }
                });
                return await Promise.all(promises);
            });
        },
        get _$$config() {
            return this.cache('_$$config', async () => {
                return await this.$$fields;
            });
        },
        getValue($object, root) {
            if ($object)
                return $object.getValue(this, root);
            return (root && root[this.name]);
        },
        getFormattedValue($object, root, format) {
            return this.getValue($object, root)
        },
        async setValue($object, value, root) {
            if ($object)
                return $object.setValue(this, value, root);
            throw new Error('No context odaObject');
        },
        async getValidate($object, root) {
            if (!$object || $object.readOnly)
                return null;
            if (this.ValueList && this.onlyValueList) {
                const value = await this.getValue($object, root);
                if (!value) return null;
                const list = await this.getValueList($object, root);
                const fn = (v) => v === value || (typeof v === 'object' && v.value === value);
                if (!list.some(fn)) return `Only values from the value list are allowed.`;
            }
            return this.getMetadata('validate', $object, root, 0) || this.getMetadata('Validate', $object, root, 0);
        },
        getOwnerRoot($object, currentRoot) {
            if (!currentRoot) return null;
            const getRoots = (node, path) => {
                const step = `$${path.shift()}`;
                const cPath = Array.from(path);
                const subNodes = node[step];
                if (!subNodes || path.length === 0) return subNodes;
                return subNodes.reduce((res, n) => {
                    const roots = getRoots(n, cPath);
                    if (roots) res.push(...roots);
                    return res;
                }, []);
            };
            const path = this.path.split('/').slice(1, -1); // срезаем [FIELDS] и fieldName
            if (path.length === 1) return $object.root;
            const name = path.pop();
            const root = $object.root;

            const roots = getRoots(root, path);
            return roots.find(r => {
                const root = r[`$${name}`];
                if (Array.isArray(root)) {
                    return root.some(r => r === currentRoot);
                }
                return false;
            });
        },
        getMetadata(attr, $object, dataRoot, idx) {
            return $object && $object.getMetadata(this, attr, dataRoot || $object.root, idx);
        },
        async getReadOnly($object, root) {
            if (!$object || $object.readOnly && getBool($object.readOnly))
                return true;
            const res = getBool(await this.getMetadata('ReadOnly', $object, root, 0));
            if (res || this.$owner instanceof odaExtFields) return res;
            return (await this.$owner.getReadOnly) && this.$owner.getReadOnly($object, this.getOwnerRoot($object, root));
        },
        async getNecessary($object, root) {
            if (this.Necessary) {
                if (this.Necessary.toLowerCase() === 'true') return true;
                else if (this.Necessary.toLowerCase() === 'false') return false;
            }

            if (!$object)
                return true;
            const res = await this.getMetadata('Necessary', $object, root, 0);
            return getBool(res);
        },
        async getLinkFields() {
            const val = this.LinkFields;
            if (!val) {
                return (await this.$$fields).reduce((res, i) => res += `@${i.name}, `, '').slice(0, -2);
            } else if (val === '*') {
                return '@*, *';
            }
            return val;
        },
        async fillLinkRoot($object, dataRoot) {
            const xq = `element OBJECT {OBJECT/(${await this.getLinkFields()})}`;
            const res = await $object.XQuery(xq);
            const data = Array.isArray(res.$OBJECT) && res.$OBJECT[0] || {};
            CORE.mixin(data, dataRoot);
        }
    })
    odaField({
        is: 'odaExtFields',
        name: '[FIELDS]',
        label: 'FIELDS',
        icon: 'fields',
        zIndex: 1,
        url() {
            return this.$owner.url;
        },
        get disabled() {
            return this.$owner.disabled;
        },
        get root() {
            return this.$owner.root?.$METADATA?.[0];
        }
    })
    odaExtFields({
        is: 'odaStatic',
        name: '[STATIC]',
        label: 'STATIC',
        zIndex: 2,
        get root() {
            return this.$owner.root?.$STATIC?.[0];
        }
    })
    odaStatic({
        is: 'odaDetails',
        name: '[DETAILS]',
        label: 'DETAILS',
        zIndex: 3,
        get root() {
            return this.$base.root?.$STATIC?.[0];
        }
    })
    // TODO: возможно стоит перенести в компонент support (используется только в нём)
    odaPart({
        is: 'odaMarketHost',
        props: {
            label() {
                return this.$host.label;
            },
            supportListNames: [
                { i: 'Modules', t: 'module', p: '$M', m: '$$modules' },
                { i: 'Solutions', t: 'workplace', p: '$W', m: '$$workplaces' },
                { i: 'Classes', t: 'class', p: '$C', m: '$$classes' }
            ]
        },
        get _$$support() {
            return this._getSupport();
        },
        async _getSupport() {
            await this.getConfig();
            const result = [];
            for (const s of this.supportListNames) {
                const config = this.config[s.p];
                if (!config) continue;

                const group = new odaMarketGroup();
                group.$owner = this;
                group.id = s.i;
                group.type = s.t;
                group.method = s.m;
                group.count = config.length;
                result.push(group);
            }
            return result;
        }
    })
    // TODO: возможно стоит перенести в компонент support (используется только в нём)
    odaGroup({
        is: 'odaMarketGroup',
        props: {
            method: String
        },
        get _$$support() {
            return this._getSupport();
        },
        async _getSupport() {
            return await this.$owner[this.method];
        }
    })
    CORE.waitIcon = 'odant:spin';
    CORE.host = new odaHost({ origin: CORE.URL.origin });
    CORE.host.getConfig();
    CORE.host.updateAuthorization();
}

export default globalThis.CORE;