ODANT({
    is: 'odant-packets-bar', template: /*template*/`
    <style>
        :host {
            @apply --vertical;
            flex-grow: 0;
        }
    </style>
    <oda-packets-bar-row ref="rows" ~for="(item, idx) in items" :items="item.data" :meta="item.meta" :mode="item.mode" :key="idx"
    @selection-changed="_onRowSelectionChanged($detail.value, idx)"></oda-packets-row>
    `,
    props: {
        contextItem: {
            type: Object,
            set(contextItem, old) {
                if(old) {
                    this.unlisten('pack-create', '_getPackets', { target: old });
                    this.unlisten('pack-delete', '_getPackets', { target: old });
                }
                if (this.packets || !contextItem) return;
                this.listen('pack-create', '_getPackets', { target: contextItem });
                this.listen('pack-delete', '_getPackets', { target: contextItem });
                this._getPackets();
            }
        },
        selection: {
            type: Array,
            default: [],
            set(selection) {
                if (selection.length === this.items.length) {
                    const selectionMask = selection.map(selectionRow => selectionRow.map(i => i.mask.split('|')).flat());
                    // const flatMapResult = selection.flatMap(i => i[0].name).join(', ');
                    const nMasks = [...selectionMask[0]];
                    for (let idx = 1; idx < selectionMask.length; idx++) {
                        let tMasks = nMasks.reduce((res, nm) => {
                            selectionMask[idx].forEach(m => {
                                res.push(`${nm}/${m}`);
                            });
                            return res;
                        }, []);
                        if (tMasks) nMasks.splice(0, nMasks.length, ...tMasks);
                    }
                    const result = selection.flat().map(i => i.name).join(' + ') + ' = ' + nMasks.join('|');
                    this.mask = nMasks.join('|');
                    // ODA.console.status(this.localName + ' selection --> ', result);
                }
            }
        },
        packets: Array,
        items: Array,
        mask: String
    },
    listeners: { execute: '_execute' },
    async _getPackets() {
        await this.contextItem.body;
        const root = this.contextItem.root;
        const packets = await this.contextItem.$$packets;
        if (packets && packets.length > 0 && !(packets.length === 1 && packets[0].name === 'pack')) {
            // this.packets = [packets];
            // packets.forEach(packet => {
            //     if (packet.hasPacks)
            //         this._buildPackets(packet.Packets);
            // });
            this.packets = this.packets || [];
            this._buildPackets(packets);
            const preItems = root && root.$Pack ? root.$Pack.map((sPack, index) => {
                const mode = sPack.PackMode;
                const arrayOfPacketsMap = [...this.packets[index].values()];
                const row = {
                    meta: sPack,
                    mode: mode,
                    data: this._rebuildPackets(arrayOfPacketsMap, mode) || []
                };
                return row;
            }) : this.packets.map((packs, index) => ({ data: this._rebuildPackets([...packs.values()]) }));
            this.items = preItems;
            this.hidden = false;
        } else {
            this._hideBar();
        }
    },
    _onRowSelectionChanged(selection, idx) {
        this.selection[idx] = Object.assign([], selection);
        this.selection = Object.assign([], this.selection);
    },
    _buildPackets(packets, level = 0) {
        this.packets[level] = this.packets[level] || new Map();
        packets.forEach(packet => {
            this.packets[level].set(packet.name, packet);
            if (packet.hasPacks) this._buildPackets(packet.$$packets, level + 1);
        });
    },
    _rebuildPackets(packets, mode) {
        if (/day|month|quart|year/i.test(mode)) {
            let isValidDate = [];
            let isNotValidDate = [];
            packets.forEach(p => {
                if (/^(\d{4})[\/\-]?(0[1-9]|1[012]|q[1-4])?[\/\-]?(0[1-9]|[12][0-9]|3[01])?$/.test(p.name)) isValidDate.push(p);
                else isNotValidDate.push(p);
            });
            if (isValidDate.length > 0) isValidDate = this._makeDateStructure(isValidDate, mode);
            if (isNotValidDate.length > 0) isNotValidDate = this._makeFieldStructure(isNotValidDate);
            const _getMinDate = (dates) => {
                if (dates.length > 0 && dates[0].items && dates[0].items.length > 0) return _getMinDate(dates[0].items);
                else return dates[0]
            };
            const _minDate = isValidDate[isValidDate.length - 1].items[1] ?
                isValidDate[isValidDate.length - 1].items[1]?.items.length > 0 ? _getMinDate(isValidDate[isValidDate.length - 1].items[1]?.items).mask.split('|')[0].replace('*', '') :
                isValidDate[isValidDate.length - 1].items[1] :
                isValidDate[isValidDate.length - 1].items[0];
            const nowDate = new Date().toISOString().split('T')[0];
            const nowName = mode === 'Year' ? nowDate.substr(0, 4) :
                            mode === 'Quart' ? nowDate.substr(0, 5) + 'q' + this._getQuarterOfMonth(nowDate.substr(5, 2)) :
                            mode === 'Month' ? nowDate.substr(0, 7) :
                            nowDate
            return [
                this._getItem({ name: nowName, label: mode, is: 'oda-packets-bar-now-selector', type: 'now' }, mode),
                ...isValidDate,
                ...isNotValidDate,
                this._getItem({ name: 'period', label: 'Period', is: 'oda-packets-bar-period-selector', _minDate, _isRight: true }, mode),
                this._getItem({ name: '*', label: '*', _isRight: true }, mode),
                this._getItem({ name: '<', label: '<', is: 'oda-packets-bar-left-arrow', _isRight: true }, mode),
                this._getItem({ name: '>', label: '>', is: 'oda-packets-bar-right-arrow', _isRight: true }, mode)
            ];
        } else {
            return this._makeFieldStructure(packets);
        }
    },
    _makeFieldStructure(packs, mode) {
        let tree = null,
            arrayOfTree = [];
        for (let x = 0; x < packs.length; x++) {
            if (packs[x].name === 'pack' || packs[x].name === '') {
                arrayOfTree.push(this._getItem({ name: packs[x].name }, mode));
                continue;
            }
            let current = null,
                existing = null;
            [...packs[x].name].forEach((i, idx, arrChar) => {
                const step = packs[x].name.substring(0, idx + 1);
                if (idx === 0) {
                    if (arrayOfTree.every(t => t.name !== step)) {
                        tree = this._getItem({ name: step }, mode);
                        if(arrChar.length > 1) tree.items = [this._getItem({ name: step, root: tree }, mode)];
                        arrayOfTree.push(tree);
                    }
                    current = tree.items;
                } else {
                    existing = current.find(cur => cur.name === step);
                    current = existing ? existing.items : current[current.push(this._getItem({ name: step, root: tree }, mode)) - 1].items;
                }
            });
        }
        arrayOfTree = this._clearTree(arrayOfTree)
            .map(item => (item.name.length > 1 && item.name !== 'pack') ? this._getItem({ name: item.name.substring(0, 1), items: [item] }, mode) : item);
        arrayOfTree.push(this._getItem({ name: '*', label: '*', _isRight: true }, mode));
        return arrayOfTree;
    },
    _clearTree(items) {
        return items.map(item => {
            let current = item;
            if (item.items.length !== 1) {
                current.items = this._clearTree(item.items);
            } else {
                while (current.items.length === 1) {
                    current = current.items[0];
                }
                current.items = this._clearTree(current.items);
            }
            return current;
        });
    },
    _makeDateStructure(packs, mode) {
        return packs.reduce((a, c) => {
            const parts = c.name.split('-');
            const yearName = c.name.substr(0, 4);
            let current = a.find(i => i.name === yearName);
            if (!current) {
                current = this._getItem({ name: yearName, defaultLabel: yearName, items: [] }, mode);
                if(mode !== 'Year') current.items = [this._getItem({ name: yearName, parent: current, root: current }, mode)];
                a.push(current);
            }
            const monthGroupName = c.name.substr(0, 7);
            const splitQuartName = monthGroupName.split('-');
            const quartGroupName = /quart/i.test(mode) ? c.name : splitQuartName[0] + '-q' + this._getQuarterOfMonth(splitQuartName[1]);
            if(parts.length > 1 && current.items.length === 1 && mode !== 'Year') {
                for(let i = 1; i < 5; i++){
                    const quart = this._getItem({ name: yearName + '-q' + i, disabled: true, parent: current, root: current }, mode);
                    current.items.push(quart);
                    const monthsOfQuart = this._getMonthsOfQuarter(i);
                    if(mode !== 'Quart') {
                        monthsOfQuart.forEach(m => {
                            current.items.push(this._getItem({ name: yearName + '-' + m, disabled: true, parent: quart, root: current }, mode));
                        });
                    }
                }
            }
            let currentMonth;
            for (let i = 1; i < parts.length; i++) {
                if (i === 1) {
                    const currentQuart = current.items.find(item => item.name === quartGroupName);
                    delete currentQuart.disabled;
                    currentMonth = current.items.find(item => item.name === monthGroupName);
                    delete currentMonth.disabled;
                } else if (i === 2) {
                    const currentDay = currentMonth.items.find(i => i.name === c.name.substr(0, 10));
                    if (!currentDay) currentMonth.items.push(this._getItem({ name: c.name.substr(0, 10), parent: currentMonth, root: current }, mode));
                }
            }
            return a;
        }, []).sort((a, b) => parseInt(b.name) - parseInt(a.name));
    },
    _getItem(preset, mode) {
        let item = {
            mode,
            mask: this._getMask(preset, mode),
            label: this._getLabel(preset, mode),
            type: this._getType(preset),
            hideIcon: true,
            items: preset.items || []
        };
        if (item.label === '?') item._isRight = true;
        return { ...item, ...preset };
    },
    _getQuarterOfMonth(month) {
        return [1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4][parseInt(month) - 1];
    },
    _getMonthsOfQuarter(quart) {
        return [['01', '02', '03'], ['04', '05', '06'], ['07', '08', '09',], ['10', '11', '12']][parseInt(quart) - 1];
    },
    _getQuarterNumOfQuarterName(quartName) {
        return parseInt(quartName.split('q')[1]);
    },
    _getType({ name }) {
        return /^(\d{4})$/.test(name) ? 'year' :
            /^(\d{4})[\/\-](q[1-4])$/.test(name) ? 'quarter' :
                /^(\d{4})[\/\-](0[1-9]|1[012])$/.test(name) ? 'month' :
                    /^(\d{4})[\/\-](0[1-9]|1[012])[\/\-](0[1-9]|[12][0-9]|3[01])$/.test(name) ? 'day' :
                        name.length > 1 ? 'field' : 'alpha';
    },
    _getLabel(preset, mode) {
        const name = preset.name;
        let dt = new Date(name);
        let label = name === 'pack' || name === '' ? '?' : name.toUpperCase();
        if ((dt instanceof Date && !isNaN(dt)) || this._getType(preset) === 'quarter') {
            switch (this._getType(preset)) {
                case 'year':
                    label = dt.toLocaleDateString(ODANT.language, { year: 'numeric' });
                    break;
                case 'quarter':
                    // const year = name.split('-')[0];
                    const quart = parseInt(name.split('-')[1].substr(1));
                    label = `${quart} quarter`;
                    break;
                case 'month':
                    label = dt.toLocaleDateString(ODANT.language, { year: 'numeric', month: 'long' });
                    break;
                case 'day':
                    label = dt.toLocaleDateString(ODANT.language, {
                        year: 'numeric',
                        month: 'long',
                        day: 'numeric'
                    });
                    break;
            }
        }
        return label;
    },
    _getMask(preset, mode) {
        const name = preset.name;
        let mask = name === '' || name === '*' ? name : name + '*';
        if (this._getType(preset) === 'quarter' && !/quart/i.test(mode)) {
            const year = name.split('-')[0];
            const quart = parseInt(name.split('-')[1].substr(1));
            switch (quart) {
                case 1:
                    mask = `${year}-01*|${year}-02*|${year}-03*`;
                    break;
                case 2:
                    mask = `${year}-04*|${year}-05*|${year}-06*`;
                    break;
                case 3:
                    mask = `${year}-07*|${year}-08*|${year}-09*`;
                    break;
                case 4:
                    mask = `${year}-10*|${year}-11*|${year}-12*`;
                    break;
            }
        }
        return mask;
    },
    _execute(event, details) {
        event.stopPropagation();
        const {item, row, detail} = details.value;
        const _ctrlKey = detail && detail.sourceEvent && detail.sourceEvent.ctrlKey;
        row.selection = _ctrlKey ? [...row.selection, item] : [item];
        row.items.forEach(selectorItem => {
            if (selectorItem.selected && !_ctrlKey) {
                selectorItem.selected = false;
                delete selectorItem.selected;
            }
        });
        item.selected = true;
        if (item.root) {
            // row.resetLabels();
            item.root.selected = true;
            item.root.label = item.label;
        }
    },
    _sortArray(arr) {
        return arr.sort((a, b) => {
            if (a.name === 'pack' || a.name === '') return 1;
            if (b.name === 'pack' || b.name === '') return -1;
            if (/q/.test(a.name + b.name)) {
                const aa = a.name.split('-');
                const bb = b.name.split('-');
                if (aa[0] === bb[0]) {
                    const aaa = parseInt(aa[1].toString().replace('q', ''));
                    const bbb = parseInt(bb[1].toString().replace('q', ''));
                    if (!isNaN(aaa) && !isNaN(bbb)) {
                        return aaa - bbb;
                    }
                }
            }
            return a.name.localeCompare(b.name);
        })
    },
    _hideBar() {
        this.mask = '*';
        this.hidden = true;
    }
});
ODA({
    is: 'oda-packets-bar-row', template: /*template*/`
    <style>
    :host {
        @apply --flex;
        @apply --horizontal;
        justify-content: space-between;
    }
    :host > div {
        @apply --horizontal;
        align-items: center;
        padding: 4px;
    }
    </style>
    <div class="left">
        <div ~is="item.is || tmpl" :title="'by ' + mode" ref="selectors" ~for="(item, key) in leftItems" :item="item" :label="item?.label" :disabled="item?.disabled" :selected="item?.selected" :key="key"></div>
    </div>
    <div class="right">
        <div ~is="item.is || tmpl" :title="'by ' + mode" ref="selectors" ~for="(item, key) in rightItems" :item="item" :label="item?.label" :disabled="item?.disabled" :selected="item?.selected" :key="key"></div>
    </div>
    `,
    props: {
        items: {
            type: Array,
            set(items, old) {
                if (!old)
                    this.fire('execute', {item: items[0], row: this});
            }
        },
        leftItems: {
            type: Array,
            get() {
                return this.items?.filter(i => !i._isRight);
            }
        },
        rightItems: {
            type: Array,
            get() {
                return this.items?.filter(i => i._isRight);
            }
        },
        selection: {
            type: Array,
            set(selection, old) {
                this.fire('selection-changed', selection);
            }
        },
        meta: Object,
        mode: {
            type: String,
            enum: ['Alpha', 'Field', 'Day', 'Month', 'Quart', 'Year']
        },
        tmpl: 'oda-packets-bar-selector'
    },
    listeners: {
        left: '_setLeft',
        right: '_setRight'
    },
    resetLabels() {
        this.items.forEach(i => {
            if(i.defaultLabel !== i.label && i.type === 'year') i.label = i.defaultLabel;
        });
    },
    filter(callback) {
        const find = (items) => {
            const res = items.filter(i => i.items).reduce((res, item) => {
                res.push(...find(item.items));
                return res;
            }, []);
            res.push(...items.filter(callback));
            return res;
        };
        return find(this.items);
    },
    _getSelectionFromPeriod({ start, end }) {
        return this.filter(i => {
            let dateName = i.name;
            switch (this.mode) {
                case 'Year': dateName = dateName + '-01-01'; break;
                case 'Quart': dateName = i.mask.substr(0, 10); break;
                case 'Month': dateName = dateName + '-01'; break;
            }
            const itemDate = new Date(dateName).getTime();
            start = new Date(start).getTime();
            end = new Date(end).getTime();
            return itemDate >= start && itemDate <= end;
        });
    },
    _getLeft(item) {
        const indexSel = item.parent.items.findIndex(i => item.name === i.name);
        return indexSel === 0 ? this._getPrev(item) : item.parent.items[indexSel - 1];
    },
    _getRight(item) {
        const indexSel = item.parent.items.findIndex(i => item.name === i.name);
        return indexSel === item.parent.items.length - 1 ? this._getNext(item) : item.parent.items[indexSel + 1];
    },
    _getPrev(item) {
        const indexSel = item.root.items.findIndex(i => i.name === item.parent.name);
        return indexSel === 2 ? this._getLast(item.root, item.type) : item.root.items[indexSel - 1]?.items[item.root.items[indexSel - 1]?.items.length - 1];
    },
    _getNext(item) {
        const indexSel = item.root.items.findIndex(i => i.name === item.parent.name);
        return indexSel === item.root.items.length - 1 ? this._getFirst(item.root, item.type) : item.root.items[indexSel + 1]?.items[0];
    },
    _getFirst(rootItem, type) {
        return rootItem.items[0];
    },
    _getLast(rootItem, type) {
        return rootItem.items[rootItem.items.length - 1]
    },
    _setLeft(e) {
        e.stopPropagation();
        let leftItem, selIndex;
        if (this.selection[0]._isRight) leftItem = this.items[this.items.length - 5];
        switch (this.selection[0].type) {
            case 'year':
                selIndex = this.items.findIndex(i => i.name === this.selection[0].name);
                leftItem = selIndex > 0 ? this.items[selIndex - 1] : this.items[this.items.length - 1];
                break;
            case 'quarter':
                break;
            case 'month':
                break;
            case 'day':
                leftItem = this._getLeft(this.selection[0]);
                break;
            case 'now':
                leftItem = this.items[this.items.length - 5];
                break;
            default:
                break;
        }
        console.log(leftItem);
        if (leftItem)
            this.fire('execute', {item: leftItem, row: this});
    },
    _setRight(e) {
        e.stopPropagation();
        let rightItem, selIndex;
        if (this.selection[0]._isRight) rightItem = this.items[0];
        switch (this.selection[0].type) {
            case 'year':
            case 'now':
                selIndex = this.items.findIndex(i => i.name === this.selection[0].name);
                rightItem = selIndex < this.items.length - 5 ? this.items[selIndex + 1] : this.items[0];
                break;
            case 'quarter':
                break;
            case 'month':
                break;
            case 'day':
                rightItem = this._getRight(this.selection[0]);
                break;
            default:
                break;
        }
        console.log(rightItem);
        if (rightItem)
            this.fire('execute', {item: rightItem, row: this});
    }
});
// import '/web/oda/components/menus/menu/menu.js';
ODA({
    imports: [ '@oda/menu' ],
    is: 'oda-packets-bar-item', template: /*template*/`
    <style>
        :host {
            padding: 6px;
            cursor: pointer;
            font-size: 14px;
            white-space: nowrap;
            max-height: 30px;
        }
        :host([selected]) {
            @apply --selected;
            color: var(--success-color) !important;
            box-shadow: inset 0 0 0 1px var(--success-color);
            text-decoration: underline;
        }
        :host([disabled]) {
            @apply --disabled;
        }
    </style>
    {{label}}
    `,
    props: {
        item: Object,
        label: String
    },
    listeners: { 'tap': '_onTap' },
    async _onTap(e) {
        if (this.item.items && this.item.items.length > 0) {
            e.stopPropagation();
            const result = { focusedItem: null };
            await ODA.showDropdown('oda-menu', {
                items: this.item.items,
                root: result,
                template: 'oda-packets-bar-context-menu-item',
                selector: this
            }, { parent: this });
            if (result.focusedItem && !result.focusedItem.disabled)
                this.fire('execute', {item: result.focusedItem, row: this.domHost});
        } else {
            this.fire('execute', {item: this.item, row: this.domHost, detail : e.detail});
        }
    }
});
ODA({
    is: 'oda-packets-bar-now-selector', extends: 'oda-packets-bar-item', template: /*template*/``,
    props: {
        label: {
            type: String,
            get() {
                return 'Now ' + this.item?.label;
            }
        }
    }
});
ODA({
    is: 'oda-packets-bar-selector', extends: 'oda-packets-bar-item', template: /*template*/``
});
ODA({
    is: 'oda-packets-bar-period-selector-input', template: /*template*/`
    <style>
        :host {
            @apply --horizontal;
            padding: 10px;
        }
        :host div {
            padding: 10px;
        }
        :host div input {
            margin-left: 5px;
        }
    </style>
    <div>
        <label for="calendar-start">Start date:</label>
        <input type="date" id="calendar-start" :min="item?._minDate" name="trip-start" ::value="start">
    </div>
    <div>
        <label for="calendar-end">End date:</label>
        <input type="date" id="calendar-end" :min="start" name="trip-end" ::value="end">
    </div>
    `,
    props: {
        item: Object,
        start: String,
        end: String
    }
});
ODA({
    is: 'oda-packets-bar-period-selector', extends: 'oda-packets-bar-item', template: /*template*/``,
    listeners: { 'tap': '_onTap' },
    async _onTap(e) {
        const inputDate = await ODA.showDialog('oda-packets-bar-period-selector-input', { item: this.item }, {});
        if (inputDate && inputDate.start && inputDate.end) {
            const selection = this.domHost._getSelectionFromPeriod(inputDate);
            this.item.label = `С ${inputDate.start} по ${inputDate.end}`;
            this.item.mask = selection.map(i => i.mask).join('|');
            this.fire('execute', {item: this.item, row: this.domHost});
        }
    }
});
ODA({
    is: 'oda-packets-bar-left-arrow', extends: 'oda-packets-bar-item', template: /*template*/``,
    listeners: { 'tap': '_onTap' },
    async _onTap(e) {
        e.preventDefault();
        this.fire('left');
    }
});
ODA({
    is: 'oda-packets-bar-right-arrow', extends: 'oda-packets-bar-item', template: /*template*/``,
    listeners: { 'tap': '_onTap' },
    async _onTap(e) {
        e.preventDefault();
        this.fire('right');
    }
});
ODA({
    is: 'oda-packets-bar-context-menu-item', template: /*template*/`
    <style>
        :host {
            @apply --label;
            @apply --horizontal;
            @apply --flex;
            align-items: center;
            max-width: 480px;
            text-overflow: ellipsis;
        }
        :host>*{
            padding: 8px;
        }
        :host([disabled]) {
            @apply --disabled;
        }
        :host([is-month]) {
            padding-left: 15px;
        }
    </style>
    <div :title="item?.label">{{item?.label}}</div>`,
    props: {
        item: Object,
        disabled: {
            type: Boolean,
            reflectToAttribute: true,
            get() {
                return this.item?.disabled;
            },
            default: false
        },
        isMonth: {
            type: Boolean,
            reflectToAttribute: true,
            get() {
                return this.item?.type === 'month';
            },
            default: false
        }
    },
    listeners: { 'tap': '_tap' },
    _tap(e) {
        if(this.disabled) e.preventDefault();
    }
});