const types = [
    {
        path: 'client', label: 'Client side', sub: [
            { path: 'views', label: 'View' },
            { path: 'pages', label: 'Page' },
            { path: 'developer', label: 'Developer tool' },
            { path: 'create', label: 'Action to create new items' },
            { path: 'methods', label: 'Services, buttons and other' },
        ]
    },
    {
        path: 'server', label: 'Server side', sub: [
            { path: 'methods', label: 'Method' },
            {
                path: 'triggers', label: 'Trigger', sub: [
                    { path: 'after-create' },
                    { path: 'before-create' },
                    { path: 'after-update' },
                    { path: 'before-update' },
                    { path: 'after-delete' },
                    { path: 'before-delete' },
                ]
            },
            {
                path: 'tasks', label: 'Task', sub: [
                    { path: 'on-start' },
                    { path: 'on-timer' },
                ]
            },
        ]
    }
];
export default odaAction({
    props: {
        allowAccess: 'A',
        icon: 'icons:add',
        label: 'Add action',
    },
    async execute() {
        const control = await ODANT.createComponent('odant-add-action-dialog');
        const res = await ODA.showDialog(control, undefined, { title: 'Create action' });
        if(!res || !res.side) return;
        const side = res.side;
        const type = res.type || 'methods';
        const label = res.label?.trim() || `new-${type}`;
        let name = label.replace(/ +/g, '-');
        name = name.replace(/-{2,}/g, '-');
        console.log(`side: ${side}`);
        console.log(`type: ${type}`);
        console.log(`label: ${label}`);
        console.log(`name: ${name}`);
    }
})
ODANT({
    is: 'add-action-dialog',
    imports: ['@oda/basic'],
    template: /*html*/`
    <style>
        :host{
            @apply --vertical;
            @apply --flex;
            padding: 16px;
            min-height: 315px;
            min-width: 300px;
        }
        oda-button{
            margin: 4px;
            @apply --raised;
        }
        oda-button>div{
            @apply --flex;
        }
        oda-button[icon="icons:arrow-back"]{
            background-color: rgba(125, 125, 125, 0.5);
            align-self: flex-start;
        }
        :host>div{
            @apply --vertical;
            @apply --flex;
        }
        :host>div>info{
            padding: 4px;
        }
        oda-basic-input{
            @apply --no-flex;
            box-shadow: inset 0px 0px 0px 1px rgba(0,0,0,0.5);
        }
    </style>
    <div ~show="!side">
        <oda-button ~for="t of _getTypes()" @tap="side = t.path" :toggled="t.path === side"><div>{{t.label || t.path}}</div></oda-button>
    </div>
    <div ~show="side && !type">
        <oda-button ~for="t of _getTypes(side)" @tap="type = t.path" :toggled="t.path === type"><div>{{t.label || t.path}}</div></oda-button>
        <div class="flex"></div>
        <oda-button @tap="side = ''" icon="icons:arrow-back">back</oda-button>
    </div>
    <div ~show="side && type && ( _getTypes(type, _getTypes(side)))">
        <oda-button ~for="t of _getTypes(type, _getTypes(side))" @tap="label = t.path" :toggled="t.path === label"><div>{{t.label || t.path}}</div></oda-button>
        <div class="flex"></div>
        <oda-button @tap="type = ''" icon="icons:arrow-back">back</oda-button>
    </div>
    <div ~show="!_getTypes(type, _getTypes(side)) && side && type">
        <oda-basic-input placeholder="label" ::value="label"></oda-basic-input>
        <div class="flex"></div>
        <oda-button @tap="type = ''" icon="icons:arrow-back">back</oda-button>
    </div>
    `,
    props: {
        label: '',
        side: '',
        type: '',
    },
    listeners: {},
    keyBindings: {},
    observers: [],
    _getTypes(path, source = types) {
        if(!path) return source;
        else return source.find(e => e.path === path).sub;
    }
});
//
// (class extends odaAction {
//     static get properties() {
//         return {
//             allowAccess: 'A',
//             icon: 'icons:add',
//             label: 'Add action',
//             category: 'developer',
//         };
//     }
//
// }).register();`;
//         await actionFolder.saveFile(jsBody, name + '.js', true);
//         if (area === 'client' && ['view', 'page'].includes(type)) {
//             const htmlBody =
//                 `<script src="/web/core/oda.js"></script>
// <oda-module id="oda-${name}">
//     <template>
//         <style>
//
//         </style>
//
//     </template>
//     <script>
//         ODA({
//             is: 'oda-${name}',
//             properties:{
//
//             },
//             ${type === 'page' ? `show() { document.body.appendChild(this.host);}` : ''}
//         })
//     </script>
// </oda-module>`;
//             await actionFolder.saveFile(htmlBody, name + '.html', true);
//         }
//         if (!window.location.href.includes('/client/app-studio/')) {
//             actionFolder.open('app-studio');
//         }
//     }
// }).register();