export default odaView({
    props: {
        label: "File explorer",
        defaultViewName: 'odant-file-explorer',
        category: 'system',
        allowAccess: 'R',
        allowUse: function () {
            return (this.$owner.typeName !== 'file');
        },
        icon: 'enterprise:folder-search'
    }
});

ODANT({
    is: 'file-explorer',
    imports: [
        '@lib/file-tree',
        '@lib/file-tile',
        '@oda/button'
    ],
    template: /*html*/`
        <style>
            :host {
                @apply --vertical;
                height: 100%;
            }
            .right-pane {
                @apply --flex;
                @applay --vertical;
                overflow: hidden;
                border-right: 1px solid gray;
            }
            .main-pane {
                flex: 9;
            }
        </style>
        <div class="horizontal flex between" slot="toolbar">
            <style>
                oda-button[disabled] {
                    opacity: 0.2;
                }
            </style>
            <div style="display: flex">
                <oda-button :disabled="_getGoBackDisabled(_historyIndex)" icon="icons:arrow-back" title="Go back" @tap="_goBack"></oda-button>
                <oda-button :disabled="_getGoForwardDisabled(_historyIndex)" icon="icons:arrow-forward" title="Go forvard" @tap="_goForward"></oda-button>
                <oda-button :disabled="_getGoUpDisabled(focusedItem, contextItem)" icon="icons:arrow-back" style="transform: rotate(90deg)" title="Go up" @tap="_goUp"></oda-button>
            </div>
            <div style="display: flex">
                <oda-button ~for="m,i in _viewModes" :icon="m.icon" @tap="viewModeIndex = i"></oda-button>
            </div>
        </div>
        <odant-file-tree slot="left-drawer" :context-item="contextItem" :icon-size="iconSize" ::focused-node="focusedNode" ::focused-item="focusedItem" allow-focus allow-selection @tap="_closeLeft"></odant-file-tree>
        <odant-file-tile ref="tile" ::current-node="focusedNode" allow-selection :icon-size="Math.round(iconSize * 1.33)" :lazy="false" :view-mode-index="viewModeIndex"></odant-file-tile>
    `,
    props: {
        form: Object, // for outer form control: left.opened etc
        iconSize: 40,
        focusedNode: {
            type: Object,
            watch(n) {
                if (n) {
                    this.focusedItem = n.$item;
                }
            }
        },
        focusedItem: {
            type: Object,
            // notify: true,
            watch(n) {
                if (!this._history || !n) return;
                if ((this._historyIndex < 0) || this._history[this._historyIndex].path !== n.path) {
                    this._history.splice(this._historyIndex + 1, this._history.length - this._historyIndex);
                    this._history[this._historyIndex + 1] = n;
                    this._historyIndex += 1;
                }
            }
        },
        viewModeIndex: Number,
        _dataSet: [],
        _historyIndex: -1,
        _history: [],
        _viewModes: {
            get() {
                return this.$refs?.tile?.viewModes || [];
            }
        }
    },

    observers: [
        async function _setFocusedItem(contextItem) {
            if (contextItem) {
                this.focusedNode = {$item: await contextItem.$folder, items: [{}]};
            }
        }
    ],

    _closeLeft() {
        if (this.form.left.persistent) {
            this.form.left.opened = false;
        }
    },
    _getGoBackDisabled(_historyIndex) {
        if (!this._history || this._history.length === 0) return true;
        return _historyIndex < 1;
    },
    _getGoForwardDisabled(_historyIndex) {
        if (!this._history || this._history.length === 0) return true;
        return _historyIndex >= this._history.length - 1;
    },
    _getGoUpDisabled(focusedItem, contextItem) {
        return !focusedItem || focusedItem.name === '~' || focusedItem === contextItem;
    },
    _goBack() {
        this._goToHistoryIndex(this._historyIndex - 1);
    },
    _goForward() {
        this._goToHistoryIndex(this._historyIndex + 1);
    },
    async _goUp() {
        if (!this.focusedItem || this.focusedItem.path === '~' || this.focusedItem === this.contextItem) return;
        const ownerFolder = await this.focusedItem.$owner;
        if (ownerFolder) {
            this.focusedNode = {$item: ownerFolder};
        }
    },
    _goToHistoryIndex(historyIndex) {
        if (!this._history || (this._history.length === 0) || isNaN(Number(historyIndex)) || historyIndex === this._historyIndex) return;
        if (this._history[historyIndex]) {
            this._historyIndex = historyIndex;
            this.focusedNode = { $item: this._history[historyIndex]};
        }
    }
});