ODANT({
    is: 'file-tree',
    imports: [
        '@lib/item-tree',
        // '@lib/file-drag-and-drop',
        '/web/core/folder/lib/file-drag-and-drop/file-drag-and-drop.js'
    ],
    extends: 'odant-file-drag-and-drop, odant-item-tree',
    // extends: 'oda-tree, oda-file-dragdrop',
    props: {
        itemsList: String,
        currentNode: Object,
        allowInherit: true,
        checkedItems: [],
        showFolders: true,
        showFiles: false,
        fileExtensions: '*',
        allowDrag: true,
        allowDrop: 'extended',
        lazy: true,
        _extFilter: {
            type: Array,
            get() {
                return (this.fileExtensions === '*') ? [] : this.fileExtensions.split(',');
            }
        },
        // fileType: odaItem.Types.odaFile,
        // folderType: odaItem.Types.odaFolder,
    },
    observers: [
        function _setItemsList(showFolders, showFiles) {
            this.itemsList = (this.showFolders ? '$$folders' : '') + (this.showFiles ? ', $$files' : '');
        },
    ],
    async _checkChildren(node) {
        if (node.$item.config) {
            let length = 0;
            if (this.showFolders) length += (node.$item.config.$DIR?.length || 0);
            if (this.showFiles) length += (node.$item.config.$FILE?.length || 0);
            return length > 0;
        }
        return false;
    },
    async _setDataSet(contextItem) {
        const item = (contextItem instanceof odaFolder) ? contextItem : await contextItem.$folder;
        if (!item) {
            return contextItem;
        }
        if (this.hideRoot) {
            const node = { $item: item };
            node.items = await this._beforeExpand(node);
            this.dataSet = [node];
        } else {
            this.dataSet = [{$item: item, $expanded: true}];
        }
        this.focusedItem = item;
    },
    async _onRowContextMenu(e) {
        e.preventDefault();
        e.stopPropagation();
        let selection;
        let context;
        if (this.selection.includes(e.target.item)) {
            context = this.selection.map(i => i.$item);
        } else {
            context = e.target.item.$item;
        }
        const menu = this.create('oda-item-menu', {
            contextItem: context,
            groups: '*',
            collapseLimit: 3,
        });
        menu.open({ hideTitle: false, });
    },
    // __filter(array) {
    //     array?.splice(0, array.length, ...array.filter(i => {
    //         const f = i.$item;
    //         if (!(f instanceof odaFolder) && f.name === 'temp') return false;
    //         if (this.showFiles && !(f instanceof odaFolder) && this._extFilter.length > 0)
    //             return this._extFilter.find(i => f.ext === i);
    //         if (this.showFolders)
    //             return (this.showFiles || f instanceof odaFolder) && (this.allowInherit || !f.inherit.level);
    //         return false;
    //     }));
    //     return array;
    // },
    async _onEvent(e, item) {
        if (item) {
            const node = this.find(i => i.$item === e.currentTarget);
            switch (e.source) {
                case 'create': {
                    if (item.itemType !== this.folderType && item.name === 'temp') return;
                    const node = this.find(i => i.$item === e.currentTarget.$owner);
                    if (node) {
                        this.set(node, '$expanded', true);
                        await this._updateFolder(node);
                        this.async(() => {
                            this.clearSelection();
                            this.focusedNode = this.find(i => i.$item && i.$item.fullId === item.fullId) || this.focusedNode;
                        }, 300);
                    }
                } break;
                case 'changed': {
                    if (node) {
                        if (item.itemType === this.folderType) {
                            await this._updateFolder(node);
                        }
                    }
                } break;
                case 'deleted': {
                    if (node) {
                        if (this.focusedNode === node) {
                            this.focusedNode = null;
                        }
                        this.unlisten(node.$item, 'event', '_onEvent');
                        this.clearSelection();
                        await this._updateFolder(node.$parent);
                        this.deleteItems(i => i.$item.fullId === node.$item.fullId);
                    }
                } break;
            }
        }
        this._refresh();
    },
    async _updateFolder(folderNode) {
        if (!folderNode) return;
        folderNode.$item.reset();
        folderNode.$item.reset(['body', 'Root']);
        const items = await this._beforeExpand(folderNode);
        if (this.focusedItem) {
            for (const item of items) {
                if (item.$item.fullId === this.focusedItem.fullId) this.focusedItem = item.$item;
            }
        }
        for (const item of items) {
            const idx = folderNode.items.findIndex(f => f.$item && f.$item.fullId === item.$item.fullId);
            if (~idx) {
                if (folderNode.items[idx].$expanded) {
                    await this._updateFolder(item);
                    item.$expanded = folderNode.items[idx].$expanded;
                }
            }
        }
        folderNode.items = items;
        this._refresh();
    }
})