ODANT({
    is: 'odant-item-tree',
    imports: `@oda/tree,  @lib/item-node`,
    extends: 'oda-tree',
    props: {
        iconSize: {
            shared: true
        },
        noSort: false,
        idName: '$item',
        itemsList: String,
        expandKeysMap: Object,
        focusedNode: {
            watch(node) {
                this.focusedItem = node && node.$item;
            }
        },
        focusedItem: {
            type: Object,
            async watch(item) {
                if (item) {
                    const owners = [];
                    const getOuterItem = item => {
                        if (!item.$owner) return;
                        let path;
                        if (item.category) {
                            path = item.$owner._fullPath + '/G:' + item.category;
                        } else if (item.type === 'MODULE') {
                            path = item.$owner._fullPath + '/G:Modules';
                        } else if (item.type === 'WORKPLACE') {
                            path = item.$owner._fullPath + '/G:Workplaces';
                        }
                        return path ? { _fullPath: path, $owner: item.$owner} : item.$owner;
                    };

                    let owner = getOuterItem(item);
                    while (owner) {
                        owners.push(owner);
                        owner = getOuterItem(owner);
                    }
                    for (let i = owners.length - 1; i > -1 ; i--) {
                        const owner = owners[i];
                        const node = this.find(i => i.$item?._fullPath === owner._fullPath);
                        if (node && !node.$expanded) {
                            node.$expanded = true;
                            const next = owners[i - 1];
                            if (!node.items.find(el => el?.$item === next))
                                node.items = await this._beforeExpand(node);
                        }
                    }
                    this._refresh(() => {
                        const target = this.find(i => {
                            return Object.equal(i.$item, item) || i.$item?._fullPath !== undefined && i.$item?._fullPath === item._fullPath;
                        });
                        this.selectItem(target);
                    });
                }
                else {
                    this.focusedNode = null;
                    this.clearSelection();
                }
            }
        },
        defaultTemplate: 'odant-item-node'
    },
    observers:[
        function _refreshDataset(contextItem, itemsList) {
            this._setDataSet(contextItem);
        }
    ],
    async _setDataSet(contextItem) {
        if (this.hideRoot) {
            const node = { $item: contextItem };
            node.items = await this._beforeExpand(node);
            this.dataSet = [node];
        } else {
            this.dataSet = [{$item: contextItem}];
        }
        this.focusedItem = contextItem;
    },
    async _beforeExpand(node, deep = 1) {
        if (node?.items?.every(item => {
            return Object.keys(item).length > 0;
        })) {
            return node.items;
        }
        let res = this.itemsList.split(',').map(async src => {
            src = src.trim();
            return node.$item && (await node.$item[src] || []);
        });
        res = (await Promise.all(res)).flat();
        res = res.map(async i => {
            const obj = { $item: i, items: [] };
            if (deep) {
                const ch = await this._checkChildren(obj);
                if (ch) {
                    obj.items = [{}];
                    if (i && this.expandKeysMap && (Object.keys(this.expandKeysMap).some(k => this.expandKeysMap[k].includes(i[k]))))
                        obj.$expanded = 'expanding';
                }
            }
            return obj;
        });
        res = (await Promise.all(res)).filter(this._filter, this);
        if (!this.noSort) res = res.sort(this._sort_);
        return res;
    },
    async  _checkChildren(node) {
        return node.$item.hasChildren;
        // const items = await this._beforeExpand(node, 0);
        // return items.length > 0;
    },
    _filter(item) {
        return true;
    },
    _sort_(a, b) {
        a = a?.$item;
        b = b?.$item;
        if (a === undefined) return -1;
        if (b === undefined) return 1;
        return ((a.zIndex < b.zIndex) || (a.zIndex === b.zIndex && a.label && b.label && a.label.toLowerCase() < b.label.toLowerCase())) ? -1 : 1;
    },
    _onDragStart(e) {
        const el = e.path.find(p => p.row);
        if (el && (this.allowDrag || el.row.drag)) {
            if (el.row.$item && el.row.$item.path) {
                e.dataTransfer.setData('odant/odaitem', JSON.stringify(el.row.$item.toJSON()));
                e.dataTransfer.setData('odant/oda' + el.row.$item.typeName, JSON.stringify(el.row.$item.toJSON()));
                try {
                    const node = e.target.querySelector('oda-table-cell-tree').$core.shadowRoot.querySelector('odant-structure-node');
                    e.dataTransfer.setDragImage(node, 0, 0);
                } catch (err) {
                    e.dataTransfer.setDragImage(new Image(), 0, 0);
                }
            }
            ODA._draggableRows = this._draggableRows = this.selection.includes(el.row) ? this.selection : [el.row];
        }
    },
    _onDragOver(e) {
        let dropEffect = this.$super('oda-tree', '_onDragOver', ...arguments);
        if (dropEffect !== 'none') {
            e.dataTransfer.dropEffect = this._dataTrasfer;
        }
    },
    _onDragEnter(e) {
        this.$super('oda-tree', '_onDragEnter', ...arguments);
        let type = 'none';
        const el = e.path.find(p => p.row);
        if (el.row) {
            if (el.row.drop !== false && (this.allowDrop !== 'none' || el.row.drop)) {
                // TODO. Check drop is available
                type = 'move';
                // for (let i = 0; i < this.dragOperation.length; i++) {
                //     type = this._draggableRows[0].$item._checkRule(this.dragOperation[i].label.toLowerCase(), el.row.$item) ? 'move' : 'none';
                //     if (type !== 'none') {
                //         break
                //     }
                // }
            }
        }
        this._dataTrasfer = type;
    },
    async _onDrop(e) {
        if (this._draggableColumn) {
            return this.$super('oda-tree', '_onDrop', ...arguments);
        }

        const el = e.path.find(p => p.row);
        if (!el)
            return;
        const row = el.row;
        if (row.$role)
            return;
        e.preventDefault();

        let dragSources = this._draggableRows;
        let rowItem;
        switch (this._dropMode) {
            case 'bottom':
            case 'top':
                rowItem = row.$parent;
                break;
            case 'in':
                rowItem = row.$item
                break;
        }
        try {
            if (Array.isArray(dragSources) && dragSources.length > 0) {
                if (!e.ctrlKey) {
                    //TODO.
                }

            } else {
                const dt = e.dataTransfer;
                for (const type of dt.types) {
                    switch (type) {
                        case 'odant/odamodule':
                            const data = JSON.parse(e.dataTransfer.getData(type));
                            let module = await rowItem.findItem(data.path);
                            if (!module) module = new odaModule(data);
                            if (rowItem.inheritDomain) await rowItem.inheritDomain(module);
                            break;
                    }
                }
            }
        } catch (err) {
            console.error(err);
        } finally {
            el.row.$dropMode = '';
            this._dropMode = null;
            this._refresh();
        }
    }
});