import { LayoutItem, loadFromLocalStorage } from '/web/oda/components/layouts/layout-designer/layout-designer.js';

export default odaView({
    label: 'Designer',
    icon: 'icons:create',
})
class GLayoutItem extends LayoutItem {
    constructor(item, listKey = '$$fields', parent, uuid, render) {
        super(item, listKey, parent, uuid);
        this.$listKey = listKey;
        this.$item = item;
        this._id = uuid || item.id;
        this.render = render;
        (async () => {
            this._simple = await this.$item.input;
            this._complex = await this.$item.complex;
        })()
    }
    get actions() { this._actions = this.thisActions[this.level] || []; return this._actions; }
    get items() {
        if (!this._items) {
            (async () => {
                await this.$item.$typeClass?.getConfig();
                this._items = this.$item.hasChildren ? [{}] : [];
                const items = ((await this.$item[this._listKey]) || []).map(i => {
                    const item = new this.constructor(i, this._listKey, this, undefined, this.render);
                    item._level = this.id;
                    item._uuid = this._uuid;
                    return item;
                });
                this._items.splice(0, this._items.length, ...items);

                if (items.length) {
                    this._level = this.id;
                    this._level$r = this.$root.level;
                    this.bs.actions[this.id] = [];
                }

                await this.load();

                this.render?.();
            })();
        }

        return this._items;
    }
    get simple() {
        return this._simple;
    }
    get complex() {
        return this._complex;
    }
    get focused() {
        return this._focused;
    }
    set focused(v) {
        this._focused = v;
        if (v) {
            this.$parent.items.forEach && this.$parent.items.forEach(i => {
                if (i !== this) i.focused = false;
            })
        }
        else {
            console.log('focused')
        }
    }
    save(save) {
        if (this.bs.enableSave && this.bs.designMode && this.thisActions) {
            if (this.$item.typeName !== 'field') {
                this.$item.$class.$folder.saveFile(JSON.stringify(this.thisActions), getSettingsPath(this.$item));
            } else {
                this.getOwner().save();
                this.childActions && this.$item.$class.$folder.saveFile(JSON.stringify(this.childActions), getSettingsPath(this.$item));
            }
        }
    }
    async load() {
        if (this.$item.typeName !== 'field') {
            const file = await this.$item.$class.$folder.getFile(getSettingsPath(this.$item));
            this.thisActions = this.childActions = await file?.load() || {};
        } else {
            const owner = this.getOwner();
            if (!owner.childActions) await owner.load();
            owner.childActions[this.level] = owner.childActions[this.level] || [];
            this.thisActions = owner.childActions;

            const file = await this.$item.$class.$folder.getFile(getSettingsPath(this.$item));
            this.childActions = await file?.load() || {};
        }

        this.constructor.loadActions(this);
    }
    reset() {
        //if (this.$item.typeName !== 'field') {
        this.$item.$class.$folder.saveFile('{}', getSettingsPath(this.$item));
        //} else {
        //this.$owner.reset();
        //}
    }
    getOwner() {
        let owner = this.$owner;
        while (!(owner instanceof GLayoutItem)) {
            owner = owner.$owner;
        }
        return owner;
    }
}
function getSettingsPath(item) {
    const subPath = item.typeName === 'field' && !item.hasChildren
        ? item.path.replace(/^\[(fields|static|details)\]/gi, '')
        : '';
    return `settings${subPath}/layout.json`;
}

ODANT({
    is: 'designer',
    extends: 'oda-layout-designer',
    props: {
        contextItem: {
            async set(n) {
                this.uuid = n.id || getUUID();
                this.layout = new GLayoutItem(n, undefined, undefined, this.uuid, this.render.bind(this));
                this.layout.$root = this.layout;
                GLayoutItem._bus[this.uuid] = {
                    fileName: 'oda-layout-designer-actions-' + this.uuid,
                    fnTreeRefresh: () => { this.$refs?.tree?._refresh?.() },
                    enableSave: false,
                    designMode: false,
                    dragInfo: {},
                    selected: {},
                    selection: [],
                    selectionID: [],
                    actions: {},
                    showBorder: false,
                    root: this.layout,
                    designer: this
                };
            }
        },
        item: Object,
        iconSize: 24,
        expanded: true,
        allowExpand: false,
    }
})