import './core.js';
import '/web/oda/oda.js';
const odantComponentRegExp = /(?<=\<)(odant-[a-z\-]+)/gim;
const regexUrl = /https?:\/\/(?:.+\/)[^:?#&]+/g
globalThis.ODANT = globalThis.ODANT || async function (prototype) {
    const matches = (new Error()).stack.match(regexUrl);
    prototype.url = prototype.url || matches[matches.length - 1];
    ODANT.modules[prototype.url] = ODANT.modules[prototype.url] || [];
    if (!prototype.is || prototype.is in ODANT.deferred) {
        console.warn(prototype)
        return;
    }
    prototype.prefix = prototype.prefix || 'odant-';
    if (!prototype.is.startsWith(prototype.prefix)) prototype.is = prototype.prefix + prototype.is;
    ODANT.modules[prototype.url].push(prototype.is);
    prototype.extends = Array.isArray(prototype.extends) ? prototype.extends : prototype.extends?.split(',') || [];
    if (!prototype.extends.includes('odant-context')) {
        prototype.extends.unshift('odant-context');
    }
    prototype.imports = Array.isArray(prototype.imports) ? prototype.imports : prototype.imports?.split(',') || [];
    ODANT.deferred[prototype.is] = {
        prototype,
        deps: prototype.template?.match(odantComponentRegExp) || [],
    };
    console.log(prototype.prefix + '>', prototype.is);

}

globalThis.FIELD = globalThis.FIELD || async function (prototype) {
    prototype.prefix = 'field-'
    prototype.extends = Array.isArray(prototype.extends) ? prototype.extends : prototype.extends?.split(',') || [];
    if (!prototype.extends.includes('odant-base-field')) {
        prototype.extends.unshift('odant-base-field');
    }
    ODANT(prototype);
}

ODANT.modules = {}
ODANT.deferred = {};

FIELD.regComponent = async function (is, type) {
    // type = '';
    if (type) {
        const fid = is + '-' + type.toLowerCase();
        if (window.customElements.get(fid)) return fid;
        if (!ODANT.deferred[is]) return fid;
        ODANT.deferred[fid] = ODANT.deferred[is];
        ODANT.deferred[fid].prototype.is = fid;
        delete ODANT.deferred[is];
        is = fid;
    }
    return (await ODANT.regComponent(is));// || 'field-default' ;
}
const cache = {};
ODANT.regComponent = async function (is, origin, context) {

    // cache[is] = cache[is] || new Promise(async resolve=>{
        if (window.customElements.get(is)) return is;
        if (!(is in ODANT.deferred)) {
            console.warn(`Prototype ${is} not found!`);
            return is;
        }

        const prototype = ODANT.deferred[is].prototype;
        const modules = ODANT.modules[prototype.url] || [];
        delete ODANT.modules[prototype.url];
        await Promise.all(modules.filter(i => i !== is).map(async k => {
            return ODANT.regComponent(k, origin);
        }))

        const starts = Object.keys(ODANT.deferred);
        await Promise.all((prototype.imports).map(async i => {
            let url = i.trim();
            if (url.startsWith('@')) url = context && !url.startsWith('@oda') ? `/api/${context}/${url}` : `/${url}`;
            if (origin) url = origin + '/' + url;
            url = url.replace(/\/\//g, '/');
            return import(url);
        }));
        for (const k of Object.keys(ODANT.deferred)) {
            if (k === is) continue;
            if (!starts.includes(k) || ODANT.deferred[is].deps.includes(k) || prototype.extends.includes(k)) {
                await ODANT.regComponent(k, origin);
                starts.push(k);
            }
        }

        await ODA(prototype, origin, context);
        // resolve(is);
        return is;
    // })
    // return cache[is];
}

ODANT.createComponent = async function (id, props) {
    await ODANT.regComponent(id, ODANT.origin, undefined);
    return document.createElement(id, props);
}
ODA({
    is: 'odant-context',
    template: `
        <style>
        :host{
            @apply --flex;
            @apply --vertical;
        }       
    </style>`
    ,
    props: {
        context: {
            type: String,
            async set(context) {
                this.contextItem = (context) ? await CORE.host.findItem(context) : undefined;
            }
        },
        contextItem: Object,
    },
    observers: [
        '___contextItemChanged___(contextItem)'
    ],
    ___contextItemChanged___(ci) {
        if (ci) {
            this.saveKey = (ci.$class || ci).path;
            this.render(true);
            return true;
        } else {
            this.saveKey = '';
            return true;
        }
    }
});
ODANT({
    is: 'base-field',
    template: /*html*/`
    <style>
        :host{
            @apply --vertical
            position: relative;
            border: none;
        }
        .dropdown {
            padding: 0px;
            opacity: .5;
        }
        .dropdown:hover {
            opacity: 1;
        }
        .warning {
            cursor: pointer;
            position: absolute;
            left: 0px;
            z-index: 1;
            opacity: .5;
        }
        .error {
            cursor: pointer;
            position: absolute;
            z-index: 1;
            opacity: .5;
        }
        oda-icon {
            align-self: center;
        }
    </style>`,
    hostAttributes: {
        tabindex: 0
    },
    _warning(e) {
        ODA.push(e.target.title);
    },
    _showDropDown(value) {
        return this.$field && !this.complex && !this.readOnly && this.$field._hasList && !(this.$field._isLink && value)
    },
    props: {
        complex: {
            type: Boolean,
            reflectToAttribute: true
        },
        $object: {
            type: Object,
            default: null,
            watch(n, o) {
                if (o) {
                    //this.unlisten(n, 'changed', 'getValue')
                }
                if (n) {
                    //this.listen(n, 'changed', 'getValue')
                }
            }
        },
        warning: {
            type: String,
            reflectToAttribute: true,
            watch(n, o) {
                if (o) {
                    this.style.setProperty('padding-left', '0px');
                }
                if (n) {
                    this.style.setProperty('padding-left', (this.iconSize * .5 + 12) + 'px');
                }
            }
        },
        $field: Object,
        necessary: {
            type: Boolean
        },
        dataRoot: Object,
        focused: false,
        iconSize: 24,
        value: {
            type: [String, Number, Boolean, Object],
            notify: true,
            async watch(n) {
                if (n !== undefined && this.$field && this.$object && !this.readOnly) {
                    await this.$field.setValue(this.$object, n, this.dataRoot);
                }
            }
        },
        item: {
            type: Object,
            watch(item) {
                if (item)
                    this.dataRoot = item;
            }
        },
        column: {
            type: Object,
            watch(column) {
                this.$field = column && column.$field;
            }
        },
        readOnly: Boolean
    },
    listeners: {
        input(e) {
            e.stopPropagation();
            if (this.$field && this.$field._hasList && e.detail.value)
                this._showList(e.detail.value);
            else
                this._closeDropDown();
        },
        keyup(e) {
            if (this.focused && e.ctrlKey && e.keyCode === 32) {
                e.preventDefault();
                this._showList(null);
            }
        }
    },
    observers: [
        async function _init($field, dataRoot, $object) {
            if ($field && dataRoot) {
                if (await $field.getReadOnly($object, dataRoot)) {
                    this.readOnly = true;
                }
                this.getValue();
            }
        }
    ],
    keyBindings: {
        home: '_first',
        ArrowUp: '_before',
        ArrowDown: '_next',
        enter: '_result',
    },
    _first(e) {
        if (this.dd) {
            e.stopPropagation();
            this.dd.first();
        }
    },
    _result(e) {
        if (this.dd) {
            e.stopPropagation();
            this.dd._result();
        }
    },
    _next(e) {
        if (this.dd) {
            e.stopPropagation();
            this.dd.next();
        }
    },
    _before(e) {
        if (this.dd) {
            e.stopPropagation();
            this.dd.before();
        }
    },
    _closeDropDown() {
        this.async(() => {
            if (this.dd) {
                this.dd.close();
                this.dd = null;
            }
        }, 100)
    },
    setValue(value) {
        if (this.value === value) return;
        if (value && value.valueOf) {
            if (this.value && this.value.valueOf) {
                if (this.value.valueOf() === value.valueOf())
                    return;
            }
        }
        this.value = value === undefined ? null : value;
    },
    async getValue(e) {
        if (this.$field && this.dataRoot) {
            if (!this.$field.isTable) {
                const v = await this.$field.getValue(this.$object, this.dataRoot);
                this.setValue(v);
            }
            if (this.$object) {
                const w = await this.$field.getValidate(this.$object, this.dataRoot);
                const r = await this.$field.getReadOnly(this.$object, this.dataRoot);
                const n = await this.$field.getNecessary(this.$object, this.dataRoot);

                this.warning = w;
                this.readOnly = r;
                this.necessary = n;
            }
        }
    },
    attached() {
        //this.listen(this, 'focus', '_onFocus');
        //this.listen(this, 'blur', '_onBlur');
        let host = this.domHost;
        while (this.$object === undefined && host) {
            this.$object = host.$object;
            host = host.domHost;
        }
    },
    detached() {
        //this.unlisten(this, 'focus', '_onFocus');
        //this.unlisten(this, 'blur', '_onBlur');
    },
    toFocus() {
        this._onFocus();
        return this.focused;
    },
    _onFocus(e) {
        if (e) e.stopPropagation();
        if (!this.readOnly) {
            this.focused = true;
        }
    },
    _onBlur() {
        this.focused = false;
        if (this.dd) {
            this.async(() => {
                this._closeDropDown();
            }, 300);
        }
    },
    async _showList(filter) {
        // TODO
        if (filter === null && this.dd) {
            this._closeDropDown();
            return;
        }
        try {
            let list = await this.$field.getValueList(this.$object, this.dataRoot);
            if (list.length) {
                if (!this.dd || !this.dd.opened) {
                    this.dd = ODA.createElement('oda-dropdown-list', { parent: this });
                    this.dd.filter = filter;
                    this.dd.items = list;
                    this.dd.showCount = true;
                    let res = await this.dd.show(this.value, !this.focused).catch(() => void (0));
                    if (!res) res = this.value;
                    this._closeDropDown();
                    this.focused = false;
                    if (res && typeof res === 'object' && 'value' in res) res = res.value;
                    if (this.$field._isLink && typeof res === 'object' && !(res instanceof odaItem)) {
                        res = await this.$field._getObject(res);
                    }
                    this.$field.setValue(this.$object, res, this.dataRoot);
                }
                else {
                    this.dd.filter = filter;
                    this.dd.items = list;
                }
            }
            else {
                this._closeDropDown();
            }
        }
        catch (e) {
            this._closeDropDown();
        }
    }
});

const cssVariables = [
    { name: '--inherit-owner', syntax: '<color>', initialValue: 'orangered', inherits: true, }, //#ff4500
    { name: '--inherit-parent', syntax: '<color>', initialValue: 'blue', inherits: true, }, //#0000ff
    { name: '--inherit-type', syntax: '<color>', initialValue: 'green', inherits: true, }, //#008000
    { name: '--inherit-core', syntax: '<color>', initialValue: 'gray', inherits: true, }, //#808080
    { name: '--inherit-service', syntax: '<color>', initialValue: 'red', inherits: true, }, //#ff0000
    { name: '--inherit-self', syntax: '<color>', initialValue: 'black', inherits: true, }, //#000000

    { name: '--admin-color', syntax: '<color>', initialValue: 'black', inherits: true, },
    { name: '--rwcd-color', syntax: '<color>', initialValue: 'red', inherits: true, },
    { name: '--rwc-color', syntax: '<color>', initialValue: 'green', inherits: true, },
    { name: '--rw-color', syntax: '<color>', initialValue: 'blue', inherits: true, },
    { name: '--r-color', syntax: '<color>', initialValue: 'gray', inherits: true, },
    { name: '--p-color', syntax: '<color>', initialValue: 'gray', inherits: true, },
    { name: '--no-access-color', syntax: '<color>', initialValue: 'black', inherits: true, },

    { name: '--item-color', syntax: '<color>', initialValue: 'black', inherits: true, },
    { name: '--domain-color', syntax: '<color>', initialValue: 'gray', inherits: true, },
    { name: '--host-color', syntax: '<color>', initialValue: 'black', inherits: true, },
    { name: '--part-color', syntax: '<color>', initialValue: 'black', inherits: true, },
    { name: '--base-color', syntax: '<color>', initialValue: 'brown', inherits: true, },
    { name: '--module-color', syntax: '<color>', initialValue: 'magenta', inherits: true, },
    { name: '--workplace-color', syntax: '<color>', initialValue: 'steelblue', inherits: true, },
    { name: '--folder-color', syntax: '<color>', initialValue: 'orange', inherits: true, },
    { name: '--file-color', syntax: '<color>', initialValue: 'black', inherits: true, },
];
if ('registerProperty' in CSS){
    cssVariables.forEach(s => CSS.registerProperty(s));
}else{
    const rootStyle = document.createElement('style');
    rootStyle.textContent += ':root{';
    cssVariables.forEach(s => rootStyle.textContent += `${s.name}:${s.initialValue};\n`);
    rootStyle.textContent += '}';
    document.head.insertBefore(rootStyle, document.head.querySelectorAll('style')[0]);
}

document.addEventListener('DOMContentLoaded', () => {
    const ss = Array.from(document.body.querySelectorAll('*')).filter(i => {
        return i.localName.startsWith('odant-');
    }).forEach(i => {
        ODANT.regComponent(i.localName, ODANT.origin, i.context || i.getAttribute('context') || undefined);
    });
});


ODANT.show = async function(item, view, params = {}, dialog = false) {
    view = view || item.defaultViewName;
    const form = await ODA.createComponent('odant-form', params);
    form.viewName = view;
    form.contextItem = item;
    form.unique = item.path + "#" + form.viewName;
    return form.show(dialog);
},

ODANT.origin = location.origin.match(/https?:\/\/localhost/) ? 'https://current.odant.org' : undefined;//location.origin;
export default globalThis.ODANT;