if (!globalThis.CLASS) {
    globalThis.CLASS = function (model) {
        const parents = (typeof model.extends === 'string'
            ? model.extends.replace(/ /g, '').split(',').map((e) => {
                if (globalThis[e])
                    return globalThis[e];
                throw Error(`Parent class '${e}' for inherit to '${model.is}' not found!`)
            })
            : [model.extends]).filter(Boolean);
        model.listeners = model.listeners || {};
        model.props = model.props || {};
        // Наследование свойств и методов
        for (let key in model) {
            if (coreReservedWords.includes(key)) continue;
            const item = Object.getOwnPropertyDescriptor(model, key);
            if (typeof item.value === 'function')
                continue;
            if (item.value) {
                model.props[key] = model.props[key] || item.value;
            }
            else if (item.get || item.set) {
                model.props[key] = model.props[key] || {};
                model.props[key].get = item.get;
                model.props[key].set = item.set;
            }
            delete model[key];
        }
        for (let key in model.props) {
            let prop = model.props[key];
            if (typeof prop === 'function')
                prop = (prop.name === key) ? { get: prop } : { type: prop };
            if (Array.isArray(prop))
                prop = { default: prop, type: Array };
            else if (prop === null || typeof prop !== "object")
                prop = { default: prop, type: prop ? prop.__proto__.constructor : Object };
            else if (Object.keys(prop).length === 0 || (!prop.get && !prop.set && prop.default === undefined && !prop.type))
                prop = { default: prop, type: Object };
            if (typeof prop.get === 'string')
                prop.get = model[prop.get];
            if (typeof prop.set === 'string')
                prop.set = model[prop.set];
            if (!prop.type && prop.default !== undefined)
                prop.type = prop.default.__proto__.constructor;
            if(prop.default && !prop.get){
                if(Array.isArray(prop.default)){
                    prop.get = function(){
                        return Array.from(prop.default);
                    }
                }else if(typeof prop.default === 'object'){
                    prop.get = function(){
                        return Object.assign({}, prop.default);
                    }
                }else{
                    prop.get = function () {
                        return prop.default;
                    }
                }
            }
            model.props[key] = prop;
        }
        if (this) {
            parents.unshift(this);
            for (const k in this.__model__.listeners) {
                if (!model.listeners[k])
                    model.listeners[k] = this.__model__.listeners[k];
            }
        }
        for (let parent of parents) {
            for (let key in parent.__model__.props) {
                const parentProp = parent.__model__.props[key];
                //if (parentProp) continue;
                const targetProp = model.props[key];
                if (targetProp) {
                    for (let k in parentProp) {
                        targetProp[k] = targetProp[k] || parentProp[k];
                    }
                }
                model.props[key] = targetProp || parentProp;
            }
            for (const k in parent.__model__.listeners) {
                if (!model.listeners[k])
                    model.listeners[k] = parent.__model__.listeners[k];
            }
            for (let key in parent.__model__) {
                const targetMethod = Object.getOwnPropertyDescriptor(model, key);
                if (targetMethod && targetMethod.value)
                    continue;
                const parentMethod = Object.getOwnPropertyDescriptor(parent.__model__, key);
                if (parentMethod && (parentMethod.get || typeof parentMethod || typeof parentMethod.value === 'function'))
                    Object.defineProperty(model, key, parentMethod);
            }
        }
        const clsName = model.is || (this ? ('$' + this.name) : 'cls');
        const body = `with (globalThis) {
        function ${clsName} (props){
            if (!new.target)
                return CLASS.call(${model.is}, props);
            this.__ch__ = {};
            this.__dp__ = {};
            if (!this.constructor.created.hasParams) {
                for (let p in props) {
                    let value = props[p];
                    const prop = this.props[p];
                    if (value !== undefined && prop) {
                        if (prop.type === Boolean)
                            value = (value === 'true') || !!value;
                        else if (prop.type === Number)
                            value = +value;
                    }
                    this[p] = value;
                }
            }
            this.constructor.created.call(this, props);
            for (const k in prototype.listeners || {}) {
                const h = typeof prototype.listeners[k] === 'string'
                    ? this[prototype.listeners[k]]
                    : prototype.listeners[k];
                this.listen(k, h);
            }
        }
        return ${clsName}
    }`;
        const cls = (new Function('p, prototype', body))(this, model);
        cls.__parent__ = this || Object;
        cls.prototype = Object.create((this || CLASS).prototype);
        cls.prototype.constructor = cls;
        cls.__model__ = model;

        Object.defineProperties(cls.prototype, {
            $super: {
                enumerable: true,
                get() {
                    return cls.prototype.__proto__;
                }
            },
            listen: {
                enumerable: false,
                value: function (event, handler) {
                    if (typeof event === "function")
                        handler = event;
                    event = '__any__';
                    const ev = this.__ev__ || (this.__ev__ = {});
                    (ev[event] || (ev[event] = [])).add(handler);
                }
            },
            unlisten: {
                enumerable: false,
                value: function (event, handler) {
                    if (typeof event === "function")
                        handler = event;
                    event = '__any__';
                    const list = this.__ev__?.[event];
                    if (!list) return;
                    handler?list.remove(handler):list.clear();
                }
            },
            fire: {
                enumerable: false,
                value: async function (event, value) {
                    const ev = new CustomEvent(event, { detail: {value, target: this}, composed: true});
                    (this.__ev__?.[event] || []).forEach(i => i(ev));
                    (this.__ev__?.['__any__'] || []).forEach(i =>i(ev))
                }
            },
        })
        Object.defineProperty(cls.prototype, 'props', {
            get() {return model.props}
        })
        if (model.is && !globalThis[model.is]) {
            globalThis[model.is] = cls;
        }
        for (let key in model.props) {
            const prop = model.props[key];
            if (prop.default !== undefined && cls[key] === undefined)
                cls[key] = prop.default;
            const descriptor = { enumerable: !key.startsWith('_'), configurable: false };
            descriptor.get = function () {
                const stack = targetStack ? [...targetStack].reverse() : [];
                for (let i of stack) {
                    if (i.target === this && i.prop === key)
                        continue;
                    const deps = this.__dp__[key] || (this.__dp__[key] = []);
                    if (deps.find(d => {
                        return d.target === i.target && d.prop === i.prop;
                    })) continue;
                    deps.push(i);
                }
                if (prop.get && !(key in this.__ch__)) {
                    targetStack.push({ target: this, prop: key })
                    this.__ch__[key] = prop.get.call(this);
                    targetStack.pop();
                    const deps = this.__dp__[key] || [];
                    for (let d of deps) {
                        delete d.target.__ch__[d.prop];
                    }
                }
                return this.__ch__[key];
            }
            if (!prop.readOnly){
                descriptor.set =  function (val) {
                    if (prop.readOnly) throw new Error(`Read only property "${key}" can't be changed`);
                    if (this.__ch__[key] === val) return;
                    if (prop.set){
                        const res = prop.set.call(this, val);
                        val = (res !== undefined) ? res : val;
                    }
                    this.__ch__[key] = val;
                    const deps = this.__dp__[key] || []
                    for (let d of deps) {
                        delete d.target.__ch__[d.prop];
                    }
                    if (prop.notify)
                        this.fire(key + '-changed', this.__ch__[key])
                }
            }

            Object.defineProperty(cls.prototype, key, descriptor);
        }
        for (let key in model) {
            const d = Object.getOwnPropertyDescriptor(model, key);
            if (typeof d.value === 'function' && key !== 'created') {
                cls.prototype[key] = function (...args) {
                    return d.value.call(this, ...args);
                }
            }
            else if (d.get || d.set)
                Object.defineProperty(cls.prototype, key, d);
        }
        const prefix = model.props?.prefix?.default;
        if (prefix) {
            CLASS.__factory__[prefix] = CLASS.__factory__[prefix] || cls;
        }
        const created = this?.created;
        const constructor = model.created;
        cls.created = function (...args) {
            // created?.call(this, ...args);
            constructor?.call(this, ...args);
        }
        const funcStr = constructor?.toString().replace(/\s*/g, '') || '';
        const startArgs = funcStr.indexOf('(');
        const endArgs = funcStr.indexOf(')');
        cls.created.hasParams = created?.hasParams || ((endArgs - startArgs) > 1);
        cls.__id__ = nextId();
        return cls;
    }
    let counter = 0;
    const targetStack = [];
    const coreReservedWords = [
        'is', 'template', 'props', 'extends', 'keys', 'observers', 'listeners', 'hostAttributes'
    ]
    function nextId(){
        return counter++;
    }
    function isObject(obj) {
        return obj && typeof obj === 'object';
    }
    if (!Object.equal.__native__){
        const fn = Object.equal;
        Object.equal = function (a, b, recurse) {
            if ((a?.__op__?.obj || a) === (b?.__op__?.obj || b))
                return true;
            if (!isObject(a) || !isObject(b)) return false;
            if (a.constructor !== Object || b.constructor !== Object) return Object.is(a, b);
            if (recurse) {
                for (let key in Object.assign({}, a, b))
                    if (!Object.equal(b[key], a[key], recurse)) return false;
                return true;
            }
            return false;
        };
        Object.equal.__native__ = fn;
    }
    Object.defineProperty(Array.prototype, 'has', {
        enumerable: false, configurable: true, value: Array.prototype.includes
    });
    Object.defineProperty(Array.prototype, 'clear', {
        enumerable: false, configurable: true, value: function () {
            this.splice(0);
        }
    });
    Object.defineProperty(Array.prototype, 'last', {
        enumerable: false, configurable: true, get() {
            return this[this.length - 1];
        }
    });
    Object.defineProperty(Array.prototype, 'add', {
        enumerable: false, configurable: true, value: function (...item) {
            for (let i of item) {
                if (this.includes(i)) continue;
                this.push(i);
            }
        }
    });
    Object.defineProperty(Array.prototype, 'remove', {
        enumerable: false, configurable: true, value: function (...items) {
            for (const item of items) {
                let idx = this.indexOf(item);
                this.splice(idx, 1);
            }
        }
    });
    if (!Array.prototype.indexOf.__native__){
        const fn = Array.prototype.indexOf;
        Object.defineProperty(Array.prototype, 'indexOf', {
            enumerable: false, configurable: true, value: function (item) {
                let idx = fn.call(this, item);
                if (!~idx && item?.__op__?.obj)
                    idx = fn.call(this, item?.__op__?.obj || item);
                return idx;
            },
            __native__: fn
        });
    }
}
export default globalThis.CLASS;