import '../../buttons/button/button.js';
import '../../buttons/checkbox/checkbox.js';
import '../../menus/menu/menu.js';

ODA({
    is: "oda-table", template:
        /*html*/
        `<style>
            :host{
                @apply --flex;
                @apply --vertical;
                overflow: hidden;
                position: relative;
            }
            :host([show-borders]){
                border: 1px solid gray;
            }
            :host([even-odd]) .row:not([selected]):not([role]):not([dragging]):first-child>.cell:not([fix]),
            :host([even-odd]) .row:not([selected]):nth-child(2n+3):not([role]):not([dragging])>.cell:not([fix]) {
                background-color: rgba(0,0,0,.01) !important;
            }
            .cell{
                position: relative;
                @apply --content;
                @apply --no-flex;
                @apply --horizontal;
                overflow: hidden;
                text-overflow: ellipsis;
                white-space: nowrap;
                box-sizing: border-box;
                align-items: center;
                padding: 0px 2px;
            }
            :host([auto-width]) .cell{
                @apply --flex;
            }
            :host([row-lines]) .row{
                border-bottom: 1px  solid var(--dark-background);
                box-sizing: border-box;
            }

            :host([col-lines]) .cell{
                border-right: 1px solid var(--dark-background);
                box-sizing: border-box;
            }
            :host([col-lines]) .cell[fix=right]{
                border-left: 2px solid var(--dark-background);
                box-sizing: border-box;
            }

            .head{
                border-top: 1px  solid var(--dark-background);
                @apply --header;
            }
            .foot{
                border-top: 2px  solid var(--dark-background);
                @apply --header;
            }
            .row[focused]::after{
                content: '';
                background-color: var(--focused-color);
                position: absolute;
                bottom: 0px;
                left: 0px;
                right: 0px;
                height: 2px;
                z-index: 1;
            }
            .row[drop-mode]>.cell:first-child:before{
                position: absolute;
                top: 0px;
                /*left: 0px;*/
                bottom: 0px;
                width: 32px;
                content: '';
                pointer-events: none;
                left: var(--margin-left-drag);

                background-repeat: no-repeat;
                background-size: 20px;
                z-index: 10;
            }
            .row[drop-mode=top]>.cell:first-child:before{
                background-position: top;
                transform: rotate(90deg);

                background-image: url('./icons/arrow-forward.svg');
            }
            .row[drop-mode=bottom]>.cell:first-child:before{
                transform: rotate(-90deg);
                background-position: bottom;
                background-image: url('./icons/arrow-forward.svg');
            }
            .row[drop-mode=in]>.cell:first-child:before{
                transform: rotate(180deg);
                background-position: center;
                background-image: url('./icons/arrow-forward.svg');
            }
            .row[dragging] > .cell{
                background-color: rgb(204, 255, 205) ;
            }
            :host .row[raised][last-raised] > .cell:after {
                content: '';
                position: absolute;
                bottom: 3px;
                left: -1px;
                right: -1px;
                height: 5px;
                box-shadow: 0 2px 2px 0 rgba(0, 0, 0, 0.14), 0 1px 1px 0 rgba(0, 0, 0, 0.12), 0 2px 1px -2px rgba(0, 0, 0, 0.2);
            }
            .cell[role=group]{
                z-index: 2;
                @apply --header;
                border-color: transparent;
            }
            .cell[role=footer],
            .row[role=group],
            .cell[fix]:not([role=group]){
                @apply --header;
            }
            .group{
                position: sticky;
                position: -webkit-sticky;
                left: 0px;
                border-color: transparent !important;
                @apply --dark;
            }
            .sticky{
                position: sticky;
                position: -webkit-sticky;
                z-index: 1;
            }
            .group-row{
                z-index: 10;
                position: sticky;
                position: -webkit-sticky;
                @apply --dark;
            }
            .row{
                position: relative;
                @apply --content;
                @apply --horizontal;
            }
            oda-table-footer{
                border-color: white !important;
            }
        </style>
        <style>{{_styles}}</style>
        <oda-table-group-panel ~show="showGroupingPanel" :groups="groups"></oda-table-group-panel>
        <div ref="body" class="flex vertical" style="overflow: auto; min-height: 0px; scroll-behavior: smooth;flex: auto;" @scroll="_scroll">
            <div  ref="header" class="no-flex row header" style="top: 0px; border-bottom-width: 2px; height: auto; position: sticky; z-index: 1;" ~style="{minHeight: _rowHeight+'px', width: autoWidth?'auto':(_scrollWidth + 'px')}" ~if="showHeader">
                <div class="cell head" :allow-sort="allowSort" ~for="col, c in colsHead"  :fix="col.fix" ~is="col.header || defaultHeader" :item="col" :column="col" :show-filter="showFilter" ~class="['col-'+col.id]"></div>
            </div>
            <div class="no-flex vertical body" style="overflow: visible; min-height: 0px;" ~style="{minHeight: (size * _rowHeight)+'px'}">
                <div is-data class="sticky" ~style="{top: headerHeight + 'px'}" style="min-height: 1px;" @dblclick="_dblclick"
                @tap="_tapRows"
                @down="_tapRows"
                @contextmenu="_onRowContextMenu" @dragleave="_onDragLeave" @dragover="_onDragOver"  @drop="_onDrop" @dragenter="_onDragEnter">
                    <div ~for="(row, r) in rows" :item="row" class="row"
                        ~style="_getRowStyle(row)" :row="row" :role="row.$role"
                        ~class="[row.$group?'group-row':'']"
                        :drop-mode="row.$dropMode"
                        :raised="_getRaised(row)"
                        :last-raised="_getLastRaised(row)"
                        :id="row.$id"
                        :dragging="allowDrag && _draggableRows && _draggableRows.includes(row)"
                        :draggable="_getDraggable(row)" @dragstart="_onDragStart"
                        :focused="_isFocusedRow(row)"
                        :selected="_isSelectedRow(row)">

                        <div class="cell" ~for="(col, c) in row.$group? groups :cols" :tabindex="_getTabIndex(col, row, c, r)"
                            :role="row.$role" :fix="col.fix"  :column="col" :item="row"
                            ~is="_getTemplateTag(row, col)" ~props="_getTemplateProps(row, col)"
                            ~if="row.$group?(c < 1):true" ~class="[row.$group ? 'group' : 'col-' + col.id]"></div>
                    </div>
                </div>
            </div>
            <div class="flex" @drop.stop.prevent="" @dragover.stop.prevent="" @down="focusedRow = null; selection.clear()"></div>
            <div ref="footer" class="no-flex horizontal header sticky" style="bottom: 0px;" ~show="showFooter" ~style="{maxHeight: _rowHeight+'px',  minHeight: _rowHeight+'px',  width:(autoWidth?'auto':(_scrollWidth + 'px'))}">
                <div ~is="footer && footer[col.name+'.footer'] || footer.footer || col.footer || defaultFooter" class="foot cell"  :item="footer" ~for="(col, c) in cols"  :fix="col.fix"  is-footer :column="col" ~class="['col-'+col.id]" ></div>
            </div>
        </div>`,
    equalProxy(a, b) { // to prevent wrong proxy comparison
        if (a === b)
            return true;
        if (a === undefined || b === undefined)
            return false;
        a = ('__op__' in a) ? a.__op__.obj : a;
        b = ('__op__' in b) ? b.__op__.obj : b;
        return a === b;
    },
    _tapRows(e) {
        const evt = e.sourceEvent || e;
        if (evt.which !== 1) return;
        if (e.type.includes('down') || e.type.includes('start')) {
            this._select(evt);
        } else {
            this._focus(evt);
        }
    },
    _isFocusedRow(row) {
        const focusedRow = this.focusedRow;
        if (this.allowFocus && focusedRow && !row.$group && row.$allowFocus !== false) {
            if (this.idName)
                return row[this.idName] === focusedRow[this.idName];
            return this.equalProxy(row, focusedRow); // to prevent wrong proxy comparison
            // return row === this.focusedRow;
        }
        return false;
    },
    _getRaised(row) {
        return this.raised.includes(row);
    },
    _getLastRaised(row) {
        return this.raised[0] === row;
    },
    _isSelectedRow(row) {
        if (this.allowSelection) {
            if (this.selectAll) return !this.selection.includes(row);
            if (this.selection.length && !row.$group && row.$allowFocus !== false) {
                if (this.idName) return this.selection.some(r => r[this.idName] === row[this.idName]);
                // return this.selection.includes(row);
                return this.selection.find(item => {return this.equalProxy(item, row)});
            }
        }
        return false;
    },
    _getTemplateTag(row, col) {
        if (row.$group)
            return this.defaultGroupTemplate;
        if (col.treeMode)
            return 'oda-table-cell-tree';
        let template = col.template || row.template || this.defaultTemplate;
        if (typeof template === 'object')
            return template.tag || template.is || template.template;
        return template;
    },
    _getTemplateProps(row, col) {
        if (!row.$group && !col.treeMode) {
            let template = col.template || row.template || this.defaultTemplate;
            if (typeof template === 'object')
                return template.props;
        }
        return {};
    },
    _getTabIndex(col, row, c, r) {
        if (row.$group || !this.allowFocus)
            return null;
        return (col.name && !col.fix && !col.treeMode) ? ((r + 1) * 10000 + c + 1) : '';
    },
    _getDraggable(row) {
        return (this.allowDrag && !row.$group && row.drag !== false) ? 'true' : 'false';
    },
    _getRowStyle(row) {
        return { minHeight: this._rowHeight + 'px', width: this.autoWidth ? 'auto' : (this._scrollWidth + 'px'), top: (row.$group ? this.headerHeight : '0') + 'px' };
    },
    props: {
        showTreeLines: {
            type: Boolean,
            reflectToAttribute: true
        },
        treeLineStyle: {
            type: Object,
            default: {
                width: 1,
                color: 'rgba(0, 0, 0, 0.25)'
            }
        },
        categories: false,
        selectAll: false,
        allowSort: false,
        _height: -1,
        _scrollTop: 0,
        _scrollWidth: 0,
        size: 0,
        _rowHeight: {
            get() {
                return this.iconSize + 8; //нужно как-то получить полную высоту строки
            }
        },
        screen: {
            type: Object,
            default: {
                from: 0,
                length: 0
            },
        },
        idName: '',
        lazy: false,
        footer: {
            get() {
                if (!this._footer) this._footer = this.__footer;
                return this._footer;
            }
        },
        __footer: {
            default: {},
            get() {
                const obj = {};
                this.cols.forEach(c => {
                    if (c.treeMode) {
                        let d = this.size;
                        const i = this.size;
                        if (i > d) {
                            d = d.toLocaleString() + ' / ' + i.toLocaleString();
                        }
                        else {
                            d = d.toLocaleString();
                        }
                        if (this.screen) {
                            let to = this.screen.from + this.screen.length;
                            if (to > d)
                                to = d;
                            if (d)
                                d += '  [ ' + (this.screen.from).toLocaleString() + ' ... ' + to.toLocaleString() + ' ]';
                        }
                        obj[c.name] = d || '';
                    }

                    switch (c.summary) {
                        case 'count': {
                            obj[c.name] = this.size;
                        } break;
                        case 'sum': {
                            obj[c.name] = this.dataSet.reduce((res, r) => {
                                const v = r[c.name];
                                if (v)
                                    res += parseFloat(v);
                                return res;
                            }, 0);
                        } break;
                    }
                });
                return obj;
            }
        },
        raised: [],
        lastRaised: [],
        customRows: Array,
        rows: {
            get() {
                if (this.customRows) return [...this.customRows];
                this.raised = [];
                const items = this.items;
                if (!items) return [];
                if (!this.lazy) return items;
                const rows = items.slice(this.screen.from, this.screen.from + this.screen.length);
                if (rows.length) {
                    while (rows[0].$parent) {
                        this.raised.push(rows[0].$parent);
                        rows.unshift(rows[0].$parent);
                    }
                    while (rows[0].$groupParent) {
                        rows.unshift(rows[0].$groupParent);
                        this.raised.push(rows[0].$groupParent);
                    }
                    const currentLength = rows.length;
                    const scrolledLength = this._scrollTop / this._rowHeight;
                    let diff = this.size - (scrolledLength + currentLength);
                    while (rows.length && diff < 0) {
                        for (let i in rows) {
                            if (!this.raised.includes(rows[i])) {
                                rows.splice(i, 1);
                                diff++;
                                break;
                            }
                        }
                    }
                }
                this._footer = undefined;
                return rows;
            }
        },
        _stylesUpdater: true,
        _styles: {
            default: '',
            get() {
                const stylesUpdater = this._stylesUpdater;
                const calcStyle = (col) => {
                    if (col.id === undefined)
                        return '';
                    if (col.index === 999)
                        return `.col-${col.id}{\n\tmin-width: 0px !important;\n\tflex: 1;\n\tflex-basis: auto;\n}`;
                    if (!col.width) {
                        col.width = this.__read(this.settingsId + '/column/' + col.name + '/width', col.width || 150);
                    }
                    const min = (this.autoWidth && !col.fix) ? '10px' : ((col.width || 100) + 'px');
                    const max = (col.width || 100) + 'px';
                    let style = `.col-${col.id}{\n\tmin-width: ${min};\n\twidth: ${max};`;
                    if (col.fix) {
                        style += `\n\tposition: sticky;\n\tposition: -webkit-sticky;\n\tz-index: 1;\n\tmax-width: ${max};`;
                        if (col.fix === 'left') {
                            style += `\n\tleft: ${col.left}px;`
                        }
                        if (col.fix === 'right') {
                            style += `\n\tright: ${col.right}px;`
                        }
                    }
                    style += '\n}\n';
                    return style;
                };
                return this.cols.reduce((res, col) => {
                    while (col) {
                        if (!res.includes(col)) {
                            if (col.$parent) {
                                col.$parent.width = col.$parent.items.reduce((res, a) => { res += (a.width || 0); return res; }, 0) || col.$parent.width || 150;
                            }
                            res.push(col);
                        }
                        col = col.$parent;
                    }
                    return res;
                }, []).map(col => {
                    return calcStyle(col);
                }).join('\n');
            }
        },
        iconSize: {
            default: 24,
            // shared: true
        },
        table: {
            type: Object,
            shared: true,
            get() {
                return this;
            }
        },
        selection: {
            default: [],
            notify: true,
        },
        allowSelection: {
            reflectToAttribute: true,
            default: false,
            set(n, o) {
                if (o)
                    this.clearSelection();
            }
        },
        settingsId: String,
        headerHeight: {
            get() {
                return this.showHeader && this.$refs && this.$refs.header && this.$refs.header.offsetHeight || 0;
            }
        },
        footerHeight: 0,
        filters: {
            default: [],
            get() {
                return this.columns.filter(i => i.$filter);
            }
        },
        sorts: [],
        groups: [],
        allowFocus: false,
        allowCheck: {
            default: 'none',
            list: ['none', 'single', 'down', 'up', 'double']
        },
        autoWidth: {
            default: false,
            reflectToAttribute: true
        },
        evenOdd: {
            default: false,
            reflectToAttribute: true
        },
        showFooter: false,
        showFilterRow: false,
        defaultHeader: 'oda-table-header',
        defaultTemplate: 'oda-table-cell',
        defaultFooter: 'oda-table-footer',
        defaultGroupTemplate: 'oda-table-cell-group',
        showBorders: {
            type: Boolean,
            reflectToAttribute: true
        },
        showGroupingPanel: false,
        showGroupFooter: false,
        hideRoot: false,
        treeStep: 24,
        colLines: {
            default: false,
            reflectToAttribute: true
        },
        rowLines: {
            default: false,
            reflectToAttribute: true
        },
        showFilter: false,
        allowDrag: false,
        focusedRow: {
            type: Object,
            notify: true,
            freeze: true
        },
        // showHeader:{ // To DELETE after successful test
        //     default: false,
        //     set(n){
        //         if (n){
        //             // this.async(() => {
        //                 this.headerHeight = this.$refs.header.offsetHeight;
        //             // }, 100)
        //         }
        //         else
        //             this.headerHeight = 0;
        //     }
        // },
        showHeader: false,
        allowDrop: {
            list: ['none', 'simple', 'extended'],
            default: 'none'
        },
        columns: [],
        colsHead: {
            get() {
                this.columns.forEach((col, i) => {
                    if (col.name) {
                        col.order = this.__read(this.settingsId + '/column/' + col.name + '/order', col.order || i + 1);
                        col.hidden = this.__read(this.settingsId + '/column/' + col.name + '/hidden', col.hidden || false);
                    }
                    else {
                        // this.set(col, 'width', col.width);
                        col.order = i + 1;
                        col.hidden = false;
                    }
                    switch (col.fix) {
                        case 'left':
                            col.index = col.order - 1000;
                            break;
                        case 'right':
                            col.index = col.order + 1000;
                            break;
                        default:
                            col.index = col.order;
                            break;
                    }

                });
                const cols = this.columns.filter(i => !i.hidden);
                if (!this.autoWidth)
                    cols.push({ index: 999, template: 'div' });
                if (this.allowCheck !== 'none' && !this.columns.some(i => i.treeMode)) {
                    cols.push({ width: 32, index: -999, template: 'oda-table-check', header: 'div', fix: 'left' });
                }
                cols.sort((a, b) => (a.index - b.index));
                cols.filter(i => i.fix === 'left').reduce((res, i) => {
                    i.left = res;
                    res += i.width || 0;
                    return res;
                }, 0);

                cols.filter(i => i.fix === 'right').reverse().reduce((res, i) => {
                    i.right = res;
                    res += i.width || 0;
                    return res;
                }, 0);
                cols.forEach((c, i) => {
                    c.id = i;
                });
                return cols;
            },
        },
        cols: {
            get() {
                const reduce = (cols) => {
                    return cols.reduce((res, col) => {
                        if (col.$expanded && col.items && col.items.length) {
                            const items = col.items.map((c, i) => {
                                c.id = col.id + '-' + i;
                                c.$parent = col;
                                return c;
                            });

                            res.push(...reduce(items));
                        }
                        else {
                            res.push(col);
                        }
                        return res;
                    }, []);
                };
                return reduce(this.colsHead);
            }
        },
        dataSet: {
            default: [],
            freeze: true,
            notify: true,
            set(n, o) {
                this.items = [];
                if (n) {
                    if (!this.columns.length && !this.cols.filter(i => i.name).length) {
                        const count = n.length;
                        if (count) {
                            let i = 10;
                            const names = [];
                            while (i--) {
                                let v = n[Math.floor(Math.random() * count)];
                                for (let n in v) {
                                    if (!names.includes(n) && !Array.isArray(v[n]))
                                        names.push(n);
                                }
                            }
                            this.columns = names.map((col, i) => {
                                return { name: col }
                            });
                        }
                    }
                }
                this._scrollTop = 0;
            }
        },
        _refresher: Boolean,
        _draggableRows: [],
        _treeMode: {
            type: Boolean,
            get() {
                return this.cols.some(i => i.treeMode)
            }
        },
        items: {
            default: [],
            freeze: true,
            get() {
                let r = this._refresher;
                if (!this.dataSet) return [];
                let array = [];
                Object.assign(array, this.dataSet);
                if (this.hideRoot) {
                    Promise.all(array.map(async (i) => {
                        if (i?.items?.length === 1) {
                            const item = i.items[0];
                            if (!Object.keys(item).length || i.$expanded === 'expanding')
                                i.items = await this._beforeExpand(i);
                            i.$expanded = true;
                        }
                        return i;
                    })).then()
                    array = array.reduce((a, b) => {
                        if (!b.items) return a;
                        a.push(...b.items);
                        return a;
                    }, [])
                }
                let expanding = (data, level, parent) => {
                    let expands = data.filter(i => {
                        i.$level = level;
                        i.$parent = parent;
                        if (i.$expanded && !i.hideExpander) {
                            if (i.items) {
                                if (i.items.length === 1) {
                                    const item = i.items[0];

                                    if (!Object.keys(item).length || i.$expanded === 'expanding')
                                        this.fire('expand', i);
                                }
                            }
                            else {
                                this.fire('expand', i);
                            }
                        }
                        return i.$expanded && i.items && (i.items.length || i.items.then);
                    });
                    level++;
                    expands.forEach(async e => {
                        if (e.items.then) {
                            e.items = await e.items;
                        }
                        if ((e.items.length === 1) && (!Object.keys(e.items[0]).length))
                            return;  // to avoid the expanding of empty item of items = [{}]
                        e.items.forEach(i => {
                            if (this.allowCheck !== 'none' && this.allowCheck !== 'single')
                                if (e.checked === true) {
                                    i.checked = true;
                                }
                            i.$level = level;
                            i.$parent = e;
                        });
                        let idx = array.indexOf(e);
                        array.splice(idx + 1, 0, ...e.items);
                        expanding(e.items, level, e);
                        e.$expanded = true;
                    });
                };
                if (!this.preventExpanding) {
                    if (array.length > 0) {
                        expanding(array, 0);
                        if (array.length === 1) {
                            const i = array[0];
                            if (!i.$group && !i.hideExpander && !i.$expanded) {
                                i.$expanded = 'expanding';
                                // this.async(() => {
                                    this.fire('expand', i);
                                // });
                            }
                        }
                    }
                }
                this.filters.forEach(col => {
                    let name = col.name;
                    let filter = String(col.$filter).toLowerCase().replace('&&', '&').replace('||', '|');
                    filter = filter.replaceAll(' and ', '&').replaceAll('&&', '&').replaceAll(' or ', '|').replaceAll('||', '|');
                    filter = filter.split('&').reduce((res, and) => {
                        let or = and.split('|').reduce((res, or) => {
                            let space = or.split(' ').reduce((res, space) => {
                                if (space.trim())
                                    res.push(`String(val).toLowerCase().includes('${space.trim()}')`);
                                return res;
                            }, []).join(' || ');
                            if (space)
                                res.push(`(${space})`);
                            return res;
                        }, []).join(' || ');
                        if (or)
                            res.push(`(${or})`);
                        return res;

                    }, []).join(' && ');
                    let func = new Function('val', `return (${filter})`);

                    array = array.filter(item => {
                        return func(item[name]);
                    });
                });
                array.splice(0, array.length, ...array.filter(this._filter, this));
                if (this.groups.length) {
                    const addChildItemsToGroup = (item, group) => {
                        if (group.value)
                            item.$groupParent = group;
                        if (item.$expanded && item.items && item.items.length > 0) {
                            const idx = group.items.indexOf(item);
                            group.items.splice(idx + 1, 0, ...item.items);
                            item.items.forEach(i => addChildItemsToGroup(i, group));
                        }
                    };
                    let grouping = (data, level, parent = '-') => {
                        let group = this.groups[level];
                        let rows = [];
                        if (!group) {
                            Object.assign(rows, data);
                            if (this.showGroupFooter)
                                rows.push({ $role: 'footer' });
                            return rows;
                        }
                        let col = group;
                        let name = col.name;
                        if (!group.map) {
                            group.map = new Map();
                        }
                        let groups = group.map.get(parent);
                        groups = data.reduce((res, item) => {
                            if (res.every(g => g.items.every(r => r !== item))) {
                                let value = item[name];
                                let obj = res.find(r => {
                                    return r.value === value;
                                });
                                if (!obj) {
                                    obj = groups && groups.find(r => {
                                        return r.$expanded && r.value === value;
                                    });
                                    const newGroup = {
                                        value: value,
                                        name: name,
                                        $col: col,
                                        template: this.defaultGroupTemplate,
                                        $level: level,
                                        $expanded: obj !== undefined,
                                        items: [item],
                                        $group: true,
                                        parent: parent
                                    };
                                    addChildItemsToGroup(item, newGroup);
                                    res.push(newGroup);
                                }
                                else {
                                    obj.items.push(item);
                                    addChildItemsToGroup(item, obj);
                                }
                            }
                            return res;
                        }, []);
                        if (this.categories.allowSort) {
                            groups.sort((a, b) => {
                                return a.value > b.value ? 1 : -1
                            });
                        }
                        group.map.set(parent, groups);
                        groups.forEach(g => {
                            if (g.value)
                                rows.push(g);
                            if (!g.value || g.$expanded) {
                                rows.push(...grouping(g.items, level + 1, parent + '|' + g.name + ':' + g.value));
                            }
                        });
                        return rows;
                        // return this._sort(rows);
                    };
                    array = grouping(array, 0);
                }
                else array.splice(0, array.length, ...array.sort((a, b) => { return this._sort(a, b) }));
                this.size = array.length
                return array;
            }
        },
        expanding: false,
        preventExpanding: false,
    },
    _filter(element) {
        return true;
    },
    _sort(e1, e2) {
        let res = 0;
        this.sorts.some(col => {
            if (res) return true;
            const
                a = e1[col.name],
                b = e2[col.name];
            if (!isNaN(a) && !isNaN(b))
                res = ((parseFloat(a) || 0) - (parseFloat(b) || 0)) * col.$sort;
            else
                res = (String(a)).localeCompare(String(b)) * col.$sort;
        });
        return res;
    },
    async getChecked() {
        const isExistItems = (node) => node && node.items && node.items.length;
        const ls = [];

        const fillCheckedList = async (ls, items) => {
            if (!items || !Array.isArray(items)) return;

            for (let i of items) {
                if (!i.checked) continue;

                ls.push(i);

                if (isExistItems(i)) {
                    const needFill = i.items.length === 1 && !Object.keys(i.items[0]).length;
                    if (i.checked === true && needFill) {
                        const res = await this._beforeExpand(i);
                        res.forEach((i) => {
                            i.checked = true;
                        });
                        i.items = res;
                    }
                    await fillCheckedList(ls, i.items);
                }
            }

            return ls;
        };

        await fillCheckedList(ls, this.items);
        return ls;
    },
    _scroll(e) {
        // this.interval('scroll', ()=>{
        // if (this.$refs.header) this.headerHeight = this.$refs.header.offsetHeight; // To DELETE after successful test
            if (this.$refs.body) {
                // console.log('scrollTop: ' + this.$refs.body.scrollTop);
                this._scrollTop = this.$refs.body.scrollTop;
                this._scrollWidth = this.$refs.body.scrollWidth;
                this._height = this.$refs.body.offsetHeight;
            }
        // this.fire('scroll');
        // }, 10)
    },
    created() {
        this.settingsId = this.localName;
    },
    observers: [
        function _setFooterHeight(showFooter, showFilterRow) {
            // requestAnimationFrame(()=>{
            if (showFooter && this.$refs.footer)
                this.footerHeight = this.$refs.footer.offsetHeight;
            else
                this.footerHeight = 0;
            // })
        },
        function setScreen(_scrollTop, _rowHeight, _height, sorts, filters, columns) {
            const from = Math.round(_scrollTop / _rowHeight);
            const length = Math.round((_height || this.$refs.body.offsetHeight || window.outerHeight) / _rowHeight);
            const post = {
                sort: sorts.map(i => {
                    return { [i.name]: i.$sort === 1 ? 'asc' : 'desc' }
                }),
                filter: filters.map(i => {
                    return { [i.name]: i.$filter }
                })
            };
            const screen = { from: from, length: length, post: post };
            if (this.screen && JSON.stringify(this.screen) !== JSON.stringify(screen))
                this.screen = screen;
        },

    ],
    keyBindings: {
        escape() {
            this.clearSelection();
        },
        'ctrl+a,ctrl+ф'(e) {
            this._selectAll();
        },
        ArrowUp(e) {
            const items = this.items;
            let idx = items.indexOf(this.focusedRow);
            if (idx > 0) {
                this.focusedRow = null;
                while (idx && !this.focusedRow) {
                    idx--;
                    const item = items[idx];
                    if (item && !item.$group) {
                        this.focusedRow = item;
                        this.selection.splice(0, this.selection.length, item);
                    }
                }
            }
        },
        ArrowDown(e) {
            const items = this.items;
            let idx = items.indexOf(this.focusedRow);
            if (idx > -1) {
                this.focusedRow = null;
                while (idx < items.length && !this.focusedRow) {
                    idx++;
                    const item = items[idx];
                    if (item && !item.$group) {
                        this.focusedRow = item;
                        this.selection.splice(0, this.selection.length, item);
                    }
                }
            }
        },
    },
    listeners: {
        expand: "_onExpand",
        dragend: '_onDragEnd',
        resize: '_scroll',
        scrollToUp() {
            if (this.$refs?.body) {
                this.$refs.body.style['scroll-behavior'] = 'unset';
                this.$refs.body.scrollTo(0, 0)
                this.$refs.body.style['scroll-behavior'] = 'smooth';
            }
        }
    },
    async _onExpand(e, d) {
        e.stopPropagation();
        const items = (await this._beforeExpand(d.value)) || [];
        if (d.value.items === items) {
            d.value.$expanded = true;
            this._refresh();
            return;
        }
        d.value.items = items;
        d.value.items.forEach(async i => {
            if (i.items && i.items.then) {
                const p = await i.items;
                if (p && p.length > 0)
                    i.items = [{}];
                else
                    i.items = [];
            }
            return i.items;
        });
        d.value.$expanded = true;
        this._refresh();
    },
    _beforeExpand(item) {
        return item.items;
    },
    _refresh() {
        this.debounce('refresh', ()=>{
            this._refresher = !this._refresher;
        },100)
    },
    _focus(e, d) {
        if (e.ctrlKey || e.shiftKey) return;
        const item = e.target.item;
        if (this.allowFocus && item && item.$allowFocus !== false) {
            this.focusedRow = item;
        }
    },
    _select(e, d) {
        if (this.allowSelection) {
            const item = e.target.item;
            if (e.shiftKey) {
                const selection = this.selection;
                const rows = this.rows;
                let from = rows.indexOf(this.focusedRow);
                from = (from > -1) ? from : 0;
                let to = rows.indexOf(item);
                this.clearSelection();
                if (from <= to) {
                    while(from <= to) {
                        selection.push(rows[from]);
                        from++;
                    }
                } else {
                    while(from >= to) {
                        selection.push(rows[from]);
                        from--
                    }
                }
            } else if (e.ctrlKey) {
                const idx = this.selection.indexOf(item);
                if (idx < 0)
                    this.selection.push(item);
                else
                    this.selection.splice(idx, 1);
            } else {
                this.clearSelection()
                this.selection.push(item);
            }
        }
    },
    clearSelection() {
        this.selection.splice(0);
        this.selectAll = false;
    },
    selectItem(item) {
        // this.debounce('selectItemFuncBody', item => {
            this.selection.splice(0);
            const items = this.items;
            if (item && items) {
                this.selection.push(item);
                this.focusedRow = item;

                // to prevent wrong proxy comparison
                const target = items.find(el => {
                    return this.equalProxy(el, item);
                });
                const idx = items.indexOf(target);
                // const idx = items.indexOf(item);

                if (idx > -1) {
                    const pos = idx * this._rowHeight;
                    const shift = this._rowHeight * Math.floor(this.$refs.body.offsetHeight / (3 * this._rowHeight));
                    if ((this.$refs.body.scrollTop + 0.8 * this._rowHeight > pos) || (this.$refs.body.offsetHeight + this.$refs.body.scrollTop - 1.5 * this._rowHeight < pos)) {
                        this.debounce('changeScrollTop', () => { // for finishig of rendering
                            this.$refs.body.scrollTop = (pos - shift < 0 ) ? 0 : pos - shift;
                        }, 300);
                    }
                };
            }
        // }, 100)
    },
    _selectAll(parent) {
        if (this.allowSelection) {
            this.selection.splice(0);
            this.selectAll = true;
        }
        // if (parent)
        //     this.selection = this.items.filter(i=>!i.$role && i.$parent === parent);
        // else if (this.selection.length)
        //     this.selection = this.items.filter(i=>!i.$role && i.$parent === this.selection[0].$parent);
        // else
        //     this.selection = this.items.filter(i=>!i.$role && i.$parent === undefined);
    },
    getParentItem(item) {
        return item && this.items.find(r => {
            return !r.$role && r.items && r.items.includes(item);
        });
    },
    getParentItems(item) {
        let i = this.getParentItem(item);
        return i ? i.items : this.dataSet;
    },
    addGroup(col) {
        if (this.groups.indexOf(col) < 0) {
            this.groups.push(col);
            this.showGroupingPanel = true;
        }

    },
    removeGroup(col) {
        let idx = this.groups.indexOf(col);
        if (idx > -1)
            this.groups.splice(idx, 1);
        if (!this.groups.length)
            this.showGroupingPanel = false;
    },
    _getColumns(row) {
        if (row.$group)
            return [row.$col];
        return this.cols;
    },
    _swapColumns(col1, col2) {
        const ord = col1.order;
        col1.order = col2.order;
        col2.order = ord;
        this.__write(this.settingsId + '/column/' + col1.name + '/order', col1.order);
        this.__write(this.settingsId + '/column/' + col2.name + '/order', col2.order);
    },
    insertBeforeNode(item, items) {
        if (!Array.isArray(items))
            items = [items];
        let parent = this.getParentItems(item);
        let idx = parent.indexOf(item);
        parent.splice(idx, 0, ...items);
    },
    insertAfterNode(item, items) {
        if (!Array.isArray(items))
            items = [items];
        let parent = this.getParentItems(item);
        let idx = parent.indexOf(item);
        parent.splice(idx + 1, 0, ...items);

    },
    appendChildNode(item, items) {
        if (!Array.isArray(items))
            items = [items];
        if (item.$role) {
            this.dataSet.push(...items);
        } else {
            if (!item.items) {
                item.items = items;
            } else {
                item.items.push(...items);
            }
            item.$expanded = true;
        }
    },

    _onDragStart(e) {
        const el = e.path.find(p => p.row);
        if (el && (this.allowDrag || el.row.drag)) {
            e.dataTransfer.setDragImage(new Image(), 0, 0);
            try {
                e.dataTransfer.setData('text/plain', JSON.stringify(el.row));
            } catch (err) {
                e.dataTransfer.setData('text/plain', JSON.stringify(el.row, prop => {
                    return prop instanceof Object ? undefined : prop;
                }));
            };
            ODA._draggableRows = this._draggableRows = this.selection.includes(el.row) ? this.selection : [el.row];
        }
    },

    _onDragEnter(e) {

    },
    _onDragLeave(e) {
        const el = e.path.find(p => p.row);
        if (el) {
            el.row.$dropMode = '';
        }
        clearTimeout(this._expandTimer);
        this._expandTimer = null;
        this._dropMode = null;
    },


    _onDragOver(e) {
        e.stopPropagation();
        let dropEffect = 'none';
        if (this._draggableColumn) {
            e.preventDefault();
            return
        }
        const el = e.path.find(p => p.row);
        if (!el)
            return;
        const row = el.row;
        if (this._draggableRows && !row.$group) {
            if (!this._draggableRows.includes(row)) {
                e.preventDefault();

                if (!this._expandTimer) {
                    this._expandTimer = setTimeout(() => {
                        if (!row.$expanded && row.items && row.items.length > 0) {
                            row.$expanded = true;
                            this._refresh();
                        }
                    }, 800)
                }

                if (row.drop !== false && (this.allowDrop !== 'none' || row.drop)) {
                    if (this._draggableRows && !e.ctrlKey) {
                        dropEffect = 'move';
                    } else {
                        dropEffect = 'copy';
                    }
                }
                if (dropEffect !== 'none') {
                    const step = (this.treeStep * row.$level) - this.treeStep;
                    //this.updateStyles({'--margin-left-drag': (step > 0 ? step : 0) + 'px'});

                    let rect = el.getBoundingClientRect();
                    rect = (e.y - rect.y) / rect.height;

                    if (this.allowDrop === 'simple' ? rect < .5 : rect < .25) {
                        el.row.$dropMode = 'top';
                        this._dropMode = 'top'
                    } else if (rect < .75 && this.allowDrop === 'extended') {
                        el.row.$dropMode = 'in';
                        this._dropMode = 'in';
                        if (this._checkAllowDropIn(row)) {
                            dropEffect = 'none';
                        }

                    } else {
                        el.row.$dropMode = 'bottom';
                        this._dropMode = 'bottom'
                    }
                }
            } else {
                this._dropMode = 'none';
                el.row.$dropMod$dropMode = 'none';
            }
            e.dataTransfer.dropEffect = dropEffect;
        }
        return dropEffect
    },
    _checkAllowDropIn(row) {
        const contains = !!(this._draggableRows && row.items && Array.isArray(row.items) && row.items.find(i => this._draggableRows.includes(i)));
        // const isLoading = !!(row.items && row.items.length === 1 && row.items[0].template === this.loadingTemplate); // for DELETION
        const isLoading = !!(row.items && row.items.length === 1 && !Object.keys(row.items[0]).length); // is not tested
        return contains || isLoading;
    },

    _onDrop(e) {
        const el = e.path.find(p => p.row);
        if (!el)
            return;
        const row = el.row;
        if (row.$role)
            return;
        e.preventDefault();

        if (this._draggableColumn) {
            this._draggableColumn.hidden = true;
            this.__write(this.settingsId + '/column/' + this._draggableColumn.name + '/hidden', true);
            return;
        }
        let dragSources = this._draggableRows;
        try {
            if (Array.isArray(dragSources) && dragSources.length > 0) {
                if (!e.ctrlKey) {
                    this.deleteItems(i => dragSources.includes(i))
                }
                switch (this._dropMode) {
                    case 'top': {
                        this.insertBeforeNode(row, dragSources);
                    }
                        break;
                    case 'in': {
                        this.appendChildNode(row, dragSources);
                    }
                        break;
                    case 'bottom': {
                        this.insertAfterNode(row, dragSources);
                    }
                        break;
                }
            }
        } catch (err) {
            console.error(err);
        } finally {
            el.row.$dropMode = '';
            this._dropMode = null;
            this._refresh();
        }
    },
    find(callback) {
        const find = (items) => {
            let res = items.find(callback);
            if (!res) {
                res = items.find(i => {
                    return i.items && i.items.length && find(i.items);
                });
                if (res)
                    res = find(res.items);
            }
            return res;
        };
        return find(this.dataSet);
    },
    filter(callback) {
        const find = (items) => {
            const res = items.filter(i => i.items).reduce((res, item) => {
                res.push(...find(item.items));
                return res;
            }, []);
            res.push(...items.filter(callback));
            return res;
        };
        return find(this.dataSet);
    },
    deleteItems(callback, once = false) {
        const items = once ? [this.find(callback)] : this.filter(callback);
        items.forEach(i => {
            const array = i.$parent && i.$parent.items || this.dataSet;
            const idx = array.indexOf(i);
            if (~idx) {
                array.splice(idx, 1);
            }
        })
    },
    _onDragEnd(e) {
        // this.updateStyles({'--drop-transition': 'background-color  1s'});
        // this.async(() => {
        // this._draggableRows = [];
        // this.async(() => {
            this._draggableRows = [];
            // this.updateStyles({'--drop-transition': 'background-color  0s'});
        // }, 500)
        // }, 100);
    },
    _dblclick(e) {
        const el = e.path.find(p => p.row);
        if (el)
            this.fire('row-dblclick', el.row);
    },
    _onRowContextMenu(e) {
        const el = e.path.find(p => p.row);
        if (el)
            this.fire('row-contextmenu', el.row);
    },
    expandAll(deep) {
        return this._setExpanded2All(true, deep);
    },
    collapseAll(deep) {
        return this._setExpanded2All(false, deep);
    },
    async _setExpanded2All(val = true, deep = 100, force = false) {
        if (this.expanding && !force) return;
        if (deep <= 0) {
            this.expanding = false;
            return;
        }
        this.expanding = true;
        const items = this.items;
        this._setExpanded_R(items, val);
        return new Promise(async resolve => {
            if (!this._checkExpanded_R(items, val)) {
                // const loading = this._checkLoading_R();
                resolve(await this._setExpanded2All(val, /*loading ? deep :*/ --deep, true));
            } else {
                this.expanding = false;
                this._refresh();
                resolve(true);
            }
        });
    },
    _setExpanded_R(items = this.items, val) {
        items.forEach(i => {
            if (i.items) {
                i.$expanded = val;
                this._setExpanded_R(i.items, val);
            }
        });
    },
    _checkExpanded_R(items = this.items, val) {
        return items.every(i => {
            if (i.items) return i.$expanded === val && this._checkExpanded_R(i.items, val);
            else return true;
        });
    },
    _checkLoading_R(items = this.items) { // to do (loadingTemplate is not used now)
        return items.some(i => {
            if (i.items) return i.template === this.loadingTemplate && this._checkLoading_R(i.items);
            else return false;
        });
    },
    _updateStyles() {
        this._stylesUpdater = !this._stylesUpdater;
    }
});

import '../../buttons/button/button.js';
import '../../buttons/checkbox/checkbox.js';
ODA({
    is: "oda-table-group-panel", template:/*html*/`
         <style>
            :host {
                padding: 4px;
                @apply --header;
            }

            .item {
                max-width: 120px;
                margin: 4px;
                padding: 4px 6px;
                align-items: center;
            }
            oda-icon {
                cursor: pointer;
            }
        </style>
        <div class="horizontal">
            <div class="flex vertical">
                <span ~if="groups.length === 0">drag&drop the column for grouping here</span>
                <div class="no-flex horizontal">
                    <div class="item shadow content no-flex horizontal" ~for="column in groups">
                        <span class="label flex" ~text="column.label || column.name"></span>
                        <oda-icon icon="icons:close" :icon-size="Math.round(iconSize * .5)" @tap="_close" :column="column"></oda-icon>
                    </div>
                </div>
            </div>
            <oda-table-settings></oda-table-settings>
        </div>
    `,
    props: {
        iconSize: {
            shared: true
        },
        groups: [],
        table: Object
    },
    listeners: {
        dragover: '_dragover',
        drop: '_drop'
    },
    _close(e) {
        this.table.removeGroup(e.target.column);
        // this.groups.splice(this.groups.indexOf(e.target.column), 1);
    },
    _dragover(e) {
        if (this.table._draggableColumn && !this.groups.includes(this.table._draggableColumn)) {
            e.preventDefault();
        }
    },
    _drop(e) {
        this.groups.push(this.table._draggableColumn);
    }
});

ODA({
    is: "oda-table-settings", template:/*html*/`
        <style>
            :host>div{
                overflow: auto;
                right: 0;
                top:0;
                position: absolute;
                z-index: 10;
                /*min-width: 280px;*/
                width: 300px;
                height: 100%;
                @apply --shadow;
            }

        </style>
        <oda-button title="Settings" icon="icons:settings" @tap="_tap"></oda-button>
        <div ~if="opened" class="vertical flex content">
            <oda-table-hide-column></oda-table-hide-column>
        </div>
    `,
    props: {
        iconSize: {
            shared: true
        },
        opened: {
            type: Boolean,
            reflectToAttribute: true,
            value: false,
        },
        table: null
    },
    listeners: {
        mousedown: '_leave'
    },
    attached() {
        this.listen('mousedown', '_leave', { target: window });
    },
    _tap(e) {
        this.opened = !this.opened;
    },
    _leave(e) {
        if (e.path.includes(this)) {
            e.stopPropagation();
        } else {
            this.opened = false;
        }
    },
    detached() {
        this.unlisten('mousedown', '_leave', { target: window });
    }
});

ODA({
    is: "oda-table-hide-column", template:/*html*/`
        <style>
            :host {
                flex: 1;
            }
            .list {
                padding: 5px;
            }
        </style>
        <div class="horizontal no-flex header list">
            <oda-checkbox ref="check" :value="items.every(i=> !i.col.hidden)" @tap="_onSelectAll"></oda-checkbox>
            <span class="flex label center" style="font-size: 10pt; padding: 0px 8px;">(show all)</span>
        </div>
        <div>
            <div ~for="item in items" class="list  no-flex horizontal">
                <oda-checkbox :value="!item.col.hidden"  @value-changed="_onChange($event, item)"></oda-checkbox>
                <span class="label center" style="font-size: 10pt; padding: 0px 8px;">{{getLabel(item)}}</span>
            </div>
        </div>
    `,
    props: {
        items: [],
        table: null,
    },
    attached() {
        this.items = this.table.columns.map(i => {
            return {
                label: i.label,
                name: i.name,
                hidden: i.hidden,
                col: i
            }
        })
    },
    getLabel(item) {
        return item.label || item.name;
    },
    _onChange(e, item) {
        item.col.hidden = !e.detail.value;
        this.table.__write(this.table.settingsId + '/column/' + item.col.name + '/hidden', !e.detail.value);
    },
    _onSelectAll(e) {
        // this.async(() => {
        this.items.forEach(i => {
            i.col.hidden = !this.$refs.check.value;
            this.table.__write(this.table.settingsId + '/column/' + i.col.name + '/hidden', !this.$refs.check.value)
        });
        // },0)
    },
});

cells: {
    ODA({
        is: "oda-table-cell-base", template:/*html*/`
    <style>
        :host{
            @apply --horizontal;
            align-items: center;
            overflow: hidden;
            text-overflow: ellipsis;
            min-height: 1px;
            min-width: 1px;
        }
        :host *{
            text-overflow: ellipsis;
            position: relative;
        }
        oda-icon{
            cursor: pointer;
        }
    </style>`,
        props: {
            isFooter: false,
            column: {},
            item: {},
            table: {
                type: Object,
                shared: true
            },
            iconSize: {
                default: 24,
                shared: true
            }
        }
    });

    ODA({
        is: "oda-table-cell", extends: "oda-table-cell-base", template:/*html*/`
    <div class="flex" ~is="template || div" :column="column" :item="item" ~text="template === 'span'?value:''" :title="value" style="overflow: hidden"></div>`,
        props: {
            template: 'span',
            value: {
                get() {
                    return this.item[this.column.name]
                }
            }
        }
    });

    ODA({
        is: 'oda-table-expand', extends: "oda-table-cell-base", template:/*html*/`
        <style>
            .step {
                height: 100%;
            }
            .step > .end-step{
                position: absolute;
                right: 0;
                height: 100%;
            }
        </style>
        <div class="step no-flex" ~style="{width: (item.$level || 0) * iconSize + 'px', ...stepStyle}">
            <div class="end-step" ~style="endLineStyle"></div>
        </div>
        <div ~if="!hideEmptyIcon" class="no-flex horizontal icon" @dblclick.stop.prevent @tap.stop @down.stop.prevent="_toggleExpand" ~style="{width: iconSize+'px', height: iconSize+'px', height: '100%'}">
            <oda-icon class="no-flex" ~if="allowExpand"  :icon></oda-icon>
        </div>`,
        props: {
            hideEmptyIcon: false,
            allowExpand: {
                type: Boolean,
                get() {
                    return this.item && !this.item.hideExpander && this.item.items && this.item.items.length > 0;
                }
            },
            expandedIcon: 'icons:chevron-right:90',
            collapsedIcon: 'icons:chevron-right',
            expandingIcon: 'odant:spin',
            icon: String,
            stepStyle: {},
            endLineStyle: {}
        },
        observers: [
            function itemChanged(item) { // временное решение, разобраться с реактивностью
                let expanded = item.$expanded;
                Object.defineProperty(item, '$expanded', {
                    set: (v) => {
                        expanded = v;
                        this._setIcon(expanded);
                    },
                    get: () => {
                        return expanded;
                    },
                    configurable: true
                });
                this._setIcon(expanded);

            }
        ],
        _setIcon(v) {
            this.icon = !v
                ? this.collapsedIcon
                : v === 'expanding'
                    ? this.expandingIcon
                    : this.expandedIcon;
        },
        _toggleExpand(e, d) {
            if (!this.item.hideExpander) {
                this.item.$expanded = this.item.$expanded ? false : 'expanding';
                this.table && this.table._refresh();
            }
        }
    });

    tree: {
        ODA({
            is: "oda-table-cell-tree", extends: 'oda-table-expand', template:/*html*/`
            <style>
                :host *{
                  --line-border: 1px solid silver;
                    align-items: center;
                    background-color: unset;
                }
                :host:last-child .step {
                    height: 50%;
                }
                .icon{
                    height: auto;
                    align-items: center;
                    align-content: center;
                }
            </style>
            <oda-table-check class="no-flex" ~if="(item?.allowCheck && item.allowCheck !== 'none') || (table?.allowCheck && table.allowCheck !== 'none' && !item.hideCheckbox)" :column="column" :item="item"></oda-table-check>
            <div ~is="item?[column.name+'.template'] || item?.template || column?.template || table?.defaultTemplate || 'span'" :column="column" :item="item" class="flex horizontal cell">{{item[column.name]}}</div>`,
            props: {
                stepStyle: {
                    type: Object,
                    get() {
                        const table = this.table;
                        if (this.iconSize && table && table.showTreeLines) {
                            let color = table.treeLineStyle.color || 'rgba(0, 0, 0, 0.25)';
                            let width = table.treeLineStyle.width || '1px';
                            return this._buildStepStyle(this.iconSize, color, width);
                        } else return {};
                    }
                },
                endLineStyle: {
                    get() {
                        const table = this.table;
                        if (this.iconSize && table && table.showTreeLines) {
                            const width = table.treeLineStyle.width || 1;
                            const halfWidth = Math.round(width/2);
                            const pre = `50% - ${halfWidth}px`;
                            const post = `${pre} + ${width}px`;
                            const color = table.treeLineStyle.color || 'rgba(0, 0, 0, 0.25)'
                            return {
                                width: this.iconSize / 2 + 'px',
                                background: `linear-gradient(0deg, transparent 0, transparent calc(${pre}), ${color} calc(${pre}), ${color} calc(${post}), transparent calc(${post}))`
                            }
                        }
                    }
                }
            },
            _buildStepStyle(step, color, width) {
                const h = step / 2;
                const hWidth = Math.round(width / 2);
                let str = '';
                const lineStart = h - hWidth;
                const lineEnd = lineStart + width;
                str =`, transparent, transparent ${lineStart}px, ${color} ${lineStart}px, ${color} ${lineEnd}px, transparent ${lineEnd}px, transparent ${step}px`;
                return {
                    background: `repeating-linear-gradient(90deg ${str})`,
                }
            }
        });

        ODA({
            is: "oda-empty-tree-cell", extends: "oda-icon, oda-table-cell-base", template: `
        <label class="label flex">nothing</label>`,
            props: {
                icon: 'notification:do-not-disturb'
            }
        });
        // ODA({is: "oda-table-loading", extends: "oda-icon, oda-table-cell-base", template:`
        //     <oda-icon :icon="icon"></oda-icon>
        //     <label class="label flex">loading...</label>`,
        //     props: {
        //         icon: 'odant:spin'
        //     }
        // });

        ODA({
            is: "oda-table-check", extends: 'oda-table-cell-base', template: `
            <oda-icon class="no-flex" @down.stop.prevent="_toggleChecked" :icon="_getCheckedIcon(item)" :icon-size ~style="{padding: Math.round(iconSize*.2)+'px'}"></oda-icon>`,
            props: {
                allowCheck() {
                    return this.item.allowCheck && !this.item.hideCheckbox;// (this.item.allowCheck && this.item.allowCheck !== 'none') || (this.item.hideCheckbox ? 'none' : this.table.allowCheck)
                },
                checkMode() {
                    return this.item.hideCheckbox ? 'none' : this.table.allowCheck;
                }
            },
            _getCheckedIcon(item) {
                if (item.checked)
                    return item.checked === true ? 'icons:check-box' : 'icons:check-box-indeterminate';
                return 'icons:check-box-outline-blank'
            },
            _toggleChecked(e) {
                switch (this.checkMode) {
                    case 'none': return;
                    case 'single':
                        this.item.checked = !(!!this.item.checked);
                        this.render();
                        return;
                    case 'down':
                    case 'up':
                    case 'double':
                        this.item.checked = !(!!this.item.checked);
                        break;
                    default: {
                        if (!this.item.checked) {
                            if (this.item.items && this.item.items.length)
                                this.item.checked = 'indeterminate';
                            else
                                this.item.checked = true;
                        }
                        else if (this.item.checked === true)
                            this.item.checked = false;
                        else
                            this.item.checked = true;
                    }
                }
                if (this.checkMode === 'double' || this.checkMode === 'down') {
                    checkChildren(this.item);
                }
                if (this.checkMode === 'double' || this.checkMode === 'up') {
                    let parent = this.table.getParentItem(this.item);
                    while (parent) {
                        updateChecked(parent);
                        parent = this.table.getParentItem(parent);
                    }
                }
                this.render();

                function checkChildren(item) {
                    (item.items || []).forEach(i => {
                        if (item.checked !== 'indeterminate') {
                            i.checked = item.checked;
                            checkChildren(i);
                        }
                    })
                }
                function updateChecked(item) {
                    item.checked = (item.items || []).reduce((res, i) => {
                        if (res === 'indeterminate')
                            return res;
                        else if (res === undefined)
                            res = i.checked ? i.checked : false;
                        else if (res !== ((i.checked === undefined) ? false : i.checked))
                            res = 'indeterminate';
                        return res;
                    }, undefined);
                }
            }
        });
    }


    ODA({
        is: "oda-table-error", extends: "oda-table-cell-base", template:/*html*/`
    <oda-icon icon="icons:error"></oda-icon>
    <span class="label flex" :text="item.error"></span>`
    });



    ODA({
        is: "oda-table-header", extends: 'oda-table-expand', template: /*html*/`
    <style>
        :host{
            @apply --header;
            @apply --flex;
            @apply --horizontal;
            cursor: pointer;
            justify-content: space-around;
            overflow: hidden;
            align-items: initial !important;
            box-sizing: border-box;
        }
        .split{
            cursor: col-resize;
            border: 2px solid transparent;
            transition: border-color .5s;
            /*border-color: silver;*/
        }
        .split:hover{
            border-color: silver;
        }
        oda-icon{
            opacity: .5;
        }
        :host(:hover)>oda-icon{
            opacity: 1;
        }
        input{
            width: 0px;
            font-size: x-small;
            opacity: .8;
        }
        oda-button{
            padding: 0px;
            min-width: 16px;
            min-height: 16px;
        }
        span{
            margin: 4px 0px;
            text-overflow: ellipsis;
        }
        div{
            overflow: hidden;
        }
        oda-table-header{
            box-sizing: content-box;
            /*border: 1px solid red;*/
        }
        .sub-cols{
            border-top: 1px solid var(--dark-background);
        }
        .sub-cols>oda-table-header:not(:nth-child(1)){
            border-left: 1px solid var(--dark-background);
        }
        oda-icon:after{
            content: attr(sort);
            font-size: small;
            color: blue;
        }
    </style>
    <div class="flex vertical" style="cursor: pointer" @tap.stop="_sort">
        <div class="flex horizontal" ~if="column.name" ~style="{flexDirection: column.fix === 'right'?'row-reverse':'row'}">
            <div class="flex horizontal" style="align-items: center;">
                <oda-icon :icon="icon" ~if="column.items && column.items.length" @tap.stop.prevent="_toggleExpand"></oda-icon>
                <span class="label flex" :text="column.label || column.name" draggable="true" @dragover="_dragover" @dragstart="_dragstart" @dragend="_dragend" @drop="_drop"></span>
                <oda-icon :show="allowSort && sortIcon" title="sort" :icon="sortIcon" :sort="sortSubIcon"></oda-icon>
            </div>

            <div class="split" @tap.stop @track="_track" ~style="{minHeight: iconSize+'px'}"></div>
        </div>
        <div class="flex horizontal" ~if="!expanded && column.name && showFilter" style="align-items: center">
            <input class="flex" ::value="filter" @tap.stop>
            <oda-button :icon-size="Math.round(iconSize * .3)" icon="icons:filter"></oda-button>
        </div>
        <div class="flex sub-cols horizontal" ~if="expanded">
            <oda-table-header ~for="col in column.items" :key="col.id" :item="col" :column="col" :show-filter="showFilter" ~style="{width: col.width?col.width+'px':'auto' }"></oda-table-header>
        </div>
        <div class="flex" ~if="!column.name"></div>
    </div>`,
        props: {
            allowSort: false,
            hideEmptyIcon: true,
            _dragging: false,
            showFilter: false,
            filter: {
                type: String,
                set(val) {
                    if (val)
                        this.column.$filter = val;
                    else
                        this.column.$filter = null
                }
            },
            sortIcon: {
                type: String,
                get() {
                    switch (this.column.$sort) {
                        case 1:
                            return 'icons:arrow-drop-up';
                        case -1:
                            return 'icons:arrow-drop-down';
                        default:
                            return '';
                    }
                }
            },
            expanded: {
                get() {
                    return !!this.column.$expanded;
                }
            },
            sortSubIcon: {
                type: String,
                get() {
                    if (this.table.sorts)
                        return this.table.sorts.indexOf(this.column) + 1;
                    return '';
                }
            }
        },
        updated() {
            if (this.table && this.table.set && this.column.$parent) {
                this.table.set(this.column, 'width', Math.round(this.offsetWidth));
            }
            // console.log(this.column)
        },
        listeners: {
            contextmenu: '_menu'
        },
        _toggleExpand(e, d) {
            this.column.$expanded = !this.column.$expanded;
            // this.table._refresh();
        },
        _dragstart(e) {
            this.table._draggableColumn = this.column;
        },
        _dragend(e) {
            this.table._draggableColumn = null;
        },
        _dragover(e) {
            if (this.table._draggableColumn && this.table._draggableColumn !== this.column) {
                e.preventDefault();
            }
        },
        _drop(e) {
            this.table._swapColumns(this.table._draggableColumn, this.column);
            this.table._draggableColumn = null;
        },
        _track(e, d) {
            switch (e.detail.state) {
                case 'start': {
                    this.column.width = Math.round(this.offsetWidth);
                } break;
                case 'track': {
                    const delta = e.detail.ddx * (this.column.fix === 'right' ? -1 : 1);
                    let p = this.column;

                    while (p) {
                        p.width = Math.round(p.width + delta);
                        p = p.$parent;
                    }
                    const setChildrenWidth = (col) => {
                        if (col.items) {
                            const w = col.items.reduce((res, a) => { res += (a.width || 0); return res; }, 0);
                            if (w && col.width !== w) {
                                const k = col.width / w;
                                col.items.forEach(c => {
                                    c.width = c.width * k;
                                    setChildrenWidth(c);
                                })
                            }
                        }
                    };
                    setChildrenWidth(this.column);
                    this.table._updateStyles();
                } break;
                case 'end': {
                    let col = this.column;
                    while (col) {
                        this.table.__write(this.table.settingsId + '/column/' + col.name + '/width', col.width);
                        col = col.$parent;
                    }
                } break;
            }
        },
        _menu(e) {
            const menu = [
                {
                    label: this.table.groups.includes(this.column) ? 'Ungrouping' : 'Grouping' + ' by this column', icon: 'icons:open-in-browser', execute() {
                        if (this.table.groups.includes(this.column))
                            this.table.removeGroup(this.column);
                        else
                            this.table.addGroup(this.column);
                    }
                },
                {
                    label: 'Hide this column', icon: 'icons:visibility-off', execute() {
                        this.column.hidden = !this.column.hidden;
                        this.table.__write(this.table.settingsId + '/column/' + this.column.name + '/hidden', this.column.hidden);
                    }
                },
                {
                    label: (this.table.showGroupingPanel ? 'Hide' : 'Show') + ' grouping panel', icon: 'icons:credit-card', group: 'more', execute() {
                        this.table.showGroupingPanel = !this.table.showGroupingPanel;
                    }
                },
                {
                    label: (this.table.showFilter ? 'Hide' : 'Show') + ' filter row', icon: 'icons:filter', execute() {
                        this.table.showFilter = !this.table.showFilter;
                    }
                }
            ];
            ODA.showDropdown('oda-menu', { parent: this, iconSize: this.iconSize, items: menu });
        },
        _sort(e) {
            if (this.allowSort) {
                e.stopPropagation();
                switch (this.column.$sort) {
                    case 1: {
                        this.column.$sort = -1;
                    } break;
                    case -1: {
                        this.column.$sort = 0;
                    } break;
                    default: {
                        this.column.$sort = 1;
                    } break;
                }
                const idx = this.table.sorts.indexOf(this.column);
                if (!this.column.$sort) {

                    if (idx > -1) {
                        this.table.sorts.splice(idx, 1);
                    }
                }
                else {
                    if (idx < 0) {
                        this.table.sorts.push(this.column);
                    }
                    else {
                        this.table.sorts = Array.from(this.table.sorts);
                    }
                }
            }
        }
    });

    ODA({
        is: "oda-table-cell-group", extends: 'oda-table-expand', template:/*html*/`
    <style>
        :host{
            justify-content: left;
            @apply --header;
            position: sticky;
            position: -webkit-sticky;
            left: 0px;
            top: 0px;
            position: sticky;
        }
        span{
            @apply --flex;
        }
    </style>
    <span :text="\`\${item.name}:\`" style="font-size: small; margin-right: 4px;" class="no-flex"></span>
    <span :text="item.value" style="font-weight: bold; margin-right: 4px;" class="no-flex"></span>
    <span ~if="item.items" :text="\`[\${item.items.length}]\`" class="no-flex"></span>`
    });

    ODA({
        is: "oda-table-footer", extends: 'oda-table-cell', template:/*html*/`
   <style>
        :host{
            justify-content: flex-end;
            padding: 4px;
            @apply --dark;
            text-align: right;
            font-size: smaller;
        }
    </style>`
    });
}
groups: {
    ODA({
        is: "oda-table-group-panel", template:/*html*/`
         <style>
            :host {
                padding: 4px;
                @apply --header;
            }

            .item {
                max-width: 120px;
                margin: 4px;
                padding: 4px 6px;
                align-items: center;
            }
            oda-icon {
                cursor: pointer;
            }
        </style>
        <div class="horizontal">
            <div class="flex vertical">
                <span ~if="groups.length === 0">drag&drop the column for grouping here</span>
                <div class="no-flex horizontal">
                    <div class="item shadow content no-flex horizontal" ~for="column in groups">
                        <span class="label flex" ~text="column.label || column.name"></span>
                        <oda-icon icon="icons:close" :icon-size="Math.round(iconSize * .5)" @tap="_close" :column="column"></oda-icon>
                    </div>
                </div>
            </div>
            <oda-table-settings></oda-table-settings>
        </div>
    `,
        props: {
            iconSize: {
                shared: true
            },
            groups: {
                type: Array,
                value() {
                    return []
                },
            },
            table: null
        },
        listeners: {
            dragover: '_dragover',
            drop: '_drop'
        },
        _close(e) {
            this.table.removeGroup(e.target.column);
            // this.groups.splice(this.groups.indexOf(e.target.column), 1);
        },
        _dragover(e) {
            if (this.table._draggableColumn && !this.groups.includes(this.table._draggableColumn)) {
                e.preventDefault();
            }
        },
        _drop(e) {
            this.groups.push(this.table._draggableColumn);
        }
    });

    ODA({
        is: "oda-table-settings", template:/*html*/`
        <style>
            :host>div{
                overflow: auto;
                right: 0;
                top:0;
                position: absolute;
                z-index: 10;
                /*min-width: 280px;*/
                width: 300px;
                height: 100%;
                @apply --shadow;
            }

        </style>
        <oda-button title="Settings" icon="icons:settings" @tap="_tap"></oda-button>
        <div ~if="opened" class="vertical flex content">
            <oda-table-hide-column></oda-table-hide-column>
        </div>
    `,
        props: {
            iconSize: {
                shared: true
            },
            opened: {
                type: Boolean,
                reflectToAttribute: true,
                value: false,
            },
            table: null
        },
        listeners: {
            mousedown: '_leave'
        },
        attached() {
            this.listen('mousedown', '_leave', { target: window });
        },
        _tap(e) {
            this.opened = !this.opened;
        },
        _leave(e) {
            if (e.path.includes(this)) {
                e.stopPropagation();
            } else {
                this.opened = false;
            }
        },
        detached() {
            this.unlisten('mousedown', '_leave', { target: window });
        }
    });

    ODA({
        is: "oda-table-hide-column", template:/*html*/`
        <style>
            :host {
                flex: 1;
            }
            .list {
                padding: 5px;
            }
        </style>
        <div class="horizontal no-flex header list">
            <oda-checkbox ref="check" :value="items.every(i=> !i.col.hidden)" @tap="_onSelectAll"></oda-checkbox>
            <span class="flex label center" style="font-size: 10pt; padding: 0px 8px;">(show all)</span>
        </div>
        <div>
            <div ~for="item in items" class="list  no-flex horizontal">
                <oda-checkbox :value="!item.col.hidden"  @value-changed="_onChange($event, item)"></oda-checkbox>
                <span class="label center" style="font-size: 10pt; padding: 0px 8px;">{{getLabel(item)}}</span>
            </div>
        </div>
    `,
        props: {
            items: {
                type: Array,
                value: () => []
            },
            table: null,
        },
        attached() {
            this.items = this.table.columns.map(i => {
                return {
                    label: i.label,
                    name: i.name,
                    hidden: i.hidden,
                    col: i
                }
            })
        },
        getLabel(item) {
            return item.label || item.name;
        },
        _onChange(e, item) {
            item.col.hidden = !e.detail.value;
            this.table.__write(this.table.settingsId + '/column/' + item.col.name + '/hidden', !e.detail.value);
        },
        _onSelectAll(e) {
            // this.async(() => {
            this.items.forEach(i => {
                i.col.hidden = !this.$refs.check.value;
                this.table.__write(this.table.settingsId + '/column/' + i.col.name + '/hidden', !this.$refs.check.value)
            });
            // },0)
        },
    });
}