import '../form-layout/form-layout.js';
import '../splitter/splitter.js';
const DEFAULT_DRAWER_WIDTH = 300;
ODA({
    is: 'oda-app-layout', extends: 'oda-form-layout', template: /*html*/`
    <style>
        :host {
            @apply --flex;
            position: relative;
            overflow: hidden;
            height: 100%;
            touch-action: manipulation;
        }
        .main {
            transition: margin, filter 0.2s;
            @apply --content;
            overflow: hidden;
        }
        slot {
           min-width: 0px;
           overflow: hidden;
        }
        .stop-pointer-events * {
            pointer-events: none;
        }
        slot>*{
            @apply --flex;
        }
        ::slotted(*){
            @apply --flex;
        }
        .title{
            transition: margin-top 0.3s ease-in-out;
            align-items: center;
        }
        .main-container{
            @apply --flex;
            @apply --horizontal;
            overflow: hidden;
        }
        .bottom{
            background-color: whitesmoke;
        }
    </style>
    <div ref="appHeader" class="top title">
        <slot name="title"></slot>
    </div>
    <div class="main-container content" ~class="{'stop-pointer-events': size === 'min'}" ~style="{'zoom': size === 'min' ? '50%' : ''}">
        <app-layout-drawer pos="left">
            <slot name="left-drawer"></slot>
        </app-layout-drawer>
        <div @resize="_resize($event)" class="main vertical flex">
            <app-layout-toolbar :icon-size class="top" ~show="!hideToolbar && !siteMode"></app-layout-toolbar>
            <slot name="main"  class="vertical flex slot" style="overflow: hidden" ref="mainSlot"></slot>
            <slot name="bottom"  class="horizontal no-flex slot bottom" ref="bottomSlot"></slot>
        </div>
        <app-layout-drawer pos="right">
            <slot name="right-drawer"></slot>
        </app-layout-drawer>
    </div>
    <slot name="footer" class="horizontal no-flex slot bottom"></slot>`,
    props: {
        siteMode: false,
        max: 300,
        _mainWidth: {
            type: Number,
            shared: true
        },
        hideToolbar: true,
        hideOnScroll: {
            default: false,
            set(n) {
                if (n) {
                    if (!this._hideOnScroll)
                        this.listen('wheel', '_scroll', { target: window, useCapture: true });
                    this._hideOnScroll = true;
                }
            }
        },
    },
    _resize(e) {
        this._mainWidth = e?.target.offsetWidth;
    },
    attached() {
        const me = this.unique && this.getModals().find(i => i.unique === this.unique);
        if (me) {
            me._top();
            this.remove();
            return;
        }

        if (this.hideOnScroll) {
            this.listen('wheel', '_scroll', { target: window, useCapture: true });
            this._hideOnScroll = true;
        }
        this.getModals().forEach(el => {
            if (el.modal) {
                if (el.pos.left === this.pos.left) {
                    this.pos.x = this.pos.x + this.iconSize;
                }
                if (el.pos.top === this.pos.top) {
                    this.pos.y = this.pos.y + this.iconSize;
                }
                this._fixPos();
            }
        });
        this._top();
    },
    detached() {
        if (this._hideOnScroll)
            this.unlisten('wheel', '_scroll', { target: window, useCapture: true });
    },
    _scroll(e) {
        this.debounce('hide-header', () => {
            requestAnimationFrame(() => {
                let h = this.$refs.appHeader;
                let t = e.target;
                if (e.detail && e.detail.value === 'clearScroll') {
                    h.style.marginTop = '0';
                    return;
                }
                if (t.slot !== 'main') return;
                if (e.wheelDelta >= 0 || e.detail > 0) {
                    h.style.marginTop = '0';
                } else {
                    h.style.marginTop = `-${h.offsetHeight}px`;
                }
            });
        }, 100);
    }
});

ODA({
    is: 'app-layout-toolbar', template: /*html*/`
    <style>
        :host{
            @apply --no-flex;
            @apply --horizontal;
            border-bottom: 1px solid silver;
            align-items: center;
        }
    </style>
    <slot name="top-left" class="horizontal no-flex" style="overflow: hidden; justify-content: flex-start;"></slot>
    <slot name="top-center" class="horizontal flex" style="overflow: hidden; justify-content: center;"></slot>
    <slot name="top-right" class="horizontal no-flex" style="overflow: hidden; justify-content: flex-end; flex-shrink: 0;"></slot>`,
    props: {
        iconSize: {
            default: 24,
            shared: true
        }
    }
});

ODA({
    is: 'app-layout-drawer', template: /*html*/`
    <style>
        :host {
            @apply --no-flex
            position: relative;
            transition: transform 0.2s;
            @apply --horizontal;
            /*overflow: hidden;*/
            opacity: 0;
            transition: opacity 0.3s;
            flex-direction: row{{pos === 'right'?'-reverse':''}};
        }
        .drawer{
            height: 100%;
            position: relative;
            overflow: hidden;
        }
        .buttons{
            z-index: 1;
            /*padding: 2px 2px;*/
            background-color: whitesmoke;
            @apply --vertical;
            align-items: center;
            justify-content: space-around;
        }
        slotted(:not([focused])){
            display: none;
        }
        :host([hidden]){ /* todo: должно работать от глобального стиля */
            display: none !important;
        }
    </style>
    <div class="buttons" ~if="!hidden">
        <oda-button :rotate="pos ==='left'?-90:90" ~style="{transform: \`rotate(\${pos==='right'?-90:90}deg)\`}"  ~for="top"  @tap="_click(item)" :icon="item?.icon || item?.getAttribute('icon') || 'icons:menu'" :focused="focused === item"></oda-button>
        <div class="flex"></div>
        <oda-button :rotate="pos ==='left'?-90:90" ~style="{transform: \`rotate(\${pos==='right'?-90:90}deg)\`}"  ~for="down"  @tap="_click(item)" :icon="item?.icon || item?.getAttribute('icon') || \`symbols:d\${index}\`" :focused="focused === item"></oda-button>
    </div>
    <div ~show="focused"  @tap.stop class="horizontal drawer no-flex"
        ~style="{flexDirection: \`row\${pos === 'right'?'-reverse':''}\`, maxWidth: width + 'px', minWidth: width + 'px', position: persistent?'absolute':'relative'}">
        <div class="flex vertical" style="overflow: hidden;">
            <div class="horizontal" ~style="{flexDirection: \`row\${pos === 'right'?'-reverse':''}\`}" ~if="focused?.title">
                <div style="padding: 4px; align-self: center; text-overflow: ellipsis; overflow: hidden; white-space: nowrap; font-size: large;" class="flex" >{{focused?.title}}</div>
                <oda-button :size="iconSize" icon="icons:close" @tap="focused = null"></oda-button>
            </div>
            <slot style="overflow: hidden;" @slotchange="_slot" class="flex vertical"></slot>
        </div>
        <oda-splitter :sign="pos==='left'?-1:1" :max="persistent?max:0" ~if="!hideResize && !persistent" ::width="width"></oda-splitter>
    </div>
    `,
    props: {
        pos: {
            default: 'left',
            enum: ['left', 'right']
        },
        hideResize: false,
        persistent: false,
        width: 300,
        hidden: {
            get() {
                return !this.controls?.length
            },
            default: true,
            reflectToAttribute: true
        },
        _mainWidth: {
            type: Number,
            set(n) {
                // if (this.persistent){
                //     if (n>400)
                //         this.persistent = false;
                // }
                // else if (n+width<400){
                //     this.persistent = true;
                // }
            }
        },
        controls: Array,
        top() {
            return this.controls?.filter(i => i.getAttribute('down') === null) || [];
        },
        down() {
            return this.controls?.filter(i => i.getAttribute('down') !== null) || [];
        },
        iconSize: {
            default: 16,
            shared: true
        },
        focused: {
            type: Object,
            set(n) {
                for (let i of this.controls) {
                    i.hidden = i !== n;
                }

            }
        },
    },
    _click(item) {
        if (item.isButton) {
            item.fire('click');
        }
        else {
            this.focused = ((this.focused === item) ? null : item)
        }
    },
    _slot(e) {
        if (e.target.$domHost === this) return;
        this.controls = e.target.assignedNodes();
        if(this.controls.length === 0){
            this.hidden = true;
        }
        this.controls.forEach(c => {
            if(c.hasAttribute?.('bar-autofocus')){
                this.focused = this.focused || c;
            }else{
                c.setAttribute('hidden', '');
            }
        });
        this.async(() => {
            this.style.setProperty('opacity', 1);
        }, 100);
    }
});