Команда **console** позволяет выводить сообщения в окне пользовательской консоли.

Данная команда имеет следующий синтаксис:

```javascript
ODA.console.«модификатор»(«список параметров»)
```

Модификатор определяет тип отправляемого сообщения. В качестве модификатора можно использовать одно из следующих значений:

1. **log** — позволяет выводить сообщение в журнал пользовательских сообщений.
1. **error** — позволяет выводить сообщение в журнал ошибок.
1. **warn** — позволяет выводить сообщение в журнал предупреждений.
1. **status** — позволяет выводить сообщение в заголовок окна консоли.
1. **success** — позволяет выводить сообщение в журнал успешных действий.

По сути модификатор является свойством объекта **console**, в котором хранится указатель на соответствующую функцию. Каждая из этих функций служит для отправки сообщений определенного типа, текст которых передается в списке ее параметров.

Например, при использовании модификатора **log** в консоль будут отправляться сообщения обычного типа.

```javascript run_line_edit_loadoda_[my-component.js]
 ODA({
    is: 'my-component',
    template: `
        <label>Текст сообщения: <input ::value></label>
    `,
    props: {
        value: {
            default: 'Введите сообщение',
            set (n) {
                ODA.console.log(n);
            }
        }
    }
});
```

При отправки первого сообщения внизу страницы появляется специальная панель, на которой изначально отображается только строка заголовка окна консоли. Для того, чтобы отобразить сам список сообщений, необходимо нажать на кнопку с изображения стрелки вверх, расположенную в этом заголовке.

![Отображение списка сообщений](learn/guide/api/commands/images/ExpandConsole.png "Отобразить список сообщений")

В этом случае список всех сообщений будет развернут. Его можно свернуть обратно, нажав на туже самую кнопку, на которой изображение стрелки вверх изменится на изображение со стрелкой вниз.

![Сворачивание списка сообщений](learn/guide/api/commands/images/CollapsConsole.png "Свернуть список сообщений")

Нажатие на кнопку с изображением минуса очищает весь список сообщений.

![Очистка списка сообщений](learn/guide/api/commands/images/ClearConsole.png "Очистить список сообщений")

В этом случае область консоли со списком сообщений будет автоматически свернута.

Текст сообщения можно сформировать на основе нескольких параметров, значения которых указываются у любого модификатора последовательно через запятую.

Например,

```javascript run_line_edit_loadoda_[my-component.js]
 ODA({
    is: 'my-component',
    template: `
        <label>Текст 1: <input ::value='text[0]'> </label></br>
        <label>Текст 2: <input ::value='text[1]'> </label></br>
        <button @tap="onTap">Добавить сообщения</button>
    `,
    props: {
        text: {
            default: ['Текст 1', 'Текст 2'],
            reactive: true
        },
        onTap() {
            ODA.console.log(this.text[0], this.text[1]);
        }
    }
});
```

В этом случае текст сообщения будет формироваться как конкатенация значений всех указанных параметров, отделяя каждое значение от другого символом пробела.

Для повышения информационности сообщения разного типа в консоли отображаются разными цветами.

Например,

```javascript run_line_edit_loadoda_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <label>Текст сообщения: <input ::value='text'></label>
        <p ~for="item in items"><input name="console" type="radio" :value='item' @tap="onTap">{{item}}</p>
    `,
    props: {
        text: 'Мое сообщение',
        items: ['log', 'error', 'warn', 'status', 'success'],
    },
    onTap(e) {
        ODA.console[e.target.value](this.text);
    }
});
```

Обычные сообщения пользователю отображаются черным цветом, сообщения с типом **error** отображаются красным цветом, сообщения с предупреждениями (**warn**) отображаются оранжевый цветом, а сообщения об успешном выполнении действий (**success**) отображаются зеленым цветом. Статусные сообщения с типом **status** отображаются и в строке заголовка самой консоли и в списке сообщений черным цветом, так же как и обычные сообщения пользователю.

Нажатие на кнопку с изображением списка открывает или закрывает дополнительную панель фильтрации сообщений.

![Фильтрация сообщений](learn/guide/api/commands/images/FilterConsole.png "Отфильтровать сообщения")

На этой панели изначально отображаются сообщения любого типа (**all**). Щелкая по соответствующим кнопкам на этой панели, можно отобразить в окне только сообщения определенного типа:

1. Сообщения пользователю (**log**).
1. Сообщения об ошибках (**error**).
1. Сообщения с предупреждениями (**warn**).
1. Статусные сообщения (**status**).
1. Сообщения о выполнении успешных действиях (**success**).

При завершении работы с консолью её можно закрыть нажатием на кнопку с изображением крестика, расположенным в строке его заголовка.

![Закрытие консоли](learn/guide/api/commands/images/CloseConsole.png "Закрыть консоль")

<div style="position:relative;padding-bottom:48%; margin:10px">
    <iframe src="https://www.youtube.com/embed/gaN6WnNQiOQ?start=0" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen
    	style="position:absolute;width:100%;height:100%;"></iframe>
</div>
