Команда **ODA.pushMessage** позволяет отправлять уведомления пользователю.

Фактически она является псевдонимом команды **ODA.push** и объявлена следующим образом:

```javascript
ODA.pushMessage = ODA.push;
```

Этой команде передаются два параметра, второй из которых подвергается деструктуризации.

```javascript
ODA.push(title = 'Warning!', {tag = 'message', body, icon = '/web/res/icons/warning.png', image}={})
```

В результате этого у команды **ODA.pushMessage** будет 5 параметров:

1. **title** — заголовок уведомления.
1. **tag** — уникальный идентификатор уведомления, с помощь которого его можно заменить или удалить.
1. **body** — основной текст уведомления.
1. **icon** — URL не большого изображения, которое будет показано в уведомлении.
1. **image** — URL большого изображения, которое будет показано в уведомлении.

При первом посещении HTML-страницы пользователь должен разрешить отображение уведомлений для данного сайта, нажав на соответствующую кнопку в диалогом окне.

![Разрешение отображать уведомления](learn/guide/api/commands/images/requestNotification.png "Разрешить уведомления")

Если диалоговое окно будет закрыто или будет нажата кнопка **«Блокировать»**, то механизм отображения уведомлений для данного сайта будет заблокирован. Включить этот механизм снова можно будет только в настройках параметров безопасности сайта в браузере.

![Отключение блокировки уведомлений](learn/guide/api/commands/images/SiteSetting.png "Отключение блокировки уведомлений").

Например,

```javascript run_line_edit_loadoda_[my-component.js]
ODA({
    is: 'my-component',
    template: `
        <input ::value='title'> <br>
        <input ::value='body'> <br>
        <label>Уникальный номер</label> <input ::value='tag'> <br>
        <label>Иконка</label> <input ::value='icon'> <br>
        <button @tap="onTap">Нажми на меня</button>
    `,
    props: {
        title: 'Заголовок сообщения',
        body: 'Тело сообщения',
        tag: 'MyMessage 1',
        icon: 'https://odajs.org/learn/guide/api/commands/images/icon.png',
        image: 'https://odajs.org/learn/guide/api/commands/images/odajs.png',
    },
    onTap() {
        ODA.pushMessage(this.title, {tag: this.tag, body: this.body, icon: this.icon, image: this.image});
    }
});
```

Если при получении уведомления поместить его в центр уведомлений, то любые другие уведомления с тем же самым уникальным идентификатором **tag** отображаться пользователю уже не будут. Они будут помещаться в центр уведомлений под тем же самым номером, заменяя содержимое предыдущего уведомления.

Если закрыть уведомление, не помещая его в центр уведомлений, то, при отправке другого уведомления с тем же самым идентификатором, оно будет отображаться пользователю как новое.
