Команда **showDialog** позволяет выводить любой компонент в отдельном диалоговом окне.

Она имеет следующий синтаксис:

```javascript
ODA.showDialog(component, props = {}, hostProps = {})
```

Ей передаются 3 параметра:

1. **component** — определяет компонент, который будет отображаться в диалоговом окне. Например, **'oda-icon'**.
1. **props** — определяет свойства компонента, который будет отображаться в диалоговом окне. Например, **{icon: 'icons:android', iconSize: 50}**.
1. **hostProps** — определяет свойства самого диалогового окна. Например, **{title: 'Диалоговое окно', titleMode: 'auto'}**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=200_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажми на меня</button>
    `,
    onTap() {
        ODA.showDialog('oda-icon', {icon: 'icons:android', iconSize: 50}, {title: 'Диалоговое окно', titleMode: 'auto'});
        }
});
```

Данная команда возвращает промис, исполнитель которого вызывает либо функцию **resolve**, либо функцию **reject** в зависимости от выбора пользователя. Если пользователь нажмет на кнопку «**Ok**», то будет вызвана функция **resolve**. Если будет нажата кнопка «**Cancel**» или диалоговое окно будет закрыто любым другим способом, то будет вызвана функция **reject**. В обработчиках вызова этих функций можно предусмотреть необходимые действия, которые должны выполняться в том или в ином случае.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=200_
import '/components/buttons/icon/icon.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажми на меня</button>
        <div>Вы нажали на кнопку: {{res}}</div>
    `,
    props: {
        res: ''
    },
    onTap() {
        ODA.showDialog('oda-icon', {icon: 'icons:android', iconSize: 50}, {title: 'Диалоговое окно', titleMode: 'auto'}).then(() => this.res = 'Ok', () => this.res = 'Cancel');
    }
});
```

При нажатии на кнопку «**Ok**» срабатывает обработчик функции **resolve**, в качестве параметра которого передается компонент, отображавшийся в диалогом окне. Через этот параметр можно обратиться к нему и узнать какие действия с ним выполнял пользователь.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=250_
import '/components/colors/palette/palette.js';
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">Нажми на меня</button>
        <div>Возвращаемый результат: {{res}}</div>
    `,
    props: {
        res: ''
    },
    onTap() {
        ODA.showDialog('oda-palette', {colors: 5, gradients: 5}, {title: 'Палитра цветов', titleMode: 'auto'}).then(palette => this.res = palette.color, () => this.res =  'Не предусмотрено');
    }
});
```

Если закрыть диалоговое окно любым другим способом, например, нажатием на кнопку «**Cancel**», то будет вызван обработчик функции **reject**. Никакой дополнительный параметр ему не передается. В этом нет никакой необходимости.

Если в качестве отображаемого компонента функции **showDialog** передать уже созданный компонент, то он будет копироваться в диалоговое окно со всеми параметрами, не создаваясь повторно.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=200_
import '/components/buttons/button/button.js';
 ODA({
    is: 'my-component',
    template: `
        <oda-button ref="btn" label="Нажми на меня" icon="icons:favorite" icon-size="50" @tap="onTap" blink="1000"></oda-button>
    `,
    props: {
        value: ''
    },
    onTap() {
        ODA.showDialog(this.$refs.btn, {icon: 'icons:android', iconSize: 50}, {title: 'Диалоговое окно', titleMode: 'auto'});
    }
});
```

В этом случае свойства компонента, передаваемые команде **showDialog** вторым параметром, ему не присваиваются. Считается, что они у него уже заданы.

Например, в предыдущим примере во втором параметре можно указать неопределенное (**undefined**) или пустое (**null**) значение.

```javascript run_line_edit_loadoda_[my-component.js]_h=200_
import '/components/buttons/button/button.js';
 ODA({
    is: 'my-component',
    template: `
        <oda-button ref="btn" label="Нажми на меня" icon="icons:favorite" icon-size="50" @tap="onTap" blink="1000"></oda-button>
    `,
    props: {
        value: ''
    },
    onTap() {
        ODA.showDialog(this.$refs.btn, undefined, {title: 'Диалоговое окно', titleMode: 'auto'});
    }
});
```

В результате этого данный пример будет работать абсолютно так же, как и предыдущий.

У самого диалогового окна существуют следующие свойства:

1. **title** — задает текст надписи в заголовке окна.
1. **titleMode** — задает режим отображения заголовка окна.
    Оно может принимать следующие значения:
    1. **none** — заголовок окна не отображается.
    1. **auto** — заголовок отображается в области с содержимым окна.
    1. **full** — заголовок отображается по ширине всего окна.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=200_
import '/components/buttons/icon/icon.js';
ODA({
    is: 'my-component',
    template: `
        <label> Режим отображения заголовка диалогового окна
            <select ::value>
                <option value="placeholder">Выберите вид отображения</option>
                <option value="none">none</option>
                <option value="auto">auto</option>
                <option value="full">full</option>
            </select>
        </label>
    `,
    props: {
        value: {
            default: 'placeholder',
            set() {
                ODA.showDialog('oda-icon', {icon: 'icons:android', iconSize: 50}, {title: this.value, titleMode: this.value});
            }
        }
    }
});
```

По умолчанию свойство **titleMode** имеет значение **none**. В этом случае строка заголовка не будет отображаться ни у диалогового окна, ни в области, в которой располагается его содержимое.

Если выбрать значение **auto**, то заголовок у окна будет отображаться в области с его содержимым, а при значении **full** — у всего окна в целом.
