**async** — это метод, который позволяет вызвать другой метод или функцию с заданной временной задержкой.

Он объявляется внутри компонента следующим образом:

```javascript
async (handler, delay = 0)
```

Ему передаются 2 параметра:

1. **handler** — вызываемая функция или метод.
1. **delay** — задержка вызова метода или функции в миллисекундах.

Например,

```javascript _run_line_edit_[my-component.js]
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">{{label}}</button>
    `,
    props: {
        label: 'Нажмите меня'
    },
    onTap() {
        this.async(() => {this.label='Я выполняюсь только через секунду'}, 1000);
        this.async(() => {this.label='Нажми на меня'}, 2000);
    }
});
```

В данном примере вызываются стрелочные функции через одну и две секунды соответственно. Указатель **this** у них привязан к контексту текущего компонента **my-component**, так как они объявляются в его методе **onTap**. По этой причине они имеют доступ к свойству компонента **label** через указатель **this**.

При передаче метода в списке параметров необходимо явно привязать к его контексту указатель на компонент с помощью метода **bind**. Если этого не сделать, то указатель **this** внутри метода будет ссылаться уже не на сам компонент, а на глобальный объект **window**.

Например,

```javascript _run_line_edit_[my-component.js]
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">{{label}}</button>
    `,
    props: {
        label: 'Нажмите меня'
    },
    up() {
        this.label = 'Я выполняюсь только через секунду';
    },
    down() {
        this.label = 'Нажми на меня';
    },
    onTap() {
        this.async(this.up.bind(this), 1000);
        this.async(this.down.bind(this), 2000);
    }
});
```

Аналогичная ситуация происходит при вызове анонимной функции, контекст которой зависит от способа её вызова. Метод **async** вызывает её с помощью метода **setTimeout**, внутри которого указатель **this** всегда по умолчанию ссылаться на глобальный объект **window**.

Например,

```javascript _run_line_edit_[my-component.js]
 ODA({
    is: 'my-component',
    template: `
        <button @tap="onTap">{{label}}</button>
    `,
    props: {
        label: 'Нажмите меня',
        count: 0
    },
    onTap() {
        this.async(function() {this.label="Я выполняюсь только через секунду";}.bind(this), 1000);
        this.async(function() {this.label="Нажми на меня";}.bind(this), 2000);
    }
});
```

Таким образом, при использовании метода **async** к не стрелочным функциям необходимо явно привязывать контекст компонента с помощью метода **bind**, так как в противном случае указатель **this** внутри обработчиков будет ссылаться на глобальный объект **window**, а не на сам компонент.

<div style="position:relative;padding-bottom:48%; margin:10px">
    <iframe src="https://www.youtube.com/embed/aPL3cwOJOCs?start=0" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen 
    	style="position:absolute;width:100%;height:100%;"></iframe>
</div>
