**SPA-роутинг** — это механизм, который позволяет организовать навигацию по содержимому одностраничных Web-приложений и сайтов.

Одностраничное Web-приложение (***Single-Page Application***) содержит только одну Web-страницу, содержимое которой формируется динамически в зависимости от действий пользователя. Такое поведение отличает SPA-приложение от обычного многостраничного Web-приложения (***Multi-Page Application***), когда с Web-сервера каждый раз загружаются разные страницы с новыми данными.

В SPA-приложениях вся необходимая информация либо извлекается браузером при загрузке одной страницы, либо соответствующие данные динамически загружаются и добавляются на исходную страницу по мере необходимости в ответ на действия пользователя. Исходная страница в этом случае целиком никогда не перезагружается, и ее URL-адрес не изменяется. В результате этого осуществить навигацию по страницам, используя стандартный механизм навигации браузера, становится невозможно.

Однако можно разработать SPA-приложение так, чтобы оно могло бы использовать стандартный механизм навигации браузера для указания местоположения контента на динамической странице, а также сохранять всю историю обращения к нему. Это позволило бы улучшить восприятие работы SPA-приложения и осуществить более быстрый доступ к необходимой пользователю информации.

ODA-фреймворк позволяет реализовать такую возможность с помощью специального объекта **ODA.router**, который создается на основе класса **odaRouter**. В результате этого можно имитировать работу многостраничного приложения, позволяя пользователю перемещаться по ранее открытым логическим страницам при помощи кнопок «Вперед» и «Назад» в истории переходов браузера, точно так же, как и на обычных сайтах.

Для этого в классе **odaRouter** предусмотрены следующие методы:

1. Конструктор (**constructor**) – в нем изначально обнуляются все правила маршрутизации и назначается слушатель для обработки события **popstate**, которое происходит тогда, когда пользователь в браузере нажимает кнопки истории переходов **«Вперед»** или **«Назад»**, либо когда программно вызываются методы **back()**, **forward()** или **go()** объекта истории сессий браузера **history**.

2. Метод **create** — создает новое правило маршрутизации и задает обработчик перехода к состоянию, которое удовлетворяет данному правилу. Этот метод объявляется следующим образом:

```javascript
create(rule, callback);
```

Ему передаются два параметра:

1. **rule** – правило маршрутизации, например, **#spa**.
1. **callback** – обработчик перехода к состоянию, значение которого удовлетворяет этому правилу.

Механизм SPA-роутинга достаточно простой. Если при переходе в какое-то состояние строка, идущая в URL-адресе страницы после символа хеш (**#**), включая сам этот символ, совпадет или будет удовлетворять одному из правил маршрутизации, указанному в параметре **rule**, то будет срабатывать обработчик данного правила, указанный в параметре **callback**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
ODA({
    is: 'my-component',
    template: `
        <label>Правило маршрутизации <input ::value='rule'> </label>
        <button @tap="onTap">Создать правило и добавить его в историю</button>
        <div>URL-адрес: {{url}}</div>
        <div>Хэш адреса: {{hash}}</div>
    `,
    props: {
        hash: '',
        rule: '#spa',
        url: ''
    },
    onTap() {
        ODA.router.create(this.rule, (hash) => {
            this.hash = hash;
        });
        window.history.pushState({path: this.rule}, '', this.rule);
        this.url = window.location.href;
    }
});
```

В этом примере создается новое правило маршрутизации, записанное в поле ввода, и ему назначается обработчик, которому передается один параметр **hash**. В документе HTML метод **pushState** добавляет новую запись в стек истории сессий браузера через специальный объект **history**. В этом случае меняется текущее состояние страницы, но переход к **URL-адресу**, указанному в третьем параметре метода **pushState**, не происходит. Страница в браузере остается той же самой, хотя ее URL-адрес формально изменяется.

Это происходит из-за того, что метод **pushState** не осуществляет переход к указанному состоянию, а только добавляет его в историю переходов браузера. В результате этого, обработчик, который должен был бы выводить хэш URL-адреса текущей страницы, не срабатывает. Чтобы это произошло необходимо, чтобы пользователь щелкнул по любой из кнопок перехода в истории браузера **«Вперед»** или **«Назад»**, либо сделать это программно, вызвав методы **back()**, **forward()** или **go()** объекта историй сессий **history**. Только в этом случае обработчик созданного состояния сработает автоматически.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
ODA({
    is: 'my-component',
    template: `
        <label>Правило маршрутизации <input ::value='rule'> </label>
        <button @tap="onTap">Создать правило и добавить его в историю</button>
        <button @tap="onPrev">Назад</button>
        <button @tap="onNext">Вперед</button>
        <div>URL-адрес: {{url}}</div>
        <div>Хэш адреса: {{hash}}</div>
    `,
    props: {
        hash: '',
        rule: '#spa/spa1',
        url: ''
    },
    onTap() {
        ODA.router.create(this.rule, (hash) => {
            this.url = window.location.href;
            this.hash = hash;
        });
        window.history.pushState({path: this.rule}, '', this.rule);
        this.url = window.location.href;
    },
    onNext() {
        window.history.forward();
    },
    onPrev() {
        window.history.back();
    }
});
```

Если в этом примере создать два разных состояния, например, **#spa/spa1** и **#spa/spa2**, то нажимая на кнопки **«Вперед»** или **«Назад»**, можно перемещаться по этим состояниям, и при каждом таком перемещение будет срабатывать обработчик, выводя в свойство **url** текущий адрес логической страницы.

Обратите внимание, что методы **back()**, **forward()** или **go()** объекта **history** являются асинхронными. Поэтому в самих обработчиках **onNext** или **onPrev** не стоит проверять состояние самого объекта **location**, оно фактически изменится только после выхода из них.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
ODA({
    is: 'my-component',
    template: `
        <label>Правило маршрутизации <input ::value='rule'> </label>
        <button @tap="onTap">Создать правило и добавить его в историю</button>
        <button @tap="onPrev">Назад</button>
        <button @tap="onNext">Вперед</button>
        <div>URL-адрес: {{url}}</div>
        <div>Новый URL-адреса: {{newurl}}</div>
    `,
    props: {
        newurl: '',
        rule: '#spa/spa1',
        url: ''
    },
    onTap() {
        ODA.router.create(this.rule, () => {
            this.newurl = window.location.href;
        });
        window.history.pushState({path: this.rule}, '', this.rule);
        this.url = window.location.href;
    },
    onNext() {
        window.history.forward();
        this.url = window.location.href;
    },
    onPrev() {
        window.history.back();
        this.url = window.location.href;
    }
});
```

Если в этом примере создать два разных состояния, например, **#spa/spa1** и **#spa/spa2**, то при перемещении по этим состояниям можно заметить, что новый и текущий URL-адреса страницы будут отличаться друг от друга из-за асинхронности работы методов **forward()** и **back()**, так как старый адрес в обработчиках **onNext** или **onPrev** еще не будет изменен.

Следует отметить, что при задании в истории браузера нового состояния, к нему можно присоединить необходимые данные, которые можно будет использовать в обработчике перехода к данному состоянию. Эти данные передаются методу **pushState** первым параметром в виде объекта **state**. Если у этого объекта задать свойство с именем **path**, то значение этого свойства будет передано обработчику данного состояния в качестве параметра. В следующем примере это значение будет доступно в теле обработчика под именем **hash**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
ODA({
    is: 'my-component',
    template: `
        <label>Правило маршрутизации <input ::value='rule'> </label>
        <button @tap="onTap">Создать правило и добавить его в историю</button>
        <button @tap="onPrev">Назад</button>
        <button @tap="onNext">Вперед</button>
        <div>URL-адрес: {{url}}</div>
        <div>Хэш адреса: {{hash}}</div>
    `,
    props: {
        hash: '',
        rule: '#spa/spa1',
        url: ''
    },
    onTap() {
        ODA.router.create(this.rule, (hash) => {
            this.hash = hash;
            this.url = window.location.href;
        });
        window.history.pushState({path: this.rule}, '', this.rule);
        this.url = window.location.href;
    },
    onNext() {
        window.history.forward();
    },
    onPrev() {
        window.history.back();
    }
});
```

Если в этом примере задать два разных состояния, например, **#spa/spa1** и **#spa/spa2**, и передать значения этих состояний в свойстве **path** объекту состояний с помощью метода **pushState**, то в обработчике каждого состояния через параметр **hash** можно узнать, какое правило изначально задавал пользователь.

Обратите внимание, что если объект **state** не будет содержать свойство с именем **path**, то обработчик перехода в соответствующее состояние не будет срабатывать из-за того, что в классе **odaRouter** идет проверка текущего состояния со значением свойства **path** переданного объекта **state**. Если они не совпадают, то назначенный обработчик перехода в это состояние не вызывается.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
ODA({
    is: 'my-component',
    template: `
        <label>Правило маршрутизации <input ::value='rule'> </label>
        <button @tap="onTap">Создать правило и добавить его в историю</button>
        <button @tap="onPrev">Назад</button>
        <button @tap="onNext">Вперед</button>
        <div>URL-адрес: {{url}}</div>
        <div>Хэш адреса: {{hash}}</div>
    `,
    props: {
        hash: '',
        rule: '#spa/spa1',
        url: ''
    },
    onTap() {
        ODA.router.create(this.rule, (hash) => {
            this.hash = hash;
            this.url = window.location.href;
        });
        window.history.pushState({}, '', this.rule);
        this.url = window.location.href;
    },
    onNext() {
        window.history.forward();
    },
    onPrev() {
        window.history.back();
    }
});
```

В данном примере первый параметр метода **pushState** передает пустой объект. В результате этого обработчик перехода в заданное состояние не вызывается.

Для перехода между состояниями можно использовать метод **go()** объекта **history**. Он позволяет перейти на определенное количество состояний вперед или назад в истории переходов браузера. Количество шагов для перехода указывается в списке его параметров.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
ODA({
    is: 'my-component',
    template: `
        <label>Правило маршрутизации <input ::value='rule'> </label>
        <button @tap="onTap">Создать правило и перейти на страницу</button>
        <button @tap="onPrev">Назад</button>
        <button @tap="onNext">Вперед</button>
        <div>URL-адрес: {{url}}</div>
        <div>Хэш адреса: {{hash}}</div>
    `,
    props: {
        hash: '',
        rule: '#spa/spa1',
        url: ''
    },
    onTap() {
        ODA.router.create(this.rule, (hash) => {
            this.hash = hash;
            this.url = window.location.href;
        });
        window.history.pushState({path: this.rule}, '', this.rule);
        this.url = window.location.href;
    },
    onNext() {
        window.history.go(1);
    },
    onPrev() {
        window.history.go(-1);
    }
});
```

В данном примере переход на одну страницу вперед осуществляется вызовом метода **go(1)**, а на одну страницу назад — **go(-1)**. Это эквивалентно вызову методов **forward()** и **back()** соответственно.

Класс **odaRouter** реализует свой собственный метод **go()**, который добавляет в историю новое состояние и одновременно переходит к нему. Это метод объявлен следующим образом:

```javascript
 go(path, idx = 0);
```

 Ему передаются 2 параметра:

 1. **path** – состояние, которое необходимо создать.
 1. **idx** – индекс символа хеша в URL-адресе страницы, начиная с которого необходимо добавить состояние, указанное в параметре **path**.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
ODA({
    is: 'my-component',
    template: `
        <label>Правило маршрутизации <input ::value='rule'> </label>
        <button @tap="onTap">Создать правило и перейти на страницу</button>
        <button @tap="onGo">Перейти</button>
        <div>URL-адрес: {{url}}</div>
        <div>Хэш адреса: {{hash}}</div>
    `,
    props: {
        hash: '',
        rule: '#spa/spa1',
        url: ''
    },
    onTap() {
        ODA.router.create(this.rule, (hash) => {
            this.hash = hash;
            this.url = window.location.href;
        });
        window.history.pushState({path: this.rule}, '', this.rule);
        this.url = window.location.href;
    },
    onGo() {
        ODA.router.go(this.rule)
    }
});
```

В этом случае в историю добавляется новое состояние с помощью метода **pushState** объекта **history** и автоматически вызывается заданный обработчик перехода к нему, как бы моделируя генерацию события **popstate**. В результате это метод **go** одновременно добавляет запись и сразу же переходит к ней.

Кроме этого, класс **odaRouter** реализует свои собственные методы **forward()** и **back()** для перехода на одно состояние вперед или назад соответственно.

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
ODA({
    is: 'my-component',
    template: `
        <label>Правило маршрутизации <input ::value='rule'> </label>
        <button @tap="onTap">Создать правило и перейти на страницу</button>
        <button @tap="onPrev">Назад</button>
        <button @tap="onNext">Вперед</button>
        <div>URL-адрес: {{url}}</div>
        <div>Хэш адреса: {{hash}}</div>
    `,
    props: {
        hash: '',
        rule: '#spa/spa1',
        url: ''
    },
    onTap() {
        ODA.router.create(this.rule, (hash) => {
            this.hash = hash;
            this.url = window.location.href;
        });
        window.history.pushState({path: this.rule}, '', this.rule);
        this.url = window.location.href;
    },
    onNext() {
        ODA.router.forward();
    },
    onPrev() {
        ODA.router.back();
    }
});
```

Фактически эти методы вызывают одноименные методы **forward()** и **back()** объекта **history**.

При задании правила маршрутизации можно использовать следующие подстановки:

1. \* – заменяет любую последовательность символов. Например, #*
1. ? – заменяет любой символ. Например, #?

Используя эти подстановки, можно задать одно правило маршрутизации, обработчик которого будет использоваться при переходе к любому состоянию, удовлетворяющему заданному правилу маршрутизации.

Например,

```javascript run_line_edit_loadoda_[my-component.js]_h=100_
ODA({
    is: 'my-component',
    template: `
        <label>Правило маршрутизации <input ::value='rule'></label>
        <button @tap="onTap">Создать правило</button> <br/>
        <label>Страница для перехода <input ::value='page'></label>
        <button @tap="onGo">Перейти на страницу</button>
        <div>URL-адрес: {{url}}</div>
        <div>Хэш адреса: {{hash}}</div>
    `,
    props: {
        hash: '',
        rule: '#*',
        page: '#spa/spa1',
        url: ''
    },
    onTap() {
        ODA.router.create(this.rule, (hash) => {
            this.hash = hash;
            this.url = window.location.href;
        });
    },
    onGo() {
        ODA.router.go(this.page);
    }
});
```

В этом примере правило **#\*** позволяет обрабатывать переход к любому состоянию, имя которого начинается с символа hash (**#**). Например, если перейти к состоянию **#spa/spa1** или **#spa/spa2**, то для каждого из них будет вызван один и тот же обработчик, так как оба этих состояния удовлетворяют исходному правилу. Узнать к какому состоянию был осуществлен переход в каждом конкретном случае можно через параметр обработчика. В данном примере это параметр имеет имя **hash**.

Использовать технологию SPA-роутинга можно следующим образом:

```javascript run_line_edit_loadoda_[my-component.js]_h=190_
ODA({
    is: 'my-component',
    template: `
        <style>
            #shape {
                width: 100px;
                height: 100px;
                background: bisque;
            }
        </style>
        <div>Фигура</div>
        <p><input name="shape" type="radio" value='circle' @change="onTap">Круг</p>
        <p><input name="shape" type="radio" value='square' @change="onTap">Квадрат</p>
        <div id="shape" ~style="{borderRadius: \`\${radius}%\`}"></div>
        <div>Навигация</div>
        <button @tap="onPrev">Назад</button>
        <button @tap="onNext">Вперед</button>
        <div>URL-адрес: {{window.location.href}}</div>
    `,
    props: {
        radius: 0
    },
    onTap(e) {
        ODA.router.go('#shape/' + e.target.value);
    },
    onNext() {
        ODA.router.forward();
    },
    onPrev() {
        ODA.router.back();
    },
    ready() {
        ODA.router.create('#shape/circle', (hash) => {
            this.radius = '50';
        });
        ODA.router.create('#shape/square', (hash) => {
            this.radius = '0';
        });
    }
});
```

В этом примере в хуке **ready** задаются два правила для обработки перехода к состояниям **#shape/circle** и **#shape/square**. В первом состоянии у фигуры задается 50%-ый радиус скругления рамки, заставляя браузер элемент **div** с идентификатором **shape** прорисоваться в виде круга. Во втором состоянии радиус скругления задается равным 0%. В этом случае тот же самый элемент **div** будет прорисовываться уже как квадрат.

В обработчиках нажатия на кнопки выбора **radio** осуществляется переход в то или иное состояние. При этом срабатывают соответствующие обработчики, заставляя браузер прорисовывать один и тот же элемент **div** по разному.

Наличие разных состояний позволяет осуществлять навигацию по ним с помощью соответствующих кнопок перехода по истории посещений в окне браузера или программно, нажимая на кнопки **«Вперед»** или **«Назад»**, предусмотренные в предыдущим примере.

Кроме этого, текущее состояние можно запомнить и передать другому пользователю, указав явно, какая из фигур была выбрана в том или ином случае.

Предыдущий пример можно реализовать несколько по-другому.

```javascript run_line_edit_loadoda_[my-component.js]_h=200_
ODA({
    is: 'my-component',
    template: `
        <style>
            #shape {
                width: 100px;
                height: 100px;
                background: bisque;
            }
        </style>
        <div>Фигура</div>
        <p><input name="shape" type="radio" value='circle' @change="onTap">Круг</p>
        <p><input name="shape" type="radio" value='square' @change="onTap">Квадрат</p>
        <div id="shape" ~style="{borderRadius: \`\${radius}%\`}"></div>
        <div>Навигация</div>
        <button @tap="onPrev">Назад</button>
        <button @tap="onNext">Вперед</button>
        <div>URL-адрес: {{window.location.href}}</div>
    `,
    props: {
        shape: {
            default: '',
            set(n) {
                if (!n)
                    return;
                const rules = n.split('/');
                switch (rules[1]) {
                    case 'circle':
                        this.radius = 50;
                        break;
                    case 'square':
                        this.radius = 0;
                        break;
                }

            }
        },
        radius: 0
    },
    onTap(e) {
        ODA.router.go('#shape/' + e.target.value);
    },
    onNext() {
        ODA.router.forward();
    },
    onPrev() {
        ODA.router.back();
    },
    ready() {
        ODA.router.create('#*', (hash) => {
            this.shape = hash;
        });
    }
});
```

В этом примере создается одно правило, которое позволяет обрабатывать любое состояние. Значение текущего состояния передается свойству компонента **shape**, в сеттере которого и предусмотрена вся необходимая для работы SPA-приложения маршрутизация.
