import '../../components/grids/tree/tree.js';
const ODA_ICON_PATH = '/oda/buttons/icon/icon.js';
let iconsPath = import.meta.url;
iconsPath = iconsPath.slice(0, iconsPath.lastIndexOf('/')) + '/../../icons/svg/';
const pars = new DOMParser();
class IconItem {
    constructor(el, group) {
        this.name = el.getAttribute('id');
        this.label = this.name;
        this.icon = group ? `${group}:${this.name}` : '';
    }
}
class IconFolder {
    constructor(template) {
        this.name = template.getAttribute('name');
        this.label = this.name
        // this.icon = `${this.name}:${template.getAttribute('icon')}`;
        this.icon = 'enterprise:folder';
        this.subIcon = `${this.name}:${template.getAttribute('icon')}`
        const content = template.content;
        this.items = Array.prototype.map.call(content.children, g => new IconItem(g, this.name));
    }
}
async function loadIcons(items) {
    return (await Promise.all(items.map(async i => {
        let file = await fetch(iconsPath + i.name);
        file = await file.text();
        file = pars.parseFromString(file, 'text/html');
        const template = file.querySelector('template');
        return new IconFolder(template);
    })));
}
ODA({
    is: 'oda-icons-tree', extends: 'oda-tree',
    props: {
        // allowFocus: true,
        lazy: true,
        allowDrag: true,
        allowSort: true,
        iconsList: {
            type: Array,
            async set(v) {
                this.dataSet = await loadIcons(v);
            }
        },
        columns: [{ name: 'label', treeMode: true, $sort: 1 }],
        filterVal: {
            default: '',
            watch(filter) {
                this.columns[0].$filter = filter === '*' ? '' : filter;
                this.hideRoot = Boolean(filter);
                this._refresh(); //todo: убрать
            }
        },
        defaultTemplate: 'oda-icons-icon',
        rootPath: '/'
    },
    async _onDragStart(e) {
        const el = e.path.find(p => p.row);
        if (el.row.subIcon) {
            e.preventDefault();
            e.stopPropagation();
            return;
        }
        if (el && (this.allowDrag || el.row.drag)) {
            try {
                const node = e.target.querySelector('oda-table-cell-tree').__shadowRoot.querySelector('oda-icons-icon');
                e.dataTransfer.setDragImage(node, 0, 0);
            } catch (err) {
                e.dataTransfer.setDragImage(new Image(), 0, 0);
            }
            e.dataTransfer.setData('text/plain', `\r\n<oda-icon icon="${el.row.icon}"></oda-icon>`);
            e.dataTransfer.setData('odant/html-links', JSON.stringify([`<script type="module" href="${this.rootPath}${ODA_ICON_PATH}">`]));
            e.dataTransfer.setData('odant/icon', el.row.icon);
        }
    },
    // async _beforeExpand(item) {
    //     return item.items;
    // },
    async ready() {
        this.iconsList = await (await fetch(iconsPath + 'info.json')).json();
        this.sorts = [this.columns[0]]; //todo: костыль, убрать
    }
});

ODA({
    is: 'oda-icons-icon',
    extends: 'oda-icon, oda-table-cell-base',
    template: /*html*/`
    <style>
        :host(:not(:hover)) > oda-button{
            display: none;
        }
        oda-button{
            opacity: 0.25;
        }
        oda-button:hover{
            opacity: 1;
        }
    </style>
    <oda-icon ~if="item?.subIcon" style="position: absolute; transform: translate(0px, 2px) scale(0.5);" :icon="item.subIcon || ''"></oda-icon>
    <label class="label flex" style="cursor: pointer;">{{item.label}}</label>
    <oda-button ~if="!item?.subIcon" icon="icons:content-copy" @tap="_copyToClipboard"></oda-button>`,
    props: {
        icon: {
            get() {
                return this.item.icon;
            }
        },
        fill(){
            return this.item?.subIcon ? 'orange' : '';
        }
    },
    _copyToClipboard() {
        const s = this.icon;
        const input = document.createElement('input');
        input.style.opacity = '0';
        input.style.display = 'fixed';
        document.body.appendChild(input);
        input.value = s;
        input.focus();
        input.select();
        document.execCommand("Copy");
        input.remove();
    }
});