import '../../components/grids/table/table.js';
import '../../components/inputs/basic/basic.js';

class InspectorItem{
    $level;
    $parent;
    constructor({name, obj, proto, desc = null}){
        this.name = name;
        this.label = name;
        this._io = obj;
        this.desc = desc || Object.getOwnPropertyDescriptor(proto, name);
        this.type = this.desc?.type?.name || (this.desc?.value || this.desc?.default)?.constructor.name || 'String';
        this.editor = this.desc?.editor;
        this.category = proto?.$core?.prototype?.is || proto?.constructor?.name;
        const val = this.value;
        this._isObject = val && typeof val === 'object';
        this._isArray = Array.isArray(val);
    }
    get value(){
        return this._io[this.name];
    }
    set value(v) {
        this._io[this.name] = v;
    }
    get isObject(){ return this._isObject; }
    get isArray(){ return Array.isArray(this.value); }
    _items = [];
    get items(){
        if (this._isObject){
            this._items = this._items.length ? this._items : extractProps(this.value, true);
            return this._items;
        }else{
            return this._items;
        }
    }
    set items(v){
        this._items = v;
    }
    get $expanded(){
        return this._expanded;
    }
    set $expanded(v){
        this._expanded = v;
        this._items = [];
    }
}

function isReadOnly(item) {
    return item.readOnly || item.disabled || !item.desc || !item.desc.set;
}
function isValid(name) {
    if (!name) return false;
    return !(/^((o_)|_|(outerHTML)|(innerHTML)|(properties)|(props))/.test(name));
}
/**
 * @param {Object} proto 
 * @param {Object} obj 
 * @param {Boolean=} expert 
 */
function getProps(proto, obj,  expert) {
    const props = [];
    if (expert) {
        Object.getOwnPropertyNames(proto).forEach(name => {
            if (isValid(name)) {
                let prop = new InspectorItem({name, obj, proto});
                if (prop.type.toLowerCase() !== 'function') props.push(prop);
            }
        });
    } else if (obj.props && typeof obj.props === 'object') {
        if (proto.props && typeof proto.props === 'object') {
            Object.getOwnPropertyNames(proto.props).forEach(name => {
                if (isValid(name)) {
                    let prop = new InspectorItem({ name, obj, proto, desc: proto.props[name] });
                    if (prop.type.toLowerCase() !== 'function') props.push(prop);
                }
            });
        }
    } else {
        return getProps(proto, obj, true);
    }
    return props;
}
function extractProps(obj, expert) {
    if (!obj || typeof obj !== 'object') return [];
    let proto = obj;
    const props = [];
    while (proto) {
        props.push(...getProps(proto, obj, expert));
        proto = Object.getPrototypeOf(proto);
    }
    return props;
}
ODA({
    is: 'oda-inspector',
    extends: 'oda-table',
    template: /*html*/``,
    props: {
        io: {
            type: Object,
            set(n){
                this._inspect(n, this.expert, this.editMode);
            }
        },
        expert: false,
        editMode: true,
        lazy: true,
        autoWidth: true,
        allowFocus: true,
        showHeader: true,
        colLines: true,
        rowLines: true,
        allowSort: true,
        columns: [
            { name: 'label',/*  template: 'oda-inspector-label', */ header: 'oda-inspector-label-header', treeMode: true, fix: 'left', $sort: 1 },
            { name: 'value', template: 'oda-inspector-value', header: 'oda-inspector-value-header', width: 'auto' },
            { name: 'category', hidden: true }
        ],
    },
    listeners: {},
    keyBindings: {},
    observers: ['_inspect(io, expert, editMode)'],
    _inspect(io, expert, editMode) {
        this.dataSet = extractProps(io, expert);
        console.log(this.dataSet);
        this._refresh();
    },
    _getItems(item) {

    },
    ready() {
        if (this.columns && (!this.groups || this.groups.length === 0)) {
            this.groups.push(this.columns.find(c => c.name === 'category'));
        }
    }

});
ODA({
    is: 'oda-inspector-label',
    extends: 'oda-table-cell-base',
    template: /*html*/`
    <div ~text="item?.label"></div>
    `,
    props: {

    },
    listeners: {},
    keyBindings: {},
    observers: [],

});
ODA({
    is: 'oda-inspector-value',
    extends: 'oda-table-cell-base',
    template: /*html*/`
    <div ~if="!item.editor" class="horizontal flex">
        <div ~if="item?.isObject" class="horizontal">
            <div ~text="\`\${item.value.constructor.name}\`"></div><div ~if="item.isArray" ~text="\`[\${item.value.length}]\`"></div>
        </div>
        <div ~if="!item?.isObject && !item?.isArray" class="flex">
            <oda-basic-input class="flex" :type="item.type" :value="item.value" @value-changed="_changed" ::focused></oda-basic-input>
        </div>
    </div>
    <div ~if="item.editor" class="horizontal flex" is="item.editor">
    `,
    props: {
        focused: false
    },
    listeners: {},
    keyBindings: {},
    observers: [],
    _changed(evt){
        if (this.focused){
            this.item.value = evt.detail.value;
        }
    }

});
ODA({
    is: 'oda-inspector-label-header',
    extends: 'oda-table-cell-base',
    template: /*html*/`
    <div style="box-shadow: inset 0 0 0 1px gray; display: flex; align-items: center;">
            <oda-basic-input ::value></oda-basic-input>
            <oda-icon icon="icons:search" icon-size="32"></oda-icon>
        </div>
    `,
    props: {
        value: {
            set(v) {
                this.table.columns[0].$filter = v;
            }
        }
    },
    listeners: {},
    keyBindings: {},
    observers: [], 

});
ODA({
    is: 'oda-inspector-value-header',
    extends: 'oda-table-cell-base',
    template: /*html*/`
    <div class="flex"></div>
    <oda-button icon="social:school" allow-toggle :toggled="table?.expert" @tap="table.expert = !table.expert"></oda-button>
    `,
    props: {
        value: {
            set(v) {
                this.table.columns[0].$filter = v;
            }
        }
    },
    listeners: {},
    keyBindings: {},
    observers: [],

});